function X = litterman(rho,sigma,lambda,varargin)
% LITTERMAN  Litterman's prior dummy observations for BVARs.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

g = 0;
if length(varargin) == 1
   % BVAR.litterman(rho,sigma,lambda,[ny,p]).
   % BVAR.litterman(rho,sigma,lambda,[ny,p,g]).
   ny = varargin{1}(1);
   p = varargin{1}(2);
   if length(varargin{1}) > 2
      g = varargin{1}(3);
   else
      g = 0;
   end
else
   % Bkw compatibility.
   % BVAR.litterman(rho,sigma,lambda,ny,p).
   ny = varargin{1};
   p = varargin{2};
   if length(varargin) > 2
      g = varargin{3};
   else
      g = 0;
   end
end

P = inputParser();
P.addRequired('rho',@(x) isnumeric(x) && (numel(x) == 1 || numel(x) == ny) && all(x >= 0 & x <= 1));
P.addRequired('sigma',@(x) isnumeric(x) && (numel(x) == 1 || numel(x) == ny) && all(x >= 0));
P.addRequired('lambda',@(x) isnumericscalar(x) && x >= 0);
P.addRequired('ny',@isnumericscalar);
P.addRequired('p',@isnumericscalar);
P.addRequired('g',@isnumericscalar);
P.parse(rho,sigma,lambda,ny,p,g);

%**************************************************************************
% Function body.

rho = rho(:);
sigma = sigma(:);
nsigma = numel(sigma);
if nsigma == 1 && ny > 1
   sigma = sigma(ones([1,ny]),1);
elseif nsigma ~= ny
   error('iris:bvar', ...
      'Number of weights in |sigma| must be scalar or match number of variables |ny|.');
end

nd = ny*p;
k = zeros([1,nd]);
g1 = zeros([g,nd]);
y0 = [diag(sigma.*rho),zeros([ny,nd-ny])];
sigma = sigma(:,ones([1,p]));
if lambda > 0
   lags = (1 : p).^lambda;
   lags = lags(ones([1,ny]),:);
   sigma = sigma .* lags;
end
y1 = diag(sigma(:));  
X = [y0;k;y1;g1];

end
% End of primary function.