function doc = docsubs(doc,options)
% DOCSUBS  Document substitutions in LaTeX code.

nl = sprintf('\n');

try
   doc = strrep(doc,'!Title',options.title);
   temptitle = options.title;
   temptitle = strrep(temptitle,'\\',' ');
   doc = strrep(doc,'!HeaderTitle',temptitle);
catch
   doc = strrep(doc,'!Title','');
end

try
   doc = strrep(doc,'!Author',options.author);
catch
   doc = strrep(doc,'!Author','');
end

try
   doc = strrep(doc,'!Date',options.date);
catch
   doc = strrep(doc,'!Date','');   
end

try
   doc = strrep(doc,'!PaperSize',options.papersize);
catch
   doc = strrep(doc,'!PaperSize','');   
end

try
   doc = strrep(doc,'!Orientation',options.orientation);
catch
   doc = strrep(doc,'!Orientation','');    
end

try
   doc = strrep(doc,'!HeaderTimeStamp',options.timestamp);
catch
   doc = strrep(doc,'!HeaderTimeStamp','');
end

try
   doc = strrep(doc,'!Scale',sprintf('%g',options.scale));
catch
   doc = strrep(doc,'!Scale','0.8');
end

try
   doc = strrep(doc,'!Preamble',options.preamble);
catch
   doc = strrep(doc,'!LaTeXPackages','');
end

try
   if options.numbered
      doc = strrep(doc,'!NumberedSection','[\theTocEntry]');
      doc = strrep(doc,'!NumberedSubsection','');
   else
      doc = strrep(doc,'!NumberedSection','*');
      doc = strrep(doc,'!NumberedSubsection','*');
   end
catch
   doc = strrep(doc,'!NumberedSection','*');
   doc = strrep(doc,'!NumberedSubsection','*');
end

try
   c = sprintf('%g,%g,%g',options.highlightcolor);
   doc = strrep(doc,'!HighlightColor',c);
catch
   doc = strrep(doc,'!HighlightColor','0.9,0.9,0.9');
end

try
   if options.maketitle
      repl = '\maketitle\thispagestyle{empty}';
   else
      repl = '';
   end
catch
   repl = '';
end   
doc = strrep(doc,'!MakeTitle',repl);

try
   if ~isempty(options.abstract) && options.maketitle
      repl = ['\begin{abstract}',nl,...
         '\input{',options.abstract,'}',nl,...
         '\end{abstract}'];
   else
      repl = '';
   end
catch
   repl = '';
end
doc = strrep(doc,'!Abstract',repl);

try
   if options.maketitle
      repl = '\clearpage';
   else
      repl = '';
   end
catch
   repl = '';
end
doc = strrep(doc,'!ClearFirstPage',repl);

try
   if options.toc
      repl = '\thispagestyle{empty}\tableofcontents\clearpage';
   else
      repl = '';
   end
catch
   repl = '';
end
doc = strrep(doc,'!TOC',repl);

end