classdef bandobj < report.seriesobj

   properties
      low = [];
      high = [];
   end
   
   methods

      function this = bandobj(varargin)
         this = this@report.seriesobj(varargin{:});
         this.default = [this.default,{...
            'bandformat','\footnotesize',@ischar,true,...
            'low','Low',@ischar,true,...
            'high','High',@ischar,true,...
            'position','relative',@(x) ischar(x) && any(strcmpi(x,{'relative','absolute'})),true, ...
            'type','patch',@(x) ischar(x) && any(strcmpi(x,{'line','patch'})),true,...
            'white',0.80,@(x) isnumericscalar(x) && x >= 0 && x <= 1,true,...
            }];
      end
      
      function [this,varargin] = specargin(this,varargin)
         [this,varargin] = specargin@report.seriesobj(this,varargin{:});
         if ~isempty(varargin)
            this.low = varargin{1};
            varargin(1) = [];
         end
         if ~isempty(varargin)
            this.high = varargin{1};
            varargin(1) = [];
         end       
      end
      
      function c = latexcode(this,parent)
         [centre,time] = this.series(parent.range);
         low = this.low(time);
         high = this.high(time);
         centre = centre(:,:);
         low = low(:,:);
         high = high(:,:);
         c = '';
         for i = 1 : this.nData
            c = [c,gettablerowstring(this,time,centre,low,high,i)];
         end
      end
      
      function c = gettablerowstring(this,time,centre,low,high,i)
         nl = sprintf('\n');
         c = gettablerowstring@report.seriesobj(this,time,centre(:,i),i);
         lowstring = latex.stringsubs(this.options.low);
         highstring = latex.stringsubs(this.options.high);
         c = [c,nl,...
            '& & {',this.options.bandformat,'{',lowstring,'}}', ...
            getdatastring(this,time,centre(:,i)+low(:,i),this.options.bandformat),'\\',nl,...
            '& & {',this.options.bandformat,'{',highstring,'}}', ...
            getdatastring(this,time,centre(:,i)+high(:,i),this.options.bandformat),'\\'];
      end
      
      function [hfrg,hbkg,lgd] = plot(this,parent,ax)
         % Create the line plot first using the parent's method.
         [hfrg,hbkg,lgd,time,centre,grid] = plot@report.seriesobj(this,parent,ax);
         grid = grid(:);
         low = this.low(time);
         high = this.high(time);
         low = low(:,:);
         high = high(:,:);
         ndata = size(centre,2);
         nextplot = get(ax,'nextPlot');
         set(ax,'nextPlot','add');
         edgeData = [low;high(end:-1:1,:)];
         if strcmpi(this.options.position,'relative')
            edgeData = edgeData + [centre;centre(end:-1:1,:)];
         end
         for i = 1 : ndata
            h = patch([grid;grid(end:-1:1)],edgeData(:,i),'white');
            hbkg(end+1) = h;
            colour = get(hfrg(i),'color');
            faceColour = (1-this.options.white)*colour+this.options.white*[1,1,1];
            set(h,'faceColor',faceColour,'edgeColor',colour,'lineStyle','none');
            excludefromlegend(h);
         end
         set(ax,'nextPlot',nextplot);
      end
      
   end   

end