classdef genericobj
% GENERICOBJ  Definition file for generic report class of objects.

   properties
      children = {};
      accept = {};
      default = {};
      caption = '';
      options = struct();
   end
      
   properties (Dependent = true)   
      shortClassName
      nChildren
   end
   
   methods

      function this = genericobj(varargin)
         if ~isempty(varargin)
            this.caption = varargin{1};
         end
      end
      
      function [this,varargin] = specargin(this,varargin)
      end
      
      function this = spechandleopt(this)
      end
     
      function this = setoptions(this,default,parentoptions,varargin)
         try
            useroptions = cell2struct(varargin(2:2:end),lower(varargin(1:2:end)),2);
         catch Error
            error('iris:report',...
               'Invalid structure of optional input arguments.\nMatlat says: %s',...
               Error.message);
         end
         % Make sure all options are tunnelled.
         this.options = parentoptions;
         for i = 1 : 2 : length(varargin)
            this.options.(varargin{i}) = varargin{i+1};
         end
         for i = 1 : 4 : length(default)
            % First, assign default.
            this.options.(default{i}) = default{i+1};
            % Then, inherit if it is inheritable and is available from parent.
            if default{i+3} && isfield(parentoptions,default{i})
               this.options.(default{i}) = parentoptions.(default{i});
            end
            % Last, get it from user if supplied.
            invalid = {};
            if isfield(useroptions,default{i})
               % Validate user option.
               if default{i+2}(useroptions.(default{i}))
                  this.options.(default{i}) = useroptions.(default{i});
               else
                  invalid{end+1} = default{i};
                  invalid{end+1} = func2str(default{i+2});
               end
               % Report values that do not pass validation.
               if ~isempty(invalid)
                  error('iris:report',...
                     'Value assigned to option ''%s'' does not pass validation test ''%s''.',...
                     invalid{:});
               end   
            end
         end
         this = spechandleopt(this);
      end
     
      function this = add(this,child,varargin)
         accepted = any(strcmp(this.accept,child.shortClassName));
         if accepted
            [child,varargin] = specargin(child,varargin{:});            
            child = setoptions(child,child.default,this.options,varargin{:});
            this.children{end+1} = child;
         elseif ~isempty(this.children)
            this.children{end} = add(this.children{end},child,varargin{:});
         else
            error('iris:mhtreport',...
               'Cannot add %s to %s.',...
               upper(child.shortClassName),upper(this.shortClassName));
         end
      end
     
      function shortClassName = get.shortClassName(this)
         shortClassName = regexp(class(this)...
            ,'(?<=.*?\.).*?(?=obj)','match','once');
      end
      
      function nChildren = get.nChildren(this)
         nChildren = numel(this.children);
      end
     
      function disp(this,level)
         if nargin == 1
            level = 0;
         end
         tab = sprintf('\t');
         fprintf('%s',tab(ones([1,1+level])));
         if level > 0
            fprintf('+');
         end
         fprintf('<a href="">%s</a>',this.shortClassName);
         if ~isempty(this.caption)
            fprintf(' ''%s''',this.caption);
         end
         fprintf('\n');
         for i = 1 : length(this.children)
            disp(this.children{i},level+1);
         end
         if level == 0
            loosespace();
         end
      end
      
      function display(this)
         loosespace();
         disp([inputname(1),' =']);
         loosespace();
         disp(this);
      end
        
   end
   
   methods (Static = true)
      
      function c = makeBox(text,format,colWidth,color)
          c = ['{',text,'}'];
          if ~isempty(format)
             c = ['{',format,c,'}'];
          end
          if ~isnan(colWidth)
            c = ['\makebox[',sprintf('%g',colWidth),'em]{',c,'}'];
          end
          if ~isempty(color)
             c = ['\colorbox{',color,'}{',c,'}'];
          end
      end
      
      function c = sprintfData(data,options)
         c = sprintf(options.format,data);
         c = strrep(c,'-','--');
         c = strrep(c,'NaN',options.nan);
         c = strrep(c,'Inf',options.inf);
      end
      
      function c = tabularSpec(nLeading,nColumns,vertLines)
         c = repmat('l',[1,nLeading]);
         if nColumns == 0
            return
         end
         % Pre-sample vertical line.
         if any(vertLines == 0)
            c = [c,'|'];
         end
         vertLines(vertLines < 1 | vertLines > nColumns) = [];
         c1 = repmat('r',[1,nColumns]);
         c2 = repmat(' ',[1,nColumns]);
         c2(vertLines) = '|';
         c3 = [c1;c2];
         c3 = transpose(c3(:));
         c = [c,c3];
         c(c == char(32)) = '';
      end
      
      function c = tabularCaption(caption,nColumns)
         c = '';
         if isempty(caption)
            return
         end
         c = [c,'\multicolumn{',sprintf('%g',nColumns),'}',...
            '{c}{\large\bfseries{',latex.stringsubs(caption),'}}',...
            '\\[5pt]'];  
      end
      
   end
   
end