classdef graphobj < report.genericobj
    
   properties (Dependent = true)
      range
   end
   
   methods

      function this = graphobj(varargin)
         this = this@report.genericobj(varargin{:});
         this.accept = {'series','band'};
         this.default = {...
            'highlight',[],@(x) isnumeric(x) || iscell(x),true, ...
            'legend',false,@(x) islogical(x) || isnumeric(x),true, ...
            'legendlocation','Best',@ischar,true, ...
            'range',Inf,@isnumeric,true, ...
            'style',struct(),@isstruct,true, ...
            'tight',true,@islogical,true, ...
            'xlabel','',@ischar,true, ...
            'ylabel','',@ischar,true, ...
            'zlabel','',@ischar,true, ...
            'zeroline',false,@islogical,true, ...
            'dateformat',irisget('plotdateformat'),@ischar,true, ...
            'datetick',Inf,@isnumeric,true, ...
         };
      end
      
      function [this,varargin] = specargin(this,varargin)
      end

      % Plot this graph.
      function h = plot(this,ax)
         if this.nChildren == 0
            h = [];
            return
         end
         cla(ax);
         hfrg = []; % Children that will be moved to foreground.
         hbkg = []; % Children that will be moved to background.
         lgdentries = {}; % Legend entries.
         for i = 1 : this.nChildren
            [tmphfrg,tmphbkg,lgdentries{end+1}] = plot(this.children{i},this,ax);
            hfrg = [hfrg,tmphfrg(:)'];
            hbkg = [hbkg,tmphbkg(:)'];
            if i == 1
               % The following two lines emulate |hold('all')|.
               set(ax,'nextPlot','add');
               setappdata(ax,'PlotHoldStyle',true);
            end   
         end          
         if this.options.zeroline
            hbkg = [hbkg,zeroline(ax)];
         end         
         h = [hbkg(:);hfrg(:)];
         set(ax,'children',h(end:-1:1));         
         if this.options.tight
            axis('tight');
         end
         lg = [];
         if (islogical(this.options.legend) && this.options.legend) ...
               || (isnumeric(this.options.legend) && ~isempty(this.options.legend))
            if isnumeric(this.options.legend) && ~isempty(this.options.legend)
               lgdentries = lgdentries(this.options.legend);
            end
            lgdentries = [lgdentries{:}];
            lg = legend(ax,lgdentries{:},'location',this.options.legendlocation);
         end
         if ~isempty(this.caption)
            if ~isempty(lg) && strcmpi(get(lg,'Location'),'NorthOutside')
               ti = get(lg,'title');
               set(ti,'string',this.caption,'interpreter','none','visible','on');
            else
               ti = title(this.caption,'interpreter','none');
            end
         end         
         if ~isempty(this.options.xlabel)
            xlabel(ax,this.options.xlabel);
         end
         if ~isempty(this.options.ylabel)
            xlabel(ax,this.options.ylabel);
         end
         if ~isempty(this.options.zlabel)
            xlabel(ax,this.options.zlabel);
         end
         if ~isempty(this.options.highlight)
            highlight(this.options.highlight);
         end
         qstyle(this.options.style,ax,'warning',false);
      end
      
      function range = get.range(this)
         range = this.options.range;
      end
      
   end

end