classdef matrixobj < report.genericobj

   properties
      data = [];
   end
   
   properties (Dependent = true)
      nColumns
      nRows
      nLeading
   end
   
   methods

      function this = matrixobj(varargin)
         this = this@report.genericobj(varargin{:});
         this.accept = {};
         this.default = {...
            'colnames',{},@iscellstr,true,...
            'rownames',{},@iscellstr,true,...
            'separator','\medskip\par',@ischar,true,...
            'format','%.2f',@ischar,true,...
            'colwidth',NaN,@isnumericscalar,true...
            'nan','$\cdots$',@ischar,true,...
            'inf','$\infty$',@ischar,true,...
            'heading','',@ischar,true,...
         };
      end
      
      function [this,varargin] = specargin(this,varargin)
         if ~isempty(varargin)
            this.data = varargin{1};
            varargin(1) = [];
         end
      end
      
      function this = spechandleopt(this)
         % Make sure numel of rownames equals nRows,
         % and numel of colnames equals nColumns.
         if numel(this.options.rownames) < this.nRows
            this.options.rownames(end+1:this.nRows) = {''};
         else
            this.options.rownames = this.options.rownames(1:this.nRows);
         end
         if numel(this.options.colnames) < this.nColumns
            this.options.colnames(end+1:this.nColumns) = {''};
         else
            this.options.colnames = this.options.colnames(1:this.nColumns);
         end         
      end      

      function [c,helperfiles] = latexcode(this,parent)
         helperfiles = {};
         nl = sprintf('\n');
         % Start of tabular and tabular spec.
         c = ['\begin{tabular}{',...
            report.matrixobj.tabularSpec(this.nLeading,this.nColumns,[]),...
            '}'];
         % Caption centred and stretched over all columns.
         if ~isempty(this.caption)
            c = [c,nl,report.matrixobj.tabularCaption(...
               this.caption,this.nLeading+this.nColumns)];
         end
         c = [c,nl,'\hline',nl];
         % User-supplied heading.
         if ~isempty(this.options.heading)
            c = [c,nl,this.options.heading];
         end
         % Column names rotated.
         if anycolname(this)
            c1 = '';
            for i = 1 : this.nColumns
               if i > 1
                  c1 = [c1,' & '];
               end
               c2 = latex.stringsubs(this.options.colnames{i});
               % Add a space when measuring the width of the string.
               c1 = [c1,'\settowidth{\tableColNameHeight}{',c2,' }',...
                  '\rule{0pt}{\tableColNameHeight}',...
                  '\turnbox{90}{',c2,'}'];
            end
            if this.nLeading == 1
               c1 = [' & ',c1];
            end
            c = [c,c1,' \\',nl,'\hline',nl];
         end
         % Cycle over the matrix rows.
         for i = 1 : this.nRows
            if this.nLeading == 1
               c = [c,nl,'{',...
                  latex.stringsubs(this.options.rownames{i}),...
                  '}'];
               if this.nColumns > 0
                  c = [c,' & '];
               end
            end
            % Cycle over the matrix columns.
            for j = 1 : this.nColumns
               c1 = report.matrixobj.sprintfData(...
                  this.data(i,j),this.options);
               c1 = report.matrixobj.makeBox(...
                  c1,'',this.options.colwidth,'');
               if j > 1
                  c1 = [' & ',c1];
               end
               c = [c,c1];
            end
            c = [c,' \\'];
         end
         c = [c,nl,'\hline',nl,'\end{tabular}'];
      end
      
      function nColumns = get.nColumns(this)
         nColumns = size(this.data,2);
      end

      function nRows = get.nRows(this)
         nRows = size(this.data,1);
      end
      
      
      function nLeading = get.nLeading(this)
         if anyrowname(this)
            nLeading = 1;
         else
            nLeading = 0;
         end
      end
      
      function flag = anyrowname(this)
         flag = ~isempty(this.options.rownames) ...
            && any(~cellfun(@isempty,this.options.rownames));
      end
      
      function flag = anycolname(this)
         flag = ~isempty(this.options.colnames) ...
            && any(~cellfun(@isempty,this.options.colnames));
      end
      
   end

end