function x = new(varargin)

% Persistent variable this is the root report object.
persistent this;

% Create new reportobj and process input arguments.
this = report.reportobj(varargin{:});
[this,varargin] = specargin(this,varargin{2:end});
this = setoptions(this,this.default,struct(),varargin{:});

% Interface functions.
x.display = @display;
x.getreportobj = @getreportobj;
x.publish = @publish;

% Level 1 objects.
x.section = @section;
x.table = @table;
x.figure = @figure;
x.matrix = @matrix;
x.text = @text;
x.clearpage = @clearpage;
x.pagebreak = @pagebreak;
x.include = @include;

% Level 2 and 3 objects.
x.graph = @graph;
x.subheading = @subheading;
x.series = @series;
x.band = @band;

   function varargout = getreportobj(varargin)
      varargout{1} = this;
   end

   % Level 1 objects.
   
   function section(varargin)
      newobj = report.sectionobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function table(varargin)
      newobj = report.tableobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end
   
   function figure(varargin)
      newobj = report.figureobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function matrix(varargin)
      newobj = report.matrixobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end
   
   function text(varargin)
      newobj = report.textobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function include(varargin)
      newobj = report.includeobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function clearpage(varargin)
      newobj = report.clearpageobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function pagebreak(varargin)
      newobj = report.clearpageobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   % Level 2 and 3 objects.

   function graph(varargin)
      newobj = report.graphobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function series(varargin)
      newobj = report.seriesobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function band(varargin)
      newobj = report.bandobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function subheading(varargin)
      newobj = report.subheadingobj(varargin{:});
      this = add(this,newobj,varargin{2:end});
   end

   function display()
      this.display();
   end

   function publish(outputfile,varargin)
      default = { ...
         'abstract','',@ischar, ...
         'author','',@ischar, ...
         'date','\today',@ischar, ...
         'deletelatex',true,@islogical, ...
         'papersize','letterpaper',@(x) any(strcmpi(x,{'a4paper','letterpaper'})), ...
         'orientation','landscape',@(x) any(strcmpi(x,{'landscape','portrait'})), ...
         'preamble','',@ischar, ...
         'title',this.caption,@ischar, ...
         'maketitle',false,@islogical, ...
         'timestamp',datestr(now()),@ischar, ...
         'scale',0.8,@isnumericscalar, ...
         'highlightcolor',[1,0.8,0.8], ...
            @(x) isnumeric(x) && numel(x) == 3 && all(x >= 0 & x <= 1), ...
      };
      % The following options will be tunnelled to latex.compilepdf:
      % 'display', 'rerun'
      [options,varargin] = passvalopt(default,varargin{:});
      %if strcmpi(options.preamble,'default')
      %   options.preamble = fullfile( ...
      %      irisget('irisroot'),'-Publish','reportLatexPreamble.tex');
      %end
      thisDir = fileparts(mfilename('fullpath'));
      templateFile = fullfile(thisDir,'reportTemplate.tex');
      [reportCode,helperFiles] = this.latexcode();
      c = file2char(templateFile);
      c = strrep(c,'!Document',reportCode);
      c = latex.docsubs(c,options);
      latexfile = [tempname(cd()),'.tex'];
      char2file(c,latexfile);
      [outputpath,outputtitle,outputext] = fileparts(outputfile);
      if isempty(outputext)
         outputfile = fullfile(outputpath,[outputtitle,'.pdf']);
      end
      % Use try-catch to make sure the helper files are deleted at the end.
      try
         % latex.findeps2pdf(latexfile);
         pdffile = latex.compilepdf(latexfile,varargin{:});
         movefile(pdffile,outputfile);
      catch Error
         warning('iris:report', ...
            'Error when compiling LaTeX and/or PDF files.\nMatlab says: %s', ...
            Error.message);
      end
      if options.deletelatex
         % Delete all helper files produced when TeX files was compiled.
         [latexpath,latextitle] = fileparts(latexfile);
         delete(fullfile(latexpath,[latextitle,'.*']));
         % Delete all helper files produced when latex codes for children
         % were built.
         for i = 1 : length(helperFiles)
            delete(helperFiles{i});
         end
      end
   end
   
end