classdef seriesobj < report.genericobj

   properties
      series = [];
   end
   
   properties (Dependent = true)
      nData
   end
   
   methods

      function this = seriesobj(varargin)
         this = this@report.genericobj(varargin{:});
         this.accept = {};
         this.default = {...
            'decimal',NaN,@isnumericscalar,true, ...
            'units','',@ischar,false, ...
            'format','%.2f',@ischar,true, ...
            'nan','$\cdots$',@ischar,true, ...
            'inf','$\infty$',@ischar,true, ...
            'plotfunc',@plot,@isfunc,true, ...  
            'marks',{},@(x) isempty(x) || iscellstr(x),true, ...
            'highlight',[],@(x) isnumeric(x) || isfunc(x),true, ...
         };         
      end
      
      function [this,varargin] = specargin(this,varargin)
         if ~isempty(varargin)
            this.series = varargin{1};
            varargin(1) = [];
         end
      end
      
      function this = setoptions(this,default,parentoptions,varargin)
         this = setoptions@report.genericobj(this,default,parentoptions,varargin{:});
         this.options.marks = this.options.marks(:)';
         nMarks = numel(this.options.marks);
         if this.nData > nMarks
            this.options.marks(end+1:this.nData) = {''};
         elseif this.nData < nMarks
            this.options.marks(this.nData+1:end) = [];
         end
         if ~isnan(this.options.decimal)
            this.options.format = sprintf('%%.%gf', ...
               round(abs(this.options.decimal)));
         end
         %{
         if isfunc(this.options.highlight)
            if ~isempty(parentoptions.range)
               f = this.options.highlight;
               dates = f(this.series);
               dates(dates < parentoptions.range(1)) = [];
               dates(dates > parentoptions.range(end)) = [];
            else
               dates = [];
            end
            this.options.highlight = dates;
         end
         %}
      end
      
      function c = latexcode(this,parent)
         [data,time] = this.series(parent.range);
         time = time.';
         data = data(:,:);
         c = '';
         nper = numel(time);
         for i = 1 : this.nData
            c = [c,gettablerowstring(this,parent,time,data(:,i),i)];
         end
      end
      
      function c = gettablerowstring(this,parent,time,data,i)
         c = [getcaptionstring(), ...
            ' & ',getunitsstring(), ...
            ' & ',latex.stringsubs(this.options.marks{i}), ...
            getdatastring(this,parent,time,data,''),' \\'];      
         function c = getcaptionstring()
            if i == 1
               c = latex.stringsubs(this.caption);
            else
               c = '';
            end
         end
         function c = getunitsstring()
            if i == 1
               c = latex.stringsubs(this.options.units);
            else
               c = '';
            end
         end    
      end 
      
      function c = getdatastring(this,parent,time,data,latexformat)
         c = '';
         if isempty(data)
            return
         end
         if nargin < 4
            latexformat = '';
         end
         nper = numel(data);
         highlightindex = false([1,nper]);
         for i = 1 : numel(this.options.highlight)
            highlightindex = highlightindex | ...
               round(time - this.options.highlight(i)) == 0;
         end         
         for i = 1 : numel(data)
            c1 = report.genericobj.sprintfData(data(i),this.options);
            color = iff(highlightindex(i),'highlightcolor','');
            c1 = report.genericobj.makeBox(...
               c1,latexformat,parent.options.colwidth,color);
            c = [c,' & ',c1];
         end
      end        
      
      function [hfrg,hbkg,lgd,time,data,grid] = plot(this,parent,ax)
         switch func2str(this.options.plotfunc)
            case {'plot','bar'}
               datetick = parent.options.datetick;
               dateformat = parent.options.dateformat;
               [hfrg,time,data,grid] = this.options.plotfunc( ...
                  ax,parent.range,this.series, ...
                  'datetick',datetick, ...
                  'dateformat',dateformat);
               set(hfrg,'userData',func2str(this.options.plotfunc));
            otherwise
               hfrg = [];
         end
         hbkg = [];
         % Legend entry consists of the series caption and mark.
         lgd = cell([1,this.nData]);
         for i = 1 : this.nData
            name = this.caption;
            mark = this.options.marks{i};
            if ~isempty(name) && ~isempty(mark)
               lgd{i} = [name,': ',mark];
            else
               lgd{i} = [name,mark];
            end
         end
      end 
      
      function nData = get.nData(this)
         tmpsize = size(this.series);
         nData = prod(tmpsize(2:end));
      end
      
   end   

end