classdef tableobj < report.genericobj

   properties (Dependent = true)
      nColumns
      range
   end
   
   properties (Constant = true)
      nLeading = 3;
   end
   
   methods

      function this = tableobj(varargin)
         this = this@report.genericobj(varargin{:});
         this.accept = {'series','subheading','band'};
         this.default = {...
            'vline',[],@isnumeric,false,...
            'range',[],@isnumeric,true,...
            'separator','\medskip\par',@ischar,true,...
            'dateformat',irisget('dateformat'),@ischar,true,...
            'colwidth',NaN,@isnumeric,true,...
         };
      end
      
      function [this,varargin] = specargin(this,varargin)
      end

      function [c,helperfiles] = latexcode(this,parent)
         c = '';
         helperfiles = {};
         if isempty(this.options.range)
            if ~isempty(this.caption)
               caption = ['table ''',this.caption,''''];
            else
               caption = 'an unnamed table';
            end
            warning('iris:report', ...
               '\n*** Empty range in %s. No table will be produced.', ...
               caption);
            return
         end
         nl = sprintf('\n');
         vline = [];
         range = [this.range(1)-1,this.range];
         for i = this.options.vline(:)'
            tmp = find(datcmp(i,range));
            if ~isempty(tmp)
               vline(end+1) = tmp - 1;
            end
         end
         c = ['\begin{tabular}{',...
            report.genericobj.tabularSpec(this.nLeading,this.nColumns,vline),...
            '}'];
         if ~isempty(this.caption)
            c = [c,nl,report.genericobj.tabularCaption(...
               this.caption,this.nColumns)];
         end
         c = [c,nl,'\hline',nl,getrangestring(),'\\',nl,'\hline',nl];
         for i = 1 : length(this.children)
            c = [c,latexcode(this.children{i},this),nl];
         end
         c = [c,'\hline',nl,'\end{tabular}'];
         function c = getrangestring()
            c = repmat('& ',[1,this.nLeading-1]);
            if isempty(this.range)
               return
            end
            dates = dat2str(this.range,'dateformat',this.options.dateformat);
            dates = latex.stringsubs(dates);
            for i = 1 : numel(dates)
               % Option 'colwidth' is used only in data cells.
               c = [c,' & ',report.genericobj.makeBox(...
                  dates{i},'',NaN,'')];
            end
         end
      end
      
      function nColumns = get.nColumns(this)
         nColumns = this.nLeading + numel(this.range);
      end

      function range = get.range(this)
         range = this.options.range;
      end
      
   end

end