function [X,Y] = fevd(T,R,K,Z,H,D,U,Omega,nper)
   % FEVD  Forecast error variance decomposition for general state space.

   % The IRIS Toolbox.
   % Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);
n = ny + nf + nb;
nalt = size(T,3);

Phi = time_domain.srf(T,R,K,Z,H,D,U,Omega,nper);

X = cumsum(Phi.^2,3); % FEVD in absolute contributions
Y = zeros(size(X)); % FEVD in relative contributions
status = warning();
warning('off','MATLAB:divideByZero');
varmat = diag(Omega);
varmat = varmat(:)';
varmat = varmat(ones([1,n]),:);
for t = 1 : nper
   X(:,:,t) = X(:,:,t) .* varmat;
   Xsum = sum(X(:,:,t),2);
   Xsum = Xsum(:,ones([1,ne]));
   Y(:,:,t) = X(:,:,t) ./ Xsum;
end
warning(status);

end
% End of primary function.