function [Phi,shksize] = srf(T,R,K,Z,H,D,U,Omega,nper,shksize)
   % SRF  Shock response function, or VMA representation, for general state space.

   % The IRIS Toolbox.
   % Copyright 2007-2009 Jaromir Benes.

if nargin < 10
   shksize = 1;
end

%**************************************************************************
% Function body.

ny = size(Z,1);
[nx,nb] = size(T);
nf = nx - nb;
ne = size(R,2);

% Shock size.
shksize = shksize(:)';
if length(shksize) == 1 && ne ~= 1
   shksize = shksize(1,ones([1,ne]));
end

Phi = zeros([ny+nx,ne,nper]);
Phi(:,:,1) = [...
   H.*shksize(ones([1,ny]),:);...
   R.*shksize(ones([1,nx]),:);...
];
if ny > 0
   Phi(1:ny,:,1) = Phi(1:ny,:,1) + Z*Phi(ny+nf+1:end,:,1);
end
for t = 2 : nper
   Phi(ny+1:end,:,t) = T*Phi(ny+nf+1:end,:,t-1);
   if ny > 0
      Phi(1:ny,:,t) = Z*Phi(ny+nf+1:end,:,t);
   end
end

if ~isempty(U)
   for t = 1 : nper
      Phi(ny+nf+1:end,:,t) = U*Phi(ny+nf+1:end,:,t);
   end
end

end
% End of primary function.