%% Compare unconditional and conditional forecasts.

%% Clear workspace and load model, data, and dates.
%
% Load the estimated model object. If it isn't available, load the
% calibrated one instead.
%

clear('all');
close('all');
home();
load('read_model.mat','m');
load('filter_histdata.mat','F');
load('read_data.mat','starthist','endhist');

startfcst = endhist + 1;
endfcst = startfcst + 3*4;

%% Run unconditional forecast.

Unc = forecast(m,F,startfcst:endfcst,[], ...
   'deviation',true,'output','dbase');

Unc.mean = dbextend(F.mean,Unc.mean);

%% Set up and run conditional forecast.
%
% * Set up a tune database.
% * Set up structural shocks used.
%

% Tune database.
Tune = struct();
Tune.r_ = tseries(startfcst:startfcst+2,F.mean.r_(endhist));

% Switch on/off desired shocks.
disp(get(m,'std'));

% This commmand would set all std devs to zero, and then switch on the
% monetary policy shock only.
%m = assign(m,'std_.*',0);
%m.std_er = s.std_er;

% Keep all shocks except the monetary policy shock.
m.std_er = 0;

disp(get(m,'std'));

% Run forecast conditional on the tune database.
[Unc1,Con1] = forecast(m,F,startfcst:endfcst,Tune, ...
   'deviation',true,'anticipate',false,'initcond','data','output','dbase');

%% Compare the two forecasts.
%
% Splice the unconditional and conditional forecast databases, and report
% the measurement variables and the shocks.
%

sty = struct();
sty.line.linestyle = {'-',':',':','-',':',':'};
sty.line.color = {'blue','blue','blue','red','red','red'};
sty.line.marker = {'.','none','none','.','none','none'};
sty.line.linewidth = {1,1.5,1.5,1,1.5,1.5};

R = struct();
R.mean = dbfun(@horzcat,Unc.mean,Con1.mean);
R.std = dbfun(@horzcat,Unc.std,Con1.std);

[fi,ax] = qplot('forecast_report.q',R,startfcst-4:endfcst, ...
   'highlight',startfcst-4:endhist, ...
   'style',sty);

legend(ax{1}(1),'Unconditional','','','Conditional','','');
legend(ax{2}(1),'Unconditional','Conditional');

figure(fi(1));
