%% Historical simulations.

%% Clear workspace and load model.
% Read the model object with MLE of parameters if available. Otherwise, read the
% model object with first-pass calibration made in |read_model.m|.

clear('all');
close('all');
home();
load('read_model.mat','m');
load('read_data.mat','Gap','starthist','endhist');
D = Gap;
histrange = starthist : endhist;

%% Run Kalman filter.
% Kalman filter output data, |F|, consist of three sub-databases:
%
% * |mean| with point estimates of all model variables as tseries objects,
% * |std| with std dev of those estimates as tseries objects,
% * |mse| with MSE matrices as cell arrays.

[m,F,v] = filter(m,D,starthist:endhist,'deviation',true);
display(F);

%% Re-simulate the filtered data.
% This is to illustrate that running a simulation with the initial
% conditions and shocks estimated by the Kalman filter exactly reproduces
% the historical paths of the observables.

s = simulate(m,F.mean,starthist:endhist,...
   'deviation',true,'anticipate',false);

display(dbfun(@(x,y) max(abs(x-y)),F.mean,s));

%% Contributions of shocks.
% Re-simulate the filtered data with the |'contributions'| option set to
% true. This returns each variable as a multivariate time series with $n+1$
% columns, where $n$ is the number of model shocks. The first $n$ columns
% are contributions of individual shocks (in order of their appearance in
% the |!shocks:transition| declaration block), the last, $n+1$-th column is
% the contribution of the initial condition and/or the deterministic drift.

ecomments = get(m,'ecomments');
c = simulate(m,F.mean,starthist:endhist+8,...
   'deviation',true,'anticipate',false,'contributions',true);

fg = qplot('contrib_report.q',c,starthist:endhist+8,...
   'highlight',endhist+1:endhist+8);
legend(ecomments{:});

figure();
conbar(400*c.dp_{:,1:end-1});
conlegend(ecomments{:});

%% Save estimated model and filtered data.

save('filter_histdata.mat');
