%% Use a separate steady-state code file to find steady state.
%
% This m-file requires Symbolic Math Tbx, version 3.2.3 (i.e. Matlab
% Release 2008a) or LOWER, not higher!
%
% Symbolic Math Tbx 5.1 (Matlab Release 2008b) or higher cannot
% handle the system of equations.
%

%% Clear workspace.

clear('all');
close('all');
home();

%% Write a separate steady-state code.
%
% See the file |SPBC_sstate.s|. Convert the steady-state code to an
% m-file.
%

s = sstate('SPBC_sstate.s');
compile(s,'SPBC_sstate.m');

%% Load the parameter database.

load('read_model.mat','P');
display(P);

%% Run the steady-state m-file with these parameters.
%
% The database returned from |SPBC_sstate.m| will contain the
% steady-state values for all variables. Load the model again, and
% create a model object using the newly computed steady state.
%

if issymbolic()
   P = SPBC_sstate(P);
else
   % If you don't have the Symbolic Math Tbx, the steady state will be
   % computed numerically. Hence it's a good idea to set the tolerance
   % higher the what the Optimization Tbx's defaults are.
   P = SPBC_sstate(P,'tolfun',1e-12,'tolx',1e-12,'display','iter');
end

m1 = model('SPBC.model','std',0.01,'assign',P);
display(m1);

[flag,discrepancy,list] = chksstate(m1);
if flag
   disp('Steady state OK.');
else
   display(discrepancy);
   error('Steady state error: "%s".\n',list{:});
end


%% Compare this steady state with that computed in |read_model.m|.

load('read_model.mat','m');
disp(' ');
disp('Compare steady-state computed in by |sstate| in |read_model.m| with that computed using SPBC_sstate.s');
dbfun(@(x,y)[x,y,x-y],get(m1,'sstateLevel'),get(m,'sstateLevel')),
