%% How to run SPBC example files.

%% Clear workspace.

clear('all');
close('all');
home();

%% Read model.
%
% Create a model object based on the model code file |SPBC.model|.
% Get access to the inside of the model object.
% NB: The m-file |read_model.m| requires Optimisation Tbx.
%

read_model;
inside_model_object;

%% Alternative way of finding steady state.
%
% Symbolic Math Tbx 5.1 (Matlab Release 2008b) and higher cannot handle
% this system of equations symbolically. The symbolic solutions is
% aatempted, but fails, and IRIS then switches to numerical solution.
%
% If you don't have the Symbolic Math Tbx, you can still run this file, and
% see how you can use the *.s files as a more structured alternative to the
% |sstate| function, but all computation will be again peformed
% numerically.

find_steady_state;

%% Simulate shocks.

simulate_simple_shock;
simulate_complex_shocks;
simulate_disinflation;

%% Read and pre-filter data.

read_data;

%% Identify parameters.
%
% Simulate and examine the Fisher information matrix. Estimate parameters using
% different methods. NB: The m-file |estimate_parameters.m| requires
% Optimisation Tbx. The m-file |estimate_parameters.m| may take very long,
% especially on slower computers. You may want to run only selected segments of
% this code.
%

fisher_information_matrix;
estimate_parameters;

%% Estimate unobservables and shocks on history, and produce forecasts.

filter_histdata;
conditional_forecast;

%% Compare the second-moment properties of the model and data.

compare_model_and_data;
