%% Simulate shocks under more complex setup.

%% Clear  workspace and load model.

clear('all');
close('all');
home();
load('read_model.mat','m');
startdate = 1;
enddate = 40;

%% Simple households preference shock.

d = zerodb(m,startdate-3:startdate);
d.ec(startdate) = log(1.01);
s = simulate(m,d,1:40,'deviation',true);
s = dbextend(d,s);

qplot('shock_report.q',s,startdate-1:enddate);
ftitle({'','Preference shock',datestr(now())});

%% Anticipated vs unanticipated households preference shock

d = zerodb(m,startdate-3:startdate);
d.ec(startdate+4) = log(1.01);
s1 = simulate(m,d,startdate:enddate,'deviation',true,'anticipate',true);
s1 = dbextend(d,s1);
s2 = simulate(m,d,startdate:enddate,'deviation',true,'anticipate',false);
s2 = dbextend(d,s2);

s12 = dbfun(@horzcat,s1,s2);
qplot('shock_report.q',s12,startdate-1:enddate);
legend('Anticipated','Unanticipated');
ftitle({'','Preference shock: Anticipated vs Unanticipated',datestr(now())});

%% Simulate consumption shock with delayed policy reaction
% compare simulations in anticipated vs unanticipated mode

nper = 3;

d = zerodb(m,startdate-3:startdate);
d.ec(startdate) = log(1.01);

p = plan(m,startdate:enddate);
p = exogenise(p,'r',startdate:startdate+nper-1);
p = endogenise(p,'er',startdate:startdate+nper-1);
d.r(startdate:startdate+nper-1) = 0;

s1 = simulate(m,d,startdate:enddate,'deviation',true);
s1 = dbextend(d,s1);

s2 = simulate(m,d,startdate:enddate,'deviation',true,'plan',p);
s2 = dbextend(d,s2);

s3 = simulate(m,d,startdate:enddate,'deviation',true,'plan',p,'anticipate',false);
s3 = dbextend(d,s3);

s123 = dbfun(@horzcat,s1,s2,s3);
qplot('shock_report.q',s123,startdate-1:enddate);
legend('No delay','Anticipated','Unanticipated');
ftitle({'','Preference shock with delayed policy reaction',datestr(now())});

%% Simulate a preference shock with a desired impact.
% We want find the size of a preference shock that leads to exactly a 1 pct increase in consumption in the first period.
% Because consumption (C) is a log-linearised variable, we need to specify the 1 pct deviation from its steady state as 1.01.

d = zerodb(m,startdate-3:startdate);
d.C(startdate) = 1.01;

p = plan(m,startdate:enddate);
p = exogenise(p,'C',startdate);
p = endogenise(p,'ec',startdate);
s = simulate(m,d,startdate:enddate,'deviation',true,'plan',p);
s = dbextend(d,s);

disp(s.ec);

qplot('shock_report.q',s,startdate-1:enddate);
ftitle({'','Preference shock with exact impact',datestr(now())});

%% Simulate a preference shocks for multiple parameterisations.

d = zerodb(m,startdate-3:startdate);
d.ec(1) = log(1.01);

m(1:10) = m;
m.mup = linspace(1.5,5,10);

m = solve(m);
s = simulate(m,d,startdate:enddate,'deviation',true);
s = dbextend(d,s);

qplot('shock_report.q',s,startdate-1:enddate);
ftitle({'','Preference shock with mutliple parameterisations',datestr(now())});
