%% Simulate disinflation

%% Clear  workspace, load model

clear('all');
close('all');
load('read_model.mat','m');
startdate = qq(2009,2);
enddate = qq(2012,4);

%% Create a model with higher steady-state inflation
% Set steady-state inflation to 3 pct.
% Solve for the new steady state of nominal variables.
% We know that real variables remain unchanged, so we can fix them.

m1 = m;
m1.pie = log(1.03)/4;
m1 = sstate(m1,'fix',{'Y','L','Q','C','MUC','MDL','MPL','A'});
flag = chksstate(m1);
display(flag);
m1 = solve(m1);

%% Simulate disinflation
% Simulate the low-inflation model m starting from the high-inflation
% model's database.

d = sstatedb(m1,startdate-3:startdate);
s1 = simulate(m,d,startdate:enddate);
s1 = dbextend(d,s1);

qplot('shock_report.q',s1,startdate-1:enddate);
ftitle({'','Disinflation',datestr(now())});

%% Sacrifice ratio
% Sacrifice ratio is the cumulative output loss after a 1% PA disinflation.
% Divide by 4 to get an annualised figure (reported in the literature).

sacrat = -cumsum(100*s1.g)/4,

%% Change price and wage stickiness and compare to baseline
% Create a model object with 2 parameterisations,
% and increase the price and wage stickiness in the second one.

m(1:2) = m;

m(2).xip = m(1).xip/2;
m(2).xiw = m(1).xiw/2;
m = solve(m);

s = simulate(m,d,startdate:enddate);
s = dbextend(d,s);

qplot('shock_report.q',s,startdate:enddate);
ftitle({'','Disinflation with less flexible prices & wages',datestr(now())});

disp('Cumulative output gap (sacrifice ratio):');
sacrifice_ratio = -cumsum(100*s.g{startdate:enddate})/4,
