%% Simulate basic shock

%% Clear  workspace and load model

clear('all');
close('all');
home();
load('read_model.mat','m');
startdate = 1;
enddate = 40;

%% Simulate households preference shock

d = zerodb(m,startdate-3:startdate);
d.ec(startdate) = log(1.01);
s = simulate(m,d,1:40,'deviation',true);
s = dbextend(d,s);
disp(s);

%% Report simulation results

figure();
plotrng = startdate-1 : enddate;

subplot(3,3,1);
h = plot(plotrng,100*(exp(4*s.dp)-1));
grid('on');
title('Final goods prices, Q/Q');

subplot(3,3,2);
h = plot(plotrng,100*(exp(4*s.r)-1));
grid('on');
title('Nominal rate');

subplot(3,3,3);
h = plot(plotrng,100*s.g);
grid('on');
title('Output cycle');

subplot(3,3,4);
h = plot(plotrng,100*(exp(4*s.dw)-1));
grid('on');
title('Nominal wage rate, Q/Q');

subplot(3,3,5);
h = plot(plotrng,100*(log(s.Q) - log(s.MPL)));
grid('on');
title('Real marginal costs');

subplot(3,3,6);
h = plot(plotrng,100*(log(s.MDL/s.MUC) - log(s.MPL)));
grid('on');
title('Inefficiency gap');

ftitle({'','Households preference shock',datestr(now())});

%% Report simulation results using |shock_report.q| and |qplot|
%
% The following call of the |qplot| function produces identical graphs.
% Open the |shock_report.q| file to see how to define the layout of the
% graphs to be produced by |qplot|.
%

qplot('shock_report.q',s,plotrng);
ftitle({'','Households preference shock',datestr(now())});
