%% Read in and solve the SOEMOLIvBC model

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');

%% Compile the steady-state file to an m-file function

s = sstate('SOEMOLIvBC_sstate.s');
compile(s,'SOEMOLIvBC_sstate.m');

%% Create a parameter database

P = struct();

% Steady-state parameters.
P.beta = 0.95^(1/4);
P.gamman = 0.50;
P.gammam = 0.20;
P.nu = 6;
P.alpha = 0.10;
P.delta = 0.04;
P.eta = 0;
P.f2ngdp = 4*0.6;

% Loan performance function parameters.
P.g1 = 1.3491030248668;
P.g2 = 0.30;
P.g3 = 0.60;

% Bank cost function parameters.
P.f0 = 0.01;
P.f1 = 0.003;
P.f2 = 0.003;
P.l2e = 10;

% Arbitrary numeraires.
P.pwstar = 1;
P.p = 1;
P.a = 0;
P.t = 0;

% Transitory parameter.
P.chi = 0.85;
P.xiw = P.nu/0.04;
P.xip = P.nu/0.07;
P.psiy =  5;
P.psix = 50;
P.psii = 7; 3;
P.theta = 1;

% Policy parameters.
P.w = 1;
P.rhor = 0.75;
P.kappa = 3;

% Autoregression coefficients.
P.rhou = 0.8;
P.rhob = 0.99;
P.rhoa = 0.8;
P.rhopw = 0.8;
P.rhot = 0.8;

%% Run the compiled steady-state file

P = SOEMOLIvBC_sstate(P,'display','iter','tolfun',1e-14,'tolx',1e-14);

%% Read in the model code

m = model('SOEMOLIvBC.model','symbolic',true,'assign',P,'addlead',true);

%{
% Using the option 'assign' is equivalent to running
m = model('test.model','symbolic',false,'addlead',true);
m = assign(m,P);
%}

%% Check the steady state

[flag,discrep,list] = chksstate(m);
display(flag);
if ~flag
   error('\n*** Steady-state error: ''%s''',list{:});
end

%% Solve the model

m = solve(m);
display(m);

save('readModel.mat');

