%% Simulations

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');
load('readModel.mat','m');

%% Play around with some of the parameters

m.w = 1;
m.theta = 1;
m = solve(m);

%% Simulate the bubble and burst

d = zerodb(m(1),-3:40);
p = plan(m,1:40);
p = exogenise(p,'B',1:9);
p = endogenise(p,'eb',1:9);
d.B(0:8) = linspace(0,0.20,9);
d.B(9) = 0;
s1 = simulate(m,d,1:400, ...
   'deviation',true,'plan',p,'anticipate',false);
s1 = dbextend(d,s1);

%{
p = nonlinear(p,{ ...
   'Expected loan performance', ...
   'Actual loan performance'},1:10);

s2 = simulate(m,d,1:400, ...
   'deviation',true,'plan',p,'anticipate',false,'lambda',0.5);
s2 = dbextend(d,s2);
%}

%% Report the results

sty = struct();
sty.line.marker = '.';

list = {'100*(GDP-1)','100*(C-1)','100*(I-1)','100*(X-1)','100*(M-1)', ...
   '400*(Rl-1)','400*(R-1)','100*(dP^4-1)', ...
   '100*(S-1)','100*(Pk-1)','100*(Pk1-1)','100*B','100*(G-1)', ...
   '100*(L-1)','100*(F-1)','100*(E-1)','100*(Re-1)'};

dbplot(s1,0:40,list, ...
   'zeroline',true, ...
   'style',sty, ...
   'tight',true);
