%% Input parameters
% All parameters that determine the model's steady state.

!input

   alpha, beta, gamman, gammam, delta, eta, nu
   pwstar, p, a, t, f2ngdp
   g2, g3
   % f0, f1, f2, l2e
   
%% Blocks of steady-state equations
   
!equations

   G = gfn(0.55,g2,g3);
   Ge = G;
   Rl = 1/beta;
   R = Rl*G;% - f1;
   Rstar = R;
   Rwstar = Rstar;
   Rk = Rl;
   Pwstar = pwstar;
   P = p;
   T = t;
   A = a;
   U = 0;
   B = 0;
   Py = P*(nu-1)/nu;
   Pk = P;
   Px = Py;
   
!equations

   nu/(nu-1) * N^eta * P*C = W;
   Ph = 1/Rl*(alpha*P*C + (1-delta)*Ph);
   Rk = (Q + (1-delta)*Pk)/Pk;
   I = delta*K;
   Y = exp(A) * K^(1-gamman-gammam) * N^gamman * M^gammam;
   gamman*Py*Y = W*N;
   gammam*Py*Y = Pm*M;
   (1-gamman-gammam)*Py*Y = Q*K;
   Pm = S*Pwstar;
   Px = exp(T)*S*Pwstar;
   Y = C + X + I;
   F = f2ngdp*NGDP;
   F = F*R - Px*X + Pm*M;
   NGDP = P*C + P*I + Px*X - Pm*M;
   
   !solvefor
   C, N, I, K, Y, X, M
   Ph, Pm, Q, S, W, F, NGDP
   !variables:positive
   !allbut
   
!equations

   %E = F/(l2e-1);
   %L = l2e*E;
   L = F;
   %Re = Rl*L/E*G - R*F/E - f0;
   %f0 = (l2e-1)*f1 - Rl*(1 - G);
   %Re = Rl;
   %L2E = l2e;
   Lambda = 1/(P*C);
   Se = S;
   V = 1;
   Pk1 = Pk;
   Rk1 = Rk;
   g1 = 0.55*Rk*Pk*K/(Rl*L);
   GDP = exp( ...
      P*C/NGDP*log(C) ...
      + P*I/NGDP*log(I) ...
      + Px*X/NGDP*log(X) ...
      - Pm*M/NGDP*log(M));   
   
   dP = 1;
   dW = 1;
   dPwstar = 1;
   d4P = 1;
   
!equations

   PxX2NGDP = Px*X / NGDP;
   PmM2NGDP = Pm*M / NGDP;
   PC2NGDP = P*C / NGDP;
   WN2NGDP = W*N / NGDP;
   PkK2NGDP = Pk*K / NGDP;
   PI2NGDP = P*I / NGDP;
   F2NGDP = F / NGDP;
 
%% Carry-on functions

!function y = gfn(x,s,mu)
   y = 1 - mu.*logncdf(x,-(s.^2)/2,s);
!end
