%% Permanent productivity and TOT shocks

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');
load('readModel.mat','m');

%% Play around with policy and dollarisation

m.w = 0;
m.theta = 0;

%% Create multiple parameterisations with different $\rho_a$

m(1:4) = m;
m.rhoa = [0, 0.3, 0.6, 0.9];
m = solve(m);
display(m);

%% Simulate the productivity or TOT shock

d = zerodb(m,-3:40);
d.ea(5,:) = 0.01 - 0.01i;
s = simulate(m,d,1:40,'deviation',true);
s = dbextend(d,s);

%% Plot the results

sty = struct();
sty.line.marker = '.';
sty.title.fontSize = 9;
sty.axes.fontSize = 9;
sty.axes.xgrid = 'on';
sty.axes.ygrid = 'on';

list = { ...
   '100*(GDP-1)','100*(C-1)','100*(I-1)','100*(X-1)','100*(M-1)', ...
   '400*(Rl-1)','400*(R-1)','100*(dP^4-1)', ...
   '100*(S-1)','100*(Pk-1)','100*(G-1)', ...
   '100*(L-1)','100*(F-1)','100*A'};

dbplot(s,0:40,list, ...
   'zeroline',true, ...
   'style',sty, ...
   'tight',true);

legend('\rho_a = 0','\rho_a = 0.3','\rho_a = 0.6','\rho_a = 0.9', ...
   'Location','Best');

