!input

   sigma, mu, k2e
   
   beta, gamman, gammam, delta, eta, nu
   pwstar, p, a, t, f2ngdp

!equations 'Financial accelerator'

   (1-lambda)*dGammafn(omega,sigma) + lambda*mu*dGfn(omega,sigma) = 0;
   
   (1 - (1-lambda)*Gammafn(omega,sigma) - lambda*mu*Gfn(omega,sigma))*rk2r
      - lambda = 0;
      
   (Gammafn(omega,sigma) - mu*Gfn(omega,sigma))*rk2r*k2e - (k2e-1) = 0;

   (k2e-1)*rl2r = rk2r*omega*k2e;
   
   re2r = rk2r*(1 - Gammafn(omega,sigma))*k2e;
   
   !solvefor
   omega, lambda, rk2r, rl2r, re2r
   
   !variables:positive
   !allbut
   
!equations

   Rwstar = 1/beta;
   Rstar = Rwstar;
   R = Rstar;
   Rk = R*rk2r;
   Rl = R*rl2r;
   Re = R*re2r;
   Pwstar = pwstar;
   P = p;
   T = t;
   A = a;
   U = 0;
   B = 0;
   Py = P*(nu-1)/nu;
   Pk = P;
   Px = Py;
   
!equations

   % !symbolic
   nu/(nu-1) * N^eta * P*C = W;
   Rk = (Q + (1-delta)*Pk)/Pk; %<<<<<<<<<<<<<<<<<<<<<<<<
   I = delta*K;
   Y = exp(A) * K^(1-gamman-gammam) * N^gamman * M^gammam;
   gamman*Py*Y = W*N;
   gammam*Py*Y = Pm*M;
   (1-gamman-gammam)*Py*Y = Q*K;
   Pm = S*Pwstar;
   Px = exp(T)*S*Pwstar;
   Y = C + X + I;
   F = f2ngdp*NGDP;
   F = F*R - Px*X + Pm*M;
   NGDP = P*C + P*I + Px*X - Pm*M;
   
   !solvefor
   C, N, I, K, Y, X, M
   Pm, Q, S, W, F, NGDP
   !variables:positive
   !allbut
   
!equations

   Lambda = 1/(P*C);
   E = Pk*K/k2e;
   L = Pk*K - E;
   Rl = Rk*Pk*K*omega/L;
   D = L - F;
   Pk1 = Pk;
   Rk1 = Rk;
   GDP = exp( ...
      P*C/NGDP*log(C) ...
      + P*I/NGDP*log(I) ...
      + Px*X/NGDP*log(X) ...
      - Pm*M/NGDP*log(M));   
   
   dP = 1;
   dW = 1;
   d4P = 1;
   
   lambda0 = lambda;
   omega0 = omega;
   Rl0 = Rl;   
   
!equations

   PxX2NGDP = Px*X / NGDP;
   PmM2NGDP = Pm*M / NGDP;
   PC2NGDP = P*C / NGDP;
   WN2NGDP = W*N / NGDP;
   PkK2NGDP = Pk*K / NGDP;
   PI2NGDP = P*I / NGDP;
   X2C = X / C;
   I2C = I / C;
   E2NGDP = E / NGDP;
   E2PkK = E / (Pk*K);
 
%% Carry-on functions
   
!function y = Gammafn(x,s)
   z = (log(x) + (s.^2)/2) ./ s;
   y = normcdf(z - s) + x.*(1 - normcdf(z));
!end

!function y = Gfn(x,s)
   z = (log(x) + (s.^2)/2) ./ s;
   y = normcdf(z - s);
!end

!function y = dGammafn(x,s)
   y = 1 - logncdf(x,-(s.^2)/2,s);
!end

!function y = dGfn(x,s)
   y = x.*lognpdf(x,-(s.^2)/2,s);
!end

!function y = cdfn(x,s)
   y = logncdf(x,-(s.^2)/2,s);
!end

!function y = pdfn(x,s)
   y = lognpdf(x,-(s.^2)/2,s);
!end
