%% Comparative statics

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');

%% Compile the steady-state block describing the fin acc

s = sstate('SOEMOLIvFA_sstate.s');
compile(s,'finacc_sstate.m', ...
   'start','Financial accelerator','end','Financial accelerator');

%% Try to run the compiled steady-state block

P = struct();
P.sigma = 0.32;
P.mu = 0.2;
P.k2e = 2;

P = finacc_sstate(P,'display','iter','tolfun',1e-14,'tolx',1e-14);
display(P);

%% Compute comparative statics I
% Change $\mu$ and the leverage ratio ($K/E$).

P = struct();
P.sigma = 0.32;
P.mu = 0.2;
P.k2e = 1.5 : 0.1 : 2.5;

rk2r = [];
rl2r = [];
k2e = [];
default = [];
omega = [];
mulegend = {};

for mu = 0.05 : 0.05 : 0.20
   P.mu = mu;
   
   P = finacc_sstate(P,'display','iter','tolfun',1e-14,'tolx',1e-14);
   
   rk2r(:,end+1) = P.rk2r.';
   rl2r(:,end+1) = P.rl2r.';
   k2e(:,end+1) = P.k2e.';
   default(:,end+1) = cdfn(P.omega,P.sigma).';
   omega(:,end+1) = P.omega.';
   mulegend{end+1} = sprintf('\\mu=%g',mu);
end

%% Report the comparative statics I

nextplot([2,2],'defaultLineLineWidth',1.5);

nextplot();
plot(k2e,100*(rk2r.^4-1));
xlabel('Leverage (K/E)');
ylabel('Capital return to risk-free rate spread % PA');
legend(mulegend{:},'Location','Best');
title('Capital return');
grid('on');

nextplot();
plot(k2e,100*(rl2r.^4-1));
xlabel('Leverage (K/E)');
ylabel('Lending rate to risk-free rate spread % PA');
legend(mulegend{:},'Location','Best');
title('Lending rate')
grid('on');

nextplot();
plot(k2e,100*(1 - (1-default).^4));
xlabel('Leverage (K/E)');
ylabel('Default rates (annualised)');
legend(mulegend{:},'Location','Best');
title('Defaults');
grid('on');

nextplot();
plot(k2e,omega);
xlabel('Leverage (K/E)');
ylabel('Productivity cut-off');
legend(mulegend{:},'Location','Best');
title('Produtivity cut-off');
grid('on');

%% Compute comparative statics II

P = struct();
P.sigma = [0.30,0.50];
P.mu = 0.2;
P.k2e = 2;

P = finacc_sstate(P,'display','iter','tolfun',1e-14,'tolx',1e-14);

x = (0:0.01:3).';
f1 = pdfn(x,P.sigma(1));
f2 = pdfn(x,P.sigma(2));
F1 = cdfn(x,P.sigma(1));
F2 = cdfn(x,P.sigma(2));

%% Report the comparative statics II

nextplot([1,2],'defaultLineLineWidth',1.5);

nextplot();
hold('on');
plot(x,f1,'color','blue');
plot(x,f2,'color','red','lineStyle','--');
ylim = get(gca,'yLim');
stem(P.omega(1),ylim(2),'color','blue');
stem(P.omega(2),ylim(2),'color','red','lineStyle','--');
title('Idiosyncratic productivity -- PDF');
legend(sprintf('\\sigma=%g',P.sigma(1)),sprintf('\\sigma=%g',P.sigma(2)));
grid('on');

nextplot();
hold('on');
plot(x,F1,'color','blue');
plot(x,F2,'color','red','lineStyle','--');
ylim = get(gca,'yLim');
stem(P.omega(1),ylim(2),'color','blue');
stem(P.omega(2),ylim(2),'color','red','lineStyle','--');
title('Idiosyncratic productivity -- CDF');
legend(sprintf('\\sigma=%g',P.sigma(1)),sprintf('\\sigma=%g',P.sigma(2)));
grid('on');

