%% Read in and solve the SOEMOLIvFA model

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');

%% Compile the steady-state file

s = sstate('SOEMOLIvFA_sstate.s');
compile(s,'SOEMOLIvFA_sstate.m');

%% Create a parameter database

P = struct();

% Fin acc parameters.
P.sigma = 0.40;
P.mu = 0.2;
P.k2e = 2.1;

% Other steady-state parameters.
P.beta = 0.97^(1/4);
P.gamman = 0.45;
P.gammam = 0.20;
P.nu = 6;
P.delta = 0.05;
P.eta = 0;
P.f2ngdp = 4*0.4;

% "Numeraires".
P.pwstar = 1;
P.p = 1;
P.a = 0;
P.t = 0;

%% Run the compiled steady-state m-file function

P = SOEMOLIvFA_sstate(P,'display','iter','tolfun',1e-14,'tolx',1e-14);

%% Add transitory parameters

P.chi = 0.8;
P.xiw = P.nu/0.05;
P.xip = P.nu/0.10;
P.psiy =  5;
P.psix = 30;
P.psii = 10;
P.rhou = 0.8;
P.rhob = 0.99;
P.rhor = 0.75;
P.kappa = 5;
P.rhoa = 0.8;
P.rhot = 0.8;
P.zeta = 0.05;

%% Read in a model version with the enterpreneurs bearing aggregate risk

P.who_bears_risk = 'enterpreneurs';
m1 = model('SOEMOLIvFA.model','assign',P,'symbolic',true,'addlead',true);

[flag1,discrep1,list1] = chksstate(m1);
display(flag1);
if ~flag1
   error('\n*** Steady-state error in model 1: ''%s''',list1{:});
end

m1 = solve(m1);
display(m1);

%% Read in a model version with the banks bearing aggregate risk

P.who_bears_risk = 'intermediaries';
m2 = model('SOEMOLIvFA.model','assign',P,'symbolic',true,'addlead',true);

[flag2,discrep2,list2] = chksstate(m2);
display(flag2);
if ~flag2
   error('\n*** Steady-state error in model 2: ''%s''',list2{:});
end

m2 = solve(m2);
display(m2);

save('readModel.mat');

%[T,R,K,Z,H,D,U] = sspace(m);

