%% Country risk premium with SC and NC accelerators

%% Clear the workspace

clear();
close('all');
set(0,'formatSpacing','compact');
load('readModel.mat','m1','m2');

%% State-contingent accelerator simulation

p1 = plan(m1,1:400);
p1 = nonlinear(p1,{'Finacc1','Finacc2','Finacc3','Finacc4'},1:20);

d1 = zerodb(m1,-3:40);
d1.eu(1) = log(1.05)/4;
s1 = simulate(m1,d1,1:400,'deviation',true,'plan',p1,'lambda',0.5);
s1 = dbextend(d1,s1);

%% Non-contingent accelerator simulation

p2 = plan(m2,1:400);
p2 = nonlinear(p2,{'Finacc1','Finacc2','Finacc3','Finacc4'},1:20);

d2 = zerodb(m2,-3:40);
d2.eu(1) = log(1.05)/4;
s2 = simulate(m2,d2,1:400,'deviation',true,'plan',p2,'lambda',0.5);
s2 = dbextend(d2,s2);

%% Plot the simulations

sty = struct();
sty.line.marker = '.';

list = {'100*(GDP-1)','100*(C-1)','100*(I-1)','100*(X-1)','100*(Re-1)', ...
   '100*(Q-1)','400*(Rl-1)','100*(M-1)','400*(Rl-1)','100*(Pk1-1)', ...
   '100*(P-1)','100*(omega-1)','100*B','100*(E-1)', ...
   '100*(Rk1-1)','100*(K-1)','100*(L-1)','100*(F-1)','100*(S-1)'};

dbplot(s1 & s2,0:20,list,'zeroline',true,'style',sty);
