classdef nonlin < metamodel & plans.range
   
   properties
      nonlinfunc = {};
      nonlinx = [];
      nonline = [];
      nonlinlabel = {};
      nonlinperiods = 0;
   end
   
   methods
      function this = nonlin(m,range,varargin)
         this = this@metamodel(m);
         this = this@plans.range(range);
      end
      function flag = isempty(this)
         flag = isempty(this.nonlinfunc) ...
            || isempty(this.nonlinx) ...
            || isempty(this.nonline) ...
            || this.nonlinperiods == 0;
      end
      function this = nonlinear(this,list,range)
         this.nonlinperiods = round(range(end) - this.startdate + 1);
         [this.nonlinfunc,this.nonlinlabel,this.nonlinx,this.nonline] = ...
            nonlinearise_(this,list);
      end
      function this = nonlinearize(this,varargin)
         this = nonlinear(this,varargin{:});
      end
      function this = nonlinearise(this,varargin)
         this = nonlinear(this,varargin{:});
      end
   end
   
end
% End of classdef nonlin.



%**************************************************************************
% Subfunction nonlinearise_().
function [func,label,xi,ei] = nonlinearise_(this,list)

t = this.tzero;
occur = reshape(full(this.occur), ...
   [size(this.occur,1),length(this.name),size(this.occur,2)/length(this.name)]);
occur = occur(:,this.nametype == 3,t);
occur(:,this.shocktype ~= 0) = false;

if ischar(list)
   list = charlist2cellstr(list);
end

eqtni = [];
notfound = [];
for i = 1 : numel(list)
   index = findeqtnlabel(this,list{i},2);
   if isempty(index)
      notfound(end+1) = i;
   else
      eqtni(end+1) = index;
   end
end
if ~isempty(notfound)
   error('iris:nonlin', ...
      '\n*** No transition equation with this label: ''%s''.', ...
      list{notfound});
end
eqtni = unique(eqtni);
eqtni = eqtni(:)';
label = this.eqtnlabel(eqtni);

ei = [];
notfound = [];
for i = eqtni
   if any(occur(i,:))
      ei(end+1) = find(occur(i,:),1);
   else
      notfound(end+1) = i;
   end
end
if ~isempty(notfound)
   error('iris:nonlin', ...
      '\n*** No non-linear shock found in this equation: ''%s''.', ...
      this.eqtn{notfound});
end

n = length(eqtni);
xi = [];
func = {};
replacefunc = @replace_;
invalid = {};
missingLead = {};
for i = eqtni
   eqtn = char(this.eqtnF{i});
   eqtn = regexprep(eqtn,'^@\([^\)]*\)','','once');
   %{
   tokens = regexp(eqtnF,...
      '^@\(x,t,L,G\)x\(:,(\d+),t\)-\((.*?)\)$',...
      'tokens','once');
   realid = str2double(tokens{1});
   tmpxi = find(this.solutionid{2} == realid);
   if isempty(tmpxi)
      xi(end+1) = NaN;
   else
      xi(end+1) = tmpxi;
   end
   tmpfunc = tokens{2};
   %}
   xi(end+1) = NaN;
   if ~isempty(eqtn)
      % tmp0 = tmpfunc;
      eqtn = regexprep(eqtn,...
         'x\(:,(\d+),t([\+\-]\d+)?\)',...
         '${replacefunc($1,$2)}');
      try
      %if this.log(realid)
      %   func{end+1} = eval(['@(x,e,p,t) log(',tmpfunc,')']);
      %else
         func{end+1} = eval(['@(x,e,p,t,L,G)',eqtn]);
      %end
      catch Error
         invalid{end+1} = this.eqtn{i};
         invalid{end+1} = Error.message;
         func{end+1} = [];
         continue
      end
   else
      func{end+1} = '';
   end
end

if ~isempty(missingLead)
   error('iris:nonlin', ...
      '\n*** The lead of this variable is missing from state space: ''%s''.',missingLead{:});
end

if ~isempty(invalid)
   error('iris:nonlin', ...
      '\n*** Error evaluating this equation: %s\nMatlab says: %s',invalid{:});
end

%{
tmpindex = isnan(xi);
if any(tmpindex)
   tmpindex = find(tmpindex);
   error('iris:nonlin', ...
      '\n*** The LHS of this equation is invalid: %s', ...
      this.eqtn(eqtni(tmpindex)));
end
%}

neqtni = length(unique(eqtni));
nxi = length(unique(xi));
nei = length(unique(ei));

if neqtni ~= nei
   error('iris:nonlin', ...
      '\n*** The number of non-linear equations (%g) does not match the number of selected shocks (%g)', ...
      neqtni,nei);
end

if neqtni ~= nxi
   error('iris:nonlin', ...
      '\n*** The number of non-linear equations (%g) does not match the number of LHS variables (%g)', ...
      neqtni,nxi);
end

% End of function body.

   %***********************************************************************
   % Nested function replace_().
   function out = replace_(n,shift)
      n = str2double(n);
      if isempty(shift)
         shift = 0;
      else
         shift = str2double(shift);
      end
      if this.nametype(n) == 2
         % Transition variables.
         index = find(this.solutionid{2} == n+1i*shift);
         if ~isempty(index)
            time = 't';
         else
            index = find(this.solutionid{2} == n+1i*(shift+1));
            if ~isempty(index)
               time = 't-1';
            else
               time = '';
               missingLead{end+1} = sprintf('%s{%+g}', this.name{n},shift);
            end
         end
         if this.log(n)
            out = sprintf('exp(x(%g,%s))',index,time);
         else
            out = sprintf('x(%g,%s)',index,time);
         end         
      elseif this.nametype(n) == 3
         % Shocks.
         index = n - sum(this.nametype < 3); 
         out = sprintf('e(%g,t)',index);
      else
         % Parameters.
         index = n - sum(this.nametype < 4);
         out = sprintf('p(%g)',index);
      end
   end
   % End of nested function replace_().

end
% End of subfunction nonlinearise_().