classdef tunes < metamodel & plans.range
   
   properties
      xAnchors = [];
      yAnchors = [];
      eAnchors = [];
   end
   
   methods
      function this = tunes(m,range)
         this = this@metamodel(m);
         this = this@plans.range(range);
         this.yAnchors = sparse(false([this.nyvec,this.nper]));
         this.xAnchors = sparse(false([this.nxvec,this.nper]));
         this.eAnchors = sparse(zeros([this.nevec,this.nper]));
      end
   end

   % Public signatures.
   methods
      this = exogenise(this,list,dates);
      this = endogenise(this,list,dates,weight);
   end
   
   % Public interface methods.
   methods
      function flag = isempty(this)
         flag = nnz(this.xAnchors)+nnz(this.yAnchors) == 0 ...
            || nnz(this.eAnchors) == 0;
      end
      function varargout = exogenize(this,varargin)
         [varargout{1:nargout}] = exogenise(this,varargin{:});
      end
      function varargout = endogenize(this,varargin)
         [varargout{1:nargout}] = endogenise(this,varargin{:});
      end
      function [yAnchors,xAnchors,eAnchorsReal,eAnchorsImag] = ...
            getanchors(this,range)
         index = range - this.startdate + 1;
         % Dates outside the plan range are redirected to an artifical
         % column created temporarily at the end of the array.
         index(index < 1 | index > this.nper) = this.nper+1;
         % Measurement-variables anchors.
         temp = this.yAnchors;
         temp(:,end+1) = false;
         yAnchors = temp(:,index);
         % Transition-variable anchors.
         temp = this.xAnchors;
         temp(:,end+1) = false;
         xAnchors = temp(:,index);
         % Shock anchors.
         temp = this.eAnchors;
         temp(:,end+1) = 0;
         eAnchorsReal = real(temp(:,index)) ~= 0;
         eAnchorsImag = imag(temp(:,index)) ~= 0;
      end
   end
   
   methods (Static = true, Access = 'protected')
      function error(code,list,varargin)
         switch code
            case 1
               msg = 'Cannot exogenise ''%s''. This is not a valid measurement or transition variable name.';
            case 2
               msg = 'Cannot endogenise ''%s''. This is not a valid shock name.';
            case 3
               msg = 'Dates out of the plan range: %s.';
         end
         if nargin == 1
            list = {};
         elseif ~iscell(list)
            list = {list};
         end
         printmsg('Plan:Tunes','error',msg,list,code);
      end
   end
   
end
% End of classdef tune.