classdef FAVAR < userdataobj & getsetobj
   
   properties
      range = [];
      fitted = [];
      Mean = [];
      Std = [];
      ynames = {}; % Names of observed variables.
      singval = []; % Singular values.
      A = []; % Transition matrices A(L) for factor VAR.
      B = []; % Coefficient matrix at orthonormalised shocks in factor VAR.
      C = []; % Measurement matrix.
      % Triangularised VAR.
      T = []; % Transition matrix.
      U = []; % Transformation matrix.
      Omega = []; % Cov of reduced-form residuals in factor VAR.
      Sigma = []; % Cov of idiosyncratic residuals.
   end
   
   methods
      function this = FAVAR(varargin)
         this = this@userdataobj();
         this = this@getsetobj();
         if ~isempty(varargin) && isfavar(varargin{1})
            this = varargin{1};
         end
      end
   end
   
   methods
      [this,varargout] = estimate(this,varargin)
      varargout = filter(this,varargin)
      varargout = forecast(this,varargin)
      varargout = mean(this)
      disp(this)
   end
   
   methods (Access = 'protected')
      varargout = standardise(this,varargin)
      varargout = specget(this,varargin)
      function varargout = specset(this,varargin)
      end
      function s = specsize(this)
         ny = size(this.C,1);
         nx = size(this.C,2);
         p = size(this.A,2)/nx;
         q = size(this.B,2);
         nalt = size(this.C,3);
         s = {ny,nx,p,q,nalt};
      end
   end
   
   methods (Static = true)
      varargout = pc(varargin)
      varargout = VAR(varargin)
      varargout = error(varargin)
      varargout = cc(varargin);
      varargout = destandardise(varargin)
   end
   
end