function [this,d,c,x,u,e,f,range] = estimate(this,varargin)
% ESTIMATE  Estimate FAVAR using static principal components.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes & Troy Matheson.

% Process input data.
[y,range,list,inputformat,varargin] = inputdata(varargin{:});
if isequal(inputformat,'dbase')
   this.ynames = transpose(list(:));
end
this.range = range;

% Parse required input arguments.
crit = varargin{1};
varargin(1) = [];

% Parse and validate options.
options = passvalopt('FAVAR.estimate',varargin{:});

% Determine format of output data.
if strcmpi(options.output,'auto')
   outputformat = inputformat;
else
   outputformat = options.output;
end

%**************************************************************************
% Function body.

% Standardise input data.
y0 = y;
[this,y] = standardise(this,y);

% Estimate static factors using principal components.
[x,this.C,u,this.Sigma,this.singval,sample,f] = ...
   FAVAR.pc(y,crit,options.method);

% Estimate VAR(p,q) on factors.
[this.A,this.B,this.Omega,this.T,this.U,e,this.fitted] = ...
   FAVAR.VAR(x,options.order,options.rank);

% Zero off-diagonal elements if requested.
if ~options.cross
   this.Sigma = diag(diag(this.Sigma));
end

if nargout > 1
   d = outputdata(this,outputformat,range,y0,[],'ynames');
end

if nargout > 2
   % Common components.
   c = FAVAR.cc(this.C,x);
   c = FAVAR.destandardise(this.Mean,this.Std,c);
   c = outputdata(this,outputformat,range,c,[],'ynames');
end

if nargout > 3
   % Factors.
   template = tseries();
   x = replace(template,permute(x,[2,1,3]),range(1));
end

if nargout > 4
   % Idiosyncratic residuals.
   u = FAVAR.destandardise(0,this.Std,u);
   u = outputdata(this,outputformat,range,u,[],'ynames');
end

if nargout > 5
   % Residuals from the factor VAR.
   e = replace(template,permute(e,[2,1,3]),range(1));
end

if nargout > 6
   % Contributions to the factors.
   f = replace(template,permute(f,[2,1,3]),range(1));
end

if nargout > 7
   range = range(sample);
end

end
% End of primary function.