function [this,d,c,f,u,e] = filter(this,data,range,varargin)

options = passvalopt('FAVAR.filter',varargin{:});

%**************************************************************************
% Function body.

[ny,nx,p,q,nalt] = size(this);

% Retrieve and standardise input data.
[outputformat,range,y] = datarequest(this,data,range,options);
[this,y] = standardise(this,y);
ndata = size(y,3);
nper = size(y,2);

% Initialise Kalman filter.
x0 = zeros([p*nx,1]);
R = this.U(1:nx,:)'*this.B;
P0 = time_domain.acovf(this.T,R,[],[],[],[],this.U,1,[],0);

% Inversion method for the FMSE matrices.
if strcmpi(options.tolerance,'auto')
   invfunc = @pinv;
else
   invfunc = @(x) pinv(x,options.tolerance);
end

% Zero off-diagonal elements in idiosyncratic cov matrix if requested.
Sigma = this.Sigma;
if ~options.cross
   Sigma = diag(diag(Sigma));
end

% Run Kalman smoother to improve on factor estimates.
[x,Px,e,u,y,Py,yindex] = time_domain.smoothervar( ...
   this.A,this.B,[],this.C,1,Sigma,y,[],x0,P0,invfunc,true);

if options.meanonly
   Px = Px(:,:,[],[]);
   Py = [];
end

if nargout > 1
   [y,Py] = FAVAR.destandardise(this.Mean,this.Std,y,Py);
   d = outputdata(this,outputformat,range,y,Py,'ynames');
end

if nargout > 2
   % Common components.
   [c,Pc] = FAVAR.cc(this.C,x(1:nx,:,:),Px(1:nx,1:nx,:,:));
   [c,Pc] = FAVAR.destandardise(this.Mean,this.Std,c,Pc);
   c = outputdata(this,outputformat,range,c,Pc,'ynames');
end

if nargout > 3
   % Factors.
   f = outputdata(this,'tseries',range,x(1:nx,:,:),Px(1:nx,1:nx,:,:));
   if ~options.meanonly
      % Means and MSEs that can be used as initial condition.
      lastobs = find(any(yindex ~= 0,1),1,'last');
      if isempty(lastobs)
         index = 1 : nper;
      else
         index = lastobs : nper;
      end
      f.init = {x(:,index,:),Px(:,:,index,:),range(index)};
   end
end

if nargout > 4
   u = FAVAR.destandardise(0,this.Std,u);
   u = outputdata(this,outputformat,range,u,NaN,'ynames');
end

if nargout > 5
   template = tseries();
   e = replace(template,permute(e,[2,1,3]),range(1));
   if ~options.meanonly
      e = struct( ...
         'mean',e, ...
         'std',replace(template,zeros([0,size(e,1),size(e,3)]),range(1)) ...
         );
   end
end

end
% End of primary function.