function [x,flag] = specget(this,query)

[ny,nx,p,q,nalt] = size(this);
flag = true;
switch query
   case {'ynames','ylist','names','list'}
      if isfunc(this.ynames)
         x = cell([1,ny]);
         for i = 1 : ny
            x{i} = this.ynames(i);
         end
      else
         x = this.ynames;
      end
   case 'a'
      if all(size(this.A) == 0)
         x = [];
      else
         x = time_domain.var2poly(this.A);
      end
   case 'a*'
      x = reshape(this.A,[nx,nx,p,nalt]);
   case 'b'
      x = this.B;
   case 'c'
      x = this.C;
   case 'omega'
      x = this.Omega;
   case 'sigma'
      x = this.Sigma;
   case 'var'
      x = getVARstruct_(this);
      x = VAR(x);
   case {'singval','sing','singvalues'}
      x = this.singval;
   otherwise
      x = [];
      flag = false;
end
end
% End of primary function.

%**************************************************************************
% Subunction getVAR_().
function w = getVARstruct_(f)
   [ny,nx,p,q,nalt] = size(f);
   w = struct();
   w.A = f.A; % Untransformed transition matrices.
   w.K = zeros([nx,nalt]); % Constant vector.
   w.B = f.B;
   w.std = 1;
   w.Omega = f.Omega; % Cov of reduced-form residuals.
   if q < nx
      for ialt = 1 : nalt
         w.Omega(:,:,ialt) = f.B(:,:,ialt)*f.B(:,:,ialt)';
      end
      w.B = [w.B,zeros([nx,nx-q,nalt])];
   end
   w.Sigma = []; % Cov of parameters.
   w.T = f.T; % Shur decomposition of transition matrix.
   w.U = f.U; % Schur transformation of variables.
   w.range = f.range; % User range.
   w.fitted = f.fitted; % Effective estimation sample.
   w.Rr = []; % Parameter restrictions.
   w.nhyper = nx*p; % Number of estimated hyperparameters.
   w.eigval = ordeig(w.T); % Vector of eigenvalues.
   w.ynames = @(n) sprintf('factor%g',n); % Names of endogenous variables.
   w.enames = @(yname,n) ['res_',yname]; % Names of residuals.
   % w.aic, w.sbc to be populated within VAR().
end
% End of subfunction getVAR_().