classdef VAR < userdataobj & getsetobj & subsobj
   
   properties (Access = 'public')
      A = []; % Untransformed transition matrices.
      K = []; % Constant vector.
      Omega = []; % Cov of reduced-form residuals.
      Sigma = []; % Cov of parameters.
      T = []; % Shur decomposition of transition matrix.
      U = []; % Schur transformation of variables.
      range = []; % User range.
      fitted = []; % Effective estimation sample.
      aic = NaN; % Akaike info criterion.
      sbc = NaN; % Schwartz bayesian criterion.
      Rr = []; % Parameter restrictions.
      nhyper = NaN; % Number of estimated hyperparameters in each equation.
      eigval = []; % Vector of eigenvalues.
      ynames = {}; % Names of endogenous variables.
      enames = {}; % Names of residuals.
      % Structural VARs.
      B = []; % Coeffiecient matrix at residuals.
      std = []; % Stdev of structural residuals.
   end
   
   methods (Access = 'public')
      % Public method signatures.
      varargout = acf(varargin)
      varargout = companion(varargin)
      varargout = demean(varargin)
      varargout = disp(varargin)
      varargout = eig(varargin)
      varargout = end(varargin)
      varargout = estimate(varargin)
      varargout = fevd(varargin)
      varargout = fmse(varargin)
      varargout = forecast(varargin)
      varargout = fprintf(varargin)
      varargout = integrate(varargin)
      varargout = irf(varargin)
      varargout = isempty(varargin)
      varargout = isexplosive(varargin)
      varargout = isrvar(varargin)
      varargout = isstationary(varargin)
      varargout = issvar(varargin)
      varargout = length(varargin)
      varargout = lrtest(varargin)
      varargout = mean(varargin)
      varargout = portest(varargin)
      varargout = resample(varargin)
      varargout = reverse(varargin)
      varargout = rvar(varargin)
      varargout = saveobj(varargin)
      varargout = simulate(varargin)
      varargout = sprintf(varargin)
      varargout = srf(varargin)
      varargout = sspace(varargin)
      varargout = subsasgn(varargin)
      varargout = SVAR(varargin)
      varargout = vma(varargin)
      varargout = xsf(varargin)
   end
   
   methods (Access = 'protected')
      varargout = glsq(varargin)
      varargout = restrict_(varargin)
      varargout = somega_(varargin)
      varargout = specget(varargin)
      varargout = specset(varargin)
      varargout = specsize(varargin)
      varargout = specsubsref(varargin)
      varargout = selectalt(varargin)
   end
   
   methods (Static = true, Access = 'protected')
      error(varargin);
      warning(varargin);
      varargout = stackdata(varargin);
      varargout = schur(varargin)      
      varargout = infocrit(varargin)      
   end
   
   methods (Static = true)
      varargout = loadobj(varargin)
   end
   
   methods (Access = 'public')
      
      function this = VAR(varargin)
         % Type or click on <a href="matlab: idoc VAR.VAR">idoc VAR.VAR</a> to get help on this function.
         this = this@userdataobj();
         this = this@getsetobj();
         this = this@subsobj();
         if nargin == 1
            if isa(varargin{1},'VAR')
               this = varargin{1};
               return
            elseif isstruct(varargin{1})
               list = properties(this);
               for i = 1 : length(list)
                  try
                     this.(list{i}) = varargin{1}.(list{i});
                  end
               end
               % Populate triangular representation.
               if isempty(this.T) || isempty(this.U) || isempty(this.eigval)
                  [this.T,this.U,this.eigval] = schur(this.A);
               end
               % Populate information criteria.
               if isempty(this.aic) || isempty(this.sbc)
                  [this.aic,this.sbc] = ...
                     VAR.infocrit(this.Omega,this.nhyper,this.fitted);
               end
               return
            end
         else
            options = passvalopt('VAR.VAR',varargin{:});
            if ~isempty(options.userdata)
               this = userdata(this,options.userdata);
            end
         end
      end
      % End of primary function.
      
   end
   
end