function [this,data] = integrate(this,data,varargin)
% <a href="matlab: edit var/integrate">INTEGRATE</a>  Integrate VAR variables.
%
% Syntax:
%   this = integrate(this)
%   [this,data] = integrate(this,data,...)
% Output arguments:
%   this [ VAR ] Intergrated VAR model.
%   data [ tseries ] Integrated data associated with VAR model.
% Required input arguments:
%   this [ VAR ] VAR model to be integrated.
%   data [ tseries ] Data associated with VAR model.
% <a href="options.html">Optional input arguments:</a>
%   'applyto' [ logical | numeric | <a href="default.html">true([1,ny])</a> ] Index of variables to be integrated.

% The IRIS Toolbox 2009/10/30.
% Copyright 2007-2009 Jaromir Benes.

[ny,p,nalt] = size(this);

options = passvalopt('VAR.integrate',varargin{:});

% Make options.applyto logical index.
if isnumeric(options.applyto)
   if any(isinf(options.applyto))
      options.applyto = true([1,ny]);
   else
      aux = options.applyto;
      options.applyto = false([1,ny]);
      options.applyto(aux) = true;
   end
else
   options.applyto = logical(options.applyto(1:ny));
end

%**************************************************************************
% Function body.

% Integrate model.
if any(options.applyto)
   D = cat(3,eye(ny),-eye(ny));
   D(~options.applyto,~options.applyto,2) = 0;
   A = this.A;
   this.A(:,end+1:end+ny,:) = NaN;
   for ialt = 1 : nalt
      a = time_domain.polyprod(time_domain.var2poly(A(:,:,ialt)),D);
      this.A(:,:,ialt) = time_domain.poly2var(a);
   end
   [this.T,this.U,this.eigval] = VAR.schur(this.A);
end

% Integrate datapack series.
if nargin > 1 && nargout > 1 && ~isempty(data)
   [outputformat,ans,y,e] = datarequest(this,data,Inf,options);
   y(:,options.applyto,:) = cumsum(y(:,options.applyto,:),1);
   presample = nan([ny,1,size(y,3)]);
   presample(options.applyto,:,:) = 0;
   y = [presample,y];
   data = outputdata(this,outputformat,range,[y;e],[],'names');
end

end
% End of primary function.