function [stat,crit] = lrtest(wu,wr,level)
% <a href="VAR/lrtest">LRTEST</a>  Likelihood ratio test for VAR models.
%
% Syntax:
%   [stat,crit] = lrtest(w1,w2,level)
% Output arguments:
%   stat [ numeric ] Test stastic.
%   crit [ numeric ] Test critical value.
% Required input arguments:
%   wu [ VAR ] Unrestricted VAR model.
%   wr [ VAR ] Restricted VAR model.
%   level [ numeric ] Significance level (e.g. 0.05).

% The IRIS Toolbox 2009/09/14.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 3
   level = 0.05;
end

%**************************************************************************
% Function body.

[ans,ans,naltr] = size(wr);
[ans,ans,naltu] = size(wu);
nloop = max([naltu,naltr]);

if wu.nhyper == wr.nhyper
   VAR.warning(3);
end

% Check the number of hyperparameters, and swap restricted and unrestricted
% VARs if needed.
if wu.nhyper < wr.nhyper
   [wu,wr] = deal(wr,wu);
end

stat = nan([1,nloop]);
flag = true;
sampler = wr.range(wr.fitted);
sampleu = wu.range(wu.fitted);
if ~cmpdat(sampleu,sampler)
   VAR.warning(33);
end
nper = length(sampleu);
for iloop = 1 : nloop
   if iloop <= naltr
      logdet_Omegar = log(det(wr.Omega(:,:,iloop)));
   end
   if iloop <= naltu
      logdet_Omegau = log(det(wu.Omega(:,:,iloop)));    
   end
   % Check if explained samples are the same in both VARs.
   stat(iloop) = nper*(logdet_Omegar - logdet_Omegau);
end

% Critical value.
if nargout > 1
   crit = chi2inv(1-level,wu.nhyper-wr.nhyper);
end

end
% End of primary function.