function data = resample(w,data,range,ndraw,varargin)
% Type <a href="matlab: idoc VAR.resample">idoc VAR.resample</a> or click the link to get help on this function.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if nargin < 4
   ndraw = range;
   range = w.range(w.fitted);
elseif ischar(ndraw)
   varargin = [{ndraw},varargin];
   ndraw = range;
   range = w.range(w.fitted);
end

% Parse required input arguments.
P = inputParser();
P.addRequired('w',@isvar);
P.addRequired('input',@(x) isnumeric(x) || isstruct(x) || istseries(x));
P.addRequired('ndraw',@(x) isnumericscalar(x));
P.parse(w,data,ndraw);

% Parse options.
options = passvalopt('VAR.resample',varargin{:});

%**************************************************************************
% Function body .

if isempty(range)
   VAR.error(34);
end

[ny,p,nalt] = size(w);
xrange = range(1)-p : range(end);
% |nper| includes pre-sample.
nper = numel(xrange);
[outputformat,xrange,y,e] = datarequest(w,data,xrange,options);
ndata = size(y,3);

% Works only with single parameterisation.
if nalt > 1
   VAR.error(19,'<<matlab:idoc(''VAR.resample'') resample>>');
end

% Works only with single data set.
if ndata > 1
   VAR.error(20,'<<matlab:idoc(''VAR.resample'') resample>>');
end

Y = nan([ny,nper,ndraw]);

% Fixed initial condtion from pre-sample data.
Y(:,1:p,:) = y(:,1:p,ones([1,ndraw]));

% TODO: randomise initial condition
%{
if options.randomise
else
end
%}

A = w.A;
K = w.K;
B = w.B;

if isempty(B)
   stdvec = ones([1,ny]);
else
   stdvec = w.std(ones([1,ny]));
end

% Diagonalise RVAR residuals.
if isempty(B) && strcmp(options.method,'montecarlo')
   % Safely factorise (chol/svd) the covariance matrix so that we can draw
   % from multivariate normal.
   F = time_domain.factorise(w.Omega);
end

% Create a command-window progress bar.
if options.progress
   progress = progressBar('IRIS model.resample progress');
end

for idraw = 1 : ndraw
   Ei = zeros([ny,nper]);
   Ei(:,p+1:end) = drawresiduals_();
   if ~isempty(B)
      % SVAR.
      Ei(:,p+1:end) = B*Ei(:,p+1:end);
   end
   Yi = Y(:,:,idraw);
   for t = p+1 : nper
      Yilags = Yi(:,t-(1:p));
      Yi(:,t) = A*Yilags(:) + Ei(:,t) + K;
   end
   Y(:,:,idraw) = Yi;
   % Update the progress bar.
   if options.progress
      progress = update(progress,idraw/ndraw);
   end   
end

% Return only endogenous variables, not shocks.
data = outputdata(w,outputformat,xrange,Y,[],'ynames');

% End of function body.

   %***********************************************************************
   % Nested function drawresiduals_().
   function E = drawresiduals_()
      if strcmp(options.method,'bootstrap')
         if options.wild
            % Wild bootstrap.
            % Setting draw = ones([1,nper-p]) would reproduce sample.
            draw = randn([1,nper-p]);
            E = e(:,p+1:end).*draw(ones([1,ny]),:);
         else
            % Standard Efron bootstrap.
            % Setting draw = 1 : nper-p would reproduce sample;
            % draw is uniform integer [1,nper-p].
            draw = randi([1,nper-p],[1,nper-p]);
            E = e(:,p+draw);
         end
      elseif strcmp(options.distribution,'montecarlo')
         E = randn([ny,nper-p]);
         if isempty(B)
            % RVAR.
            E = F*E;
         end
      %{   
      else
         if isempty(B)
            % RVAR.
            E = options.distribution([ny,nper-p],Omega);
         else
            % SVAR.
            E = options.distribution([ny,nper-p],diag(stdvec));
         end
      %}
      end
   end
   % End of nested function drawresiduals_().

end
% End of primary function.
