function [T,U,eigval] = schur(A)
% SCHUR  Triangular representation of VAR.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

[ny,p_ny,nalt] = size(A);
p = p_ny/ny;

if p == 0
  T = zeros([ny,ny,nalt]);
  U = eye(ny);
  U = U(:,:,ones([1,nalt]));
  eigval = zeros([1,ny,nalt]);
  return
end

% Companion transition matrix.
if p == 0
   A = zeros([ny,ny,nalt]);
else
   x = A;
   A = zeros([ny*p,ny*p,nalt]);
   for ialt = 1 : nalt
      A(:,:,ialt) = [x(:,:,ialt);eye([ny*(p-1),ny*p])];
   end
end

realsmall = getrealsmall();
U = nan([ny*p,ny*p,nalt]);
T = nan([ny*p,ny*p,nalt]);
eigval = nan([1,ny*p,nalt]);
for ialt = 1 : nalt
   if any(any(isnan(A(:,:,ialt))))
      continue
   else   
      [U(:,:,ialt),T(:,:,ialt)] = schur(A(:,:,ialt));
      eigval = ordeig(T(:,:,ialt));
      eigval = eigval(:)';
      unstable = abs(eigval) > 1 + realsmall;
      unit = abs(abs(eigval) - 1) <= realsmall;
      clusters = zeros(size(eigval));
      clusters(unstable) = 2;
      clusters(unit) = 1;
      [U(:,:,ialt),T(:,:,ialt)] = ...
         ordschur(U(:,:,ialt),T(:,:,ialt),clusters);
      eigval(1,:,ialt) = ordeig(T(:,:,ialt))';
   end
end

end
% End of primary function.