function [this,flag] = specset(this,name,value)
% SPECSET  Class-specific implementation of SET function.

% The IRIS Toolbox 2009/10/30.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

flag = true;
realsmall = getrealsmall();
[ny,p,nalt] = size(this);

switch name
   case 'a'
      this.A(:) = value(:);
      [this.T,this.U,this.eigval] = schur(this.A);
   case 'b'
      this.B = value(:);
       if nalt > 1 && size(this.B,3) == 1
          this.B = this.B(:,:,ones([1,nalt]));   
       end
   case {'const','c','k'}
      this.K(:) = value(:);
   case 'comment'
      this.comment = value;
   case 'mean'
      nonstat = [];
      eigval = eig(this);
      mean_ = mean(this);
      x = value;
      if nalt > 1 && size(x,3) == 1
         x = x(:,1,ones([1,nalt]));
      end
      for ialt = 1 : nalt
         index = isnan(x(:,1,ialt));
         if any(index)
            x(index,1,ialt) = mean_(index,1,ialt);
         end
         if any(abs(eigval(1,:,ialt)) > 1-realsmall)
            nonstat(end+1) = ialt;
         else
           this.K(:,ialt) = sum(...
              time_domain.var2poly(this.A(:,:,ialt)),3) ...
              *x(:,1,ialt);
         end
      end
      if ~isempty(nonstat)
         VAR.warning(7,sprintf(' #%g',nonstat));
      end
   otherwise
      flag = false;
end

end
% End of primary function.