function [Phi,Psi,s,c] = srf(this,time)
% <a href="matlat: edit VAR/srf">SRF</a>  Innovation (shock) response function.
%
% Syntax:
%   [Phi,Psi,s,c] = srf(this,nper)
%   [Phi,Psi,s,c] = srf(this,range)
% Output arguments:
%   Phi [ numeric ] VMA representation of VAR model.
%   Psi [ numeric ] Cumulative VMA.
%   s [ tseries ] Forecast error responses as multivariate time series.
%   c [ tseries ] Cumulative forecast error responses as multivariate time series.
% Required input arguments:
%   this [ VAR ] VAR model.
% Required input arguments for syntax (1):
%   nper [ numeric ] Number of periods.
% Required input arguments for syntax (2):
%   range [ numeric ] Time range (<a href="dates.html">IRIS serial date numbers</a>).

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

% Tell if time is nper or range.
if length(time) == 1 && round(time) == time && time > 0
   range = 1 : time;
else
   range = time(1) : time(end);
end
nper = length(range);

%**************************************************************************
% Function body..

% VMA / shock responses.
Phi = time_domain.var2vma(this.A,this.B,nper);

% Cumulative shock responses.
Psi = cumsum(Phi,3);

% Create tseries objects.
if nargout > 2
   s = permute(Phi,[3,1,2,4]);
   s = tseries(range,s);
   if nargout > 3
      c = permute(Psi,[3,1,2,4]);
      c = replace(s,c);
   end
end

end
% End of primary function.
