function [T,R,k,Z,H,d,U,Omega] = sspace(this,alt)
% SSPACE  Quasi-triangular state-space form for VAR.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

[ny,p,nalt] = size(this);

if nargin < 2
   alt = Inf;
end
if isequal(alt,Inf)
   alt = 1 : nalt;
elseif islogical(alt)
   alt = find(alt(:)');
else
   alt = alt(:)';
end

T = this.T(:,:,alt);
U = this.U(:,:,alt);
R = permute(U(1:ny,:,:),[2,1,3]);
K = this.K(:,alt);

% Constant term.
k = zeros([ny*p,1,length(alt)]);
for i = 1 : length(alt)
   k(:,1,i) = transpose(U(1:ny,:,i))*K(:,1,i);
end

Z = U(1:ny,:,:);

H = zeros([ny,ny,length(alt)]);
d = zeros([ny,1,length(alt)]);

if isempty(this.B)
   % Reduced-form VAR.
   Omega = this.Omega(:,:,alt);
else
   % Structural VAR.
   B = this.B(:,:,alt);
   for i = 1 : length(alt)
      R(:,:,i) = R(:,:,i)*B(:,:,i);
   end
   Omega = somega_(this,alt);
end

end
% End of primary function.