function [S,D] = xsf(w,freq)
% <a href="matlab: edit rvar/xsf">XSF</a>  Power spectrum and spectral density functions.
%
% Syntax:
%   [S,D] = xsf(w,freq)
% Output arguments:
%   S [ numeric ] Power spectrum function.
%   D [ numeric ] Spectral density function.
% Required input arguments:
%   w [ VAR ] VAR model.
%   freq [ numeric ] Frequencies at which XSF is to be evaluated.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if ~isnumeric(freq)
   error('Incorrect type of input argument(s).');
end

%**************************************************************************
% Function body..

[ny,p,nalt] = size(w);
freq = freq(:)';
nfreq = length(freq);

S = nan([ny,ny,nfreq,nalt]);
for ialt = 1 : nalt
   % Compute power spectrum function.
   S(:,:,:,ialt) = ...
      freq_domain.xsfvar(w.A(:,:,ialt),w.Omega(:,:,ialt),freq);
end
S = S / (2*pi);

if nargout > 1
   % Convert power spectrum to spectral density.
   D = freq_domain.psf2sdf(S,acf(w));
end

end
% End of primary function.