classdef container < getsetobj
   
   methods
      
      function this = container(varargin)
         if nargin == 1
            if isa(this,'container')
               this = varargin{1};
            end
         end
      end
      
      function this = put(this,varargin)
         this = set(this,varargin{:});
      end      
      
      function x = list(this)
         x = container.request('list');
      end
      
      function lock(this,varargin)
         container.request('lock',varargin{:});
      end
      
      function unlock(this,varargin)
         container.request('unlock',varargin{:});
      end
      
      function flag = islocked(this,varargin)
         flag = container.request('islocked',varargin{:});
      end
      
      function list = locked(this)
         list = container.request('locked');
      end
      
      function list = unlocked(this)
         list = container.request('unlocked');
      end
      
      function this = remove(this,varargin)
         container.request('remove',varargin{:});
      end
      
      function this = clear(this)
         container.request('clear');
         munlock('container.request');
      end
      
      function x = saveobj(this)
         x = container.request('save');
      end
      
      function disp(this)
         list = container.request('list');
         status = get(0,'formatSpacing');
         fprintf('\tcontainer object: 1-by-1\n');
         set(0,'formatSpacing','compact');
         disp(list);
         set(0,'formatSpacing',status);
      end
      
   end
   
   methods (Static = true)
      
      function this = loadobj(this)
         container.request('load',this);
         this = container();
      end
      
   end
   
   methods (Access = 'protected')
      
      function [x,flag] = specget(this,name)
         [x,flag] = container.request('get',name);
         if ~flag
            container.error(2,name);
         end
      end
      
      function [this,flag] = specset(this,name,data)
         flag = container.request('set',name,data);
         if ~flag
            container.error(1,name);
         end
      end
      
      function varargout = specsize(this)
         count = container.request('count');
         varargout = {1,count};
      end
      
   end
   
   methods (Static = true, Access = 'private')
      
      varargout = request(action,varargin)
      
      function error(code,list,varargin)
         switch code
            case 1
               msg = 'Cannot re-write container entry ''%s''. This entry is locked.';
            case 2
               msg = 'Reference to non-existent container entry ''%s''.';
         end
         if nargin == 1
            list = {};
         elseif ~iscell(list)
            list = {list};
         end
         printmsg('container','error',msg,list,code);
      end
   end
   
end