classdef irisparser
   
   properties
      fname = '';
      params = struct();
      code = '';
      labels = {};
   end
   
   methods (Access = 'public')
      
      % Constructor.
      function p = irisparser(varargin)
         % IRISPARSER  General IRIS code parser.
         % p = irisparser(outputfile)
         % p = irisparser(outputfile,assign)
         % p = irisparser(outputfile,assign,saveas)
         if nargin == 0
            return
         end
         if nargin > 0
            if ischar(varargin{1})
               p.fname = varargin{1};
            elseif iscellstr(varargin{1})
               p.fname = [varargin{1}{1}, ...
                  sprintf(' & %s',varargin{1}{2:end})];
            end
         end
         if nargin > 1
            p.params = varargin{2};
         end
         if nargin > 2
            saveas = varargin{3};
         else
            saveas = '';
         end
         [p.code,p.labels] = ...
            irisparser.readcode(varargin{1},p.params,p.labels,p.fname);
         % User requests saving the pre-parsed file.
         if ~isempty(saveas)
            c = p.code;
            % Substitute labels and tags vack for the #(...) marks.
            c = labelstore(p,c,'''%s''');
            char2file(c,saveas);
         end
      end
         
      % Public function signatures.
      [m,varargout] = model(p,m,varargin);
      [s,varargout] = sstate(p,s,varargin);
      r = reporting(p);
      varargout = labelstore(p,varargin);

   end
   
   methods (Static = true)
      block = model_readblocks(c,fname);
      [name,nametype,shocktype,label,value] = model_readnames(block);
      [subsname,subsbody] = model_readsubs(block);
   end   

   methods (Static = true)
      clone(inputfile,outputfile,varargin);
      preparse(inputfile,outputfile,varargin);
   end
   
   methods (Access = 'protected', Static = true)
      % Private static function signatures.
      [code,labels] = readcode(fname,params,varargin);
      error(code,list,varargin);
      warning(code,list,varargin);
      [name,label,value] = model_findnames(list);
   end
      
end