function block = model_readblocks(c,fname)

% Check all words starting with an !.
chkkeywords_(c,fname);

% Add new line character at the end of the file.
c = [c,char(10)];

% read blocks
blkname = {...
   '!variables:measurement\s', ...     1
   '!variables:transition\s', ...      2
   '!shocks\s', ...                    3 Untyped shocks, for bkw compatibility only.
   '!parameters\s', ...                4
   '!variables:log\s', ...             5
   '!equations:measurement\s', ...     6
   '!equations:transition\s', ...      7
   '!substitutions\s', ...             8
   '!dtrends:measurement\s', ...       9
   '!equations:reporting\s', ...       10
   '!shocks:measurement\s', ...        11 Measurement shocks (type 1).
   '!shocks:transition\s', ...         12 Transition shocks (type 2).
   '!links', ...                       13 Dynamic links for parameters and/or steady-states.
   '!shocks:nonlinear\s', ...          14 Shocks for nonlinear simulations (type 0).
};

nblk = length(blkname);

% End of block is start of another block or end of file
eoblk = sprintf('|%s',blkname{:});
eoblk = sprintf('(?=$%s)',eoblk);

% Read declaration blocks.
block = cell([1,nblk]);
for i = 1 : length(blkname)
   patt = [blkname{i},'(.*?)',eoblk];
   tokens = regexpi(c,patt,'tokens');
   tokens = [tokens{:}];
   if ~isempty(tokens)
      % !allbut must be in all or none of log declaration blocks
      if i == 5
         chkallbut_(tokens,fname);
      end
      block{i} = [tokens{:}];
   else
      block{i} = '';
   end
end

% No block of transition variables found.
if isempty(block{2}) || all(block{2} <= char(32))
   irisparser.error(0,fname,{},'variables');
end

% No block of transition equations found.
if isempty(block{7}) || all(block{7} <= char(32))
   irisparser.error(0,fname,{},'equations');
end

end

%**************************************************************************
% Subfunction chkkeywords_().
function chkkeywords_(c,fname)
   keyword = regexp(c,'!!|![\w:]+','match');
   [blk,ctrl,math] = iriskeywords();
   list = [blk,ctrl,math];
   nkeyword = length(keyword);
   valid = true([1,nkeyword]);
   for i = 1 : nkeyword
   	if ~any(strcmp(keyword{i},list))
         valid(i) = false;
      end
   end
   if any(~valid)
      % Invalid keyword(s).
      irisparser.error(1,fname,keyword(~valid));
   end
end
% End of subfunction chkkeywords_().

%**************************************************************************
% Subfunction chkallbut_().
% !allbut must be in all or none of log declaration blocks.
function chkallbut_(tokens,fname) 
   index = cellfun(@isempty,regexp(tokens,'!allbut','match','once'));
   if any(index) && any(~index)
      irisparser.error(52,fname);
   end
end
% End of subfunction chkallbut_().
