function data = bn(this,data,range,varargin)
% Type <a href="matlab: idoc model.bn">idoc model.bn</a> or click the link to get help on this function.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

% Parse required input arguments.
P = inputParser();
P.addRequired('m',@ismodel);
P.addRequired('data',@(x) isstruct(x) || iscell(x));
P.addRequired('range',@(x) isnumeric(x));
P.parse(this,data,range);

options = passvalopt('model.bn',varargin{:});

% Auto set the 'dtrends' option.
if ischar(options.dtrends)
   options.dtrends = ~options.deviation;
end

%**************************************************************************
% Function body.

tol = getrealsmall();
[ny,nx,nf,nb,ne,np,nalt] = size_(this);
nper = numel(range);

if options.contributions
   a1 = datarequest('alpha',this,data,range-1);
   e = datarequest('e',this,data,range);
   ncontributions = ne+1;
else
   a = datarequest('alpha',this,data,range);
   ncontributions = 1;
end
ndata = size(a,3);

% Cannot run |'contributions'| on mutliple params or data sets.
if options.contributions && (ndata > 1 || nalt > 1)
   error_(47,'<<matlab:idoc(''model.bn'') bn>>');
end

nloop = max([ndata,nalt,ncontributions]);

% Pre-allocate output data.
data = { ...
   nan([ny,nper,nloop]), ...
   nan([nx,nper,nloop]), ...
   zeros([ne,nper,nloop]), ...
   range(1):range(end), ...
};

[flag,nansolution] = isnan(this,'solution');
% Solution not available for some parameterisations.
if flag
   warning_(44,sprintf(' #%g',find(nansolution)));
end

repeat = ones([1,nper]);
nondiffstationary = [];

for iloop = 1 : nloop

   if nansolution(min([iloop,nalt]))
      continue
   end
   
   if iloop <= nalt
      Tfi = this.solution{1}(1:nf,:,iloop);
      Tai = this.solution{1}(nf+1:end,:,iloop);
      abseig = abs(ordeig(Tai));
      nunit = sum(abs(abseig - 1) <= tol);
      if ~iseye(Tai(1:nunit,1:nunit))
         nondiffstationary(end+1) = iloop;
      end
      Zi = this.solution{4}(:,:,iloop);
      % non-stationary xf and y
      %xfdiffuse = any(abs(Tfi(:,1:nunit)) > tol,2);
      %ydiffuse = any(abs(Zi(:,1:nunit)) > tol,2);
      % steady state for [xf;alpha]
      if ~options.deviation
         Kai = this.solution{3}(nf+1:end,1,iloop);
         abar = zeros([nb,1]);
         abar(nunit+1:end) = (eye(nb-nunit) - Tai(nunit+1:end,nunit+1:end)) \ Kai(nunit+1:end);
         abar = abar(:,repeat);
         Kfi = this.solution{3}(1:nf,1,iloop);
         Di = this.solution{6};         
         Kfi = Kfi(:,repeat);
         Di = Di(:,repeat);
      end
      if options.dtrends
         Wi = dtrendsrequest_('range',this,range,iloop);
      end
   end

   tmpindex = min([iloop,ndata]);
   if options.contributions
      a1i = a1(:,:,tmpindex);
      ei = ei(:,:,tmpindex);
      if ~options.deviation
         a1i = a1i - abar;
      end
   else
      ai = a(:,:,tmpindex);
      if ~options.deviation
         ai = ai - abar;
      end
   end
   
   %********************************************************************
   % Forward cumsum of stable alpha.
   
   acum = ...
      (eye(nb-nunit) - Tai(nunit+1:end,nunit+1:end)) \ ai(nunit+1:end,:);
   
   %********************************************************************
   % Beveridge Nelson for non-stationary variables.
   
   ai(1:nunit,:) = ai(1:nunit,:) + ...
      Tai(1:nunit,nunit+1:end)*acum;
   if options.deviation
      ai(nunit+1:end,:) = 0;
   else
      ai(nunit+1:end,:) = abar(nunit+1:end,:);
   end
   
   xf = Tfi*ai;
   y = Zi*ai;
   if ~options.deviation
      xf = xf + Kfi;
      y = y + Di;
   end
   if options.dtrends
      y = y + Wi;
   end
   
   data{1}(:,:,iloop) = y;
   data{2}(:,:,iloop) = [xf;ai];
   
end
% End of main loop.

% Parameterisations that are not difference-stationary.
if ~isempty(nondiffstationary)
   error_(70,sprintf(' #%g',nondiffstationary));
end

% Keep only current dates so that no pre-sample observations are added.
imagid = imag(this.solutionid{2});
data{2}(imagid ~= 0,:,:) = NaN;
data = alpha2xb_(this,data);

if strcmp(options.output,'dbase')
   data = dp2db(this,data);
end

end
% End of primary function.