function [flag,varargout] = chksstate(m,varargin)
% CHKSSTATE  Check equations in currently assigned steady state.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

options = passvalopt('model.chksstate',varargin{:});

%**************************************************************************
% Function body.

% Refresh dynamic links.
if options.refresh && ~isempty(m.refresh)
   m = refresh(m);
end

% Warning if some parameters are not assigned.
[flag,list] = isnan(m,'parameters');
if options.warning && flag
   warning_(2,list);
end

% warning if some steady states are not assigned
[flag,list] = isnan(m,'sstate');
if options.warning && flag
   warning_(35,list);
end

% warning if some log-lin variables have non-positive steady state
chklog_(m);

t = m.tzero;
mint = 1 - t;
if issparse(m.occur)
   nt = size(m.occur,2) / length(m.name);
else
   nt = size(m.occur,3);
end
maxt = nt - t;
nalt = size(m.assign,3);

flag = false([1,nalt]);
list = cell([1,nalt]);
mval = nan([sum(m.eqtntype <= 2),nalt]);
discrep = nan([sum(m.eqtntype <= 2),2,nalt]);

% Check two consecutive periods
% This is to detect both incorrect levels and growth rates.
tvec = mint : maxt+1;
for ialt = 1 : nalt
   x = trendarray_(m,1:length(m.name),tvec,true,ialt);
   x = shiftdim(x,-1);
   G = imag(m.assign(1,:,ialt));
   G(m.log & G == 0) = 1;
   G = G(:,:,ones([1,nt]));
   % Levels at time t.
   x1 = x(:,:,1:end-1);
   % Levels at time t+1.
   x2 = x(:,:,2:end);
   mval1 = ...
      vec(cellfun(@(fcn) fcn(x1,t,x1,G),m.eqtnF(m.eqtntype <= 2)));
   mval2 = ...
      vec(cellfun(@(fcn) fcn(x2,t,x2,G),m.eqtnF(m.eqtntype <= 2)));
   mval(:,ialt) = max(abs([mval1,mval2]),[],2);
   index = abs(mval(:,ialt)) <= options.tolerance;
   flag(ialt) = all(index == true);
   if nargout > 1 || options.error
      list{ialt} = m.eqtn(~index);
   end
   discrep(:,:,ialt) = [mval1,mval2];
end

if options.error && any(~flag)
   tmp = {};
   for i = find(~flag)
      for j = 1 : length(list{i})
         tmp{end+1} = i;
         tmp{end+1} = list{i}{j};
      end
   end
   error_(71,tmp);
end

if nalt == 1
   list = list{1};
end

if nargout == 2
   varargout{1} = list;
elseif nargout > 2
   varargout{1} = discrep;
   varargout{2} = list;
end

end
% End of primary function.