function d = dp2db(this,d,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.dp2db">idoc model.dp2db</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

template = tseries();

if iscell(d)
   range = d{4};
   d = dp2db_(d,'mean');
   d = extras_(d);
else
   range = d.mean_{4};
   d.mean = dp2db_(d.mean_,'mean');
   d.mean = extras_(d.mean);
   if isfield(d,'mse_')
      tolerance = getrealsmall('mse');
      range = d.mse_{4};
      d.std = dp2db_(d.mse_,'mse');
      d.std = extras_(d.std);
   end
end   

   %***********************************************************************
   % Nested function extras_().
   function d = extras_(d)
      % Add parameters to database.
      for i = find(this.nametype == 4)
         d.(this.name{i}) = permute(this.assign(1,i,:),[1,3,2]);
      end
      % Add comments to time series.
      for i = find(this.nametype <= 3)
         if isfield(d,this.name{i}) && istseries(d.(this.name{i}))
            d.(this.name{i}) = comment(d.(this.name{i}),this.namelabel{i});
         end
      end
   end
   % End of nested function extras_().
   
   %***********************************************************************
   % Nested function dp2db_().
   function b = dp2db_(p,type)
      b = struct();
      ismean = strcmp(type,'mean');
      realid = real(this.solutionid{1});
      % Measurement variables.
      ylist = this.name(this.nametype == 1);
      for i = 1 : length(realid)
         if ismean
            y = permute(p{1}(i,:,:),[2,3,1]);
         else
            y = permute(p{1}(i,i,:,:),[3,4,2,1]);
            y = mse2std_(y);
         end
         if this.log(realid(i))
            y = exp(y);
         end
         b.(ylist{i}) = replace(template,y,range(1));
      end
      % Transition variables.
      realid = real(this.solutionid{2});
      nx = length(this.solutionid{2});
      imagid = imag(this.solutionid{2});
      maxlag = -min(imagid);
      if ismean
         ndata = size(p{2},3);
         X = [nan([nx,maxlag,ndata]),p{2}];
      else
         ndata = size(p{2},4);
         X = cat(3,nan([nx,nx,maxlag,ndata]),p{2});
      end
      startdate = range(1) - maxlag;
      t = maxlag + 1;
      for i = find(imagid < 0)
         parentRow = realid == realid(i) & imagid == 0;
         if ismean
            X(parentRow,t+imagid(i),:) = p{2}(i,1,:);
         else
            X(parentRow,parentRow,t+imagid(i),:) = p{2}(i,i,1,:);
         end
      end
      for i = find(imagid == 0)
         if ismean
            x = permute(X(i,:,:,:),[2,3,1]); 
         else
            x = permute(X(i,i,:,:),[3,4,2,1]);
            x = mse2std_(x);
         end
         if this.log(realid(i))
            x = exp(x);
         end
         b.(this.name{realid(i)}) = replace(template,x,startdate);
      end
      % Shocks.
      elist = this.name(this.nametype == 3);
      for i = 1 : length(elist)
         if ismean
            e = permute(p{3}(i,:,:,:),[2,3,1]);
         else
            e = permute(p{3}(i,i,:,:),[3,4,2,1]);
         end
         b.(elist{i}) = replace(template,e,range(1));
      end
   end
   % End of nested function dp2db_().
   
   %***********************************************************************
   % Nested function mse2std_().
   function x = mse2std_(x)
      x(abs(x) < tolerance) = 0;
      x = sqrt(x);
   end
   % End of nested function mse2std_().   
   
end
% End of primary function.