function [X,Y,list,x,y] = fevd(this,time,varargin)
%
% To get help for this IRIS function
% * type <a href="matlab: idoc model.fevd">idoc model.fevd</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox 2008/05/05.
% Copyright 2007-2008 Jaromir Benes.

default = {
   'select',Inf,...
};
options = passopt(default,varargin{:});

if ischar(options.select)
   options.select = charlist2cellstr(options.select);
end

% tell whether time is nper or range
if length(time) == 1 && round(time) == time && time > 0
   range = 1 : time;
else
   range = time(1) : time(end);
end
nper = length(range);

%**************************************************************************
% Function body.

[ny,nx,nf,nb,ne,np,nalt] = size_(this);
X = nan([ny+nx,ne,nper,nalt]);
Y = nan([ny+nx,ne,nper,nalt]);

% compute FEVD for all solved parameterisations
[flag,index] = isnan(this,'solution');
for ialt = find(~index)
   [T,R,K,Z,H,D,Za,Omega] = sspace_(this,ialt,false);
   [X(:,:,:,ialt),Y(:,:,:,ialt)] = ...
      time_domain.fevd(T,R,K,Z,H,D,Za,Omega,nper);
end

% Some solution(s) not available.
if flag
   warning_(44,sprintf(' #%g',find(index)));
end

% Write list of variables in rows and columns.
if nargout > 2 || iscellstr(options.select)
   list = cell([1,2]);
   list{1} = [vector_(this,'y'),vector_(this,'x')];
   list{2} = vector_(this,'e');
end

% Select variables.
if iscellstr(options.select)
   [X,index,list,notfound] = select(X,list,options.select);
   if nargout > 1
      Y = Y(index{1},index{2},:,:);
   end
   if ~isempty(notfound)
      warning_(1,notfound);
   end
end

if nargout > 3
   % Select only contemporaneous variables.
   id = [this.solutionid{1:2}];
   comments = this.name(this.nametype == 3);
   comments = comments(1,:,ones([1,nalt]));
   x = struct();
   for i = find(imag(id) == 0)
      name = this.name{id(i)};
      x.(name) = tseries(range,permute(X(i,:,:,:),[3,2,4,1]),comments);
   end
   for j = find(this.nametype == 4)
      x.(this.name{j}) = permute(this.assign(1,j,:),[1,3,2]);
   end
end

if nargout > 4
   y = struct();
   for i = find(imag(id) == 0)
      name = this.name{id(i)};
      y.(name) = tseries(range,permute(Y(i,:,:,:),[3,2,4,1]),comments);
   end
   for j = find(this.nametype == 4)
      y.(this.name{j}) = permute(this.assign(1,j,:),[1,3,2]);
   end
end

end
% End of primary function.
