function [this,data,se2,delta,pe,F] = filter(this,data,range,varargin)
% <a href="matlab: edit model/filter">FILTER</a>  Kalman smoother and estimator of out-of-likelihood parameters.
% To get help for this IRIS function
% * type <a href="matlab: idoc model.filter">idoc model.filter</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and browse The IRIS Toolbox documentation in the Contents pane.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

P = inputParser();
P.addRequired('model',@ismodel);
P.addRequired('data',@(x) isstruct(x) || iscell(x));
P.addRequired('range',@isnumeric);
P.parse(this,data,range);

% KALMAN_ specific options.
[loglikoptions,varargin] = loglikopt_(this,range,'t',varargin{:});
% FILTER specific options.
filteroptions = passvalopt('model.filter',varargin{:});

%**************************************************************************
% Function body.

% Get observables and determine the format of output data.
input = dataformat(data,true);
output = outputformat(input,filteroptions);
data = datarequest('y',this,data,range);

% Call Kalman filter.
if strcmpi(filteroptions.data,'predict')
   [mloglik,se2,F,pe,delta,ans,ans,data] = ...
      kalman_(this,data,range,[],loglikoptions);
else
   [mloglik,se2,F,pe,delta,ans,ans,ans,data] = ...
      kalman_(this,data,range,[],loglikoptions);
end

% Make sure model, out-of-lik params and se2 all have the same number of
% alternative parameterisations.
if loglikoptions.relative
   nout = max([size(this.assign,3),size(delta,3),length(se2)]);
else
   nout = max([size(this.assign,3),size(delta,3)]);
end   
if size(this.assign,3) < nout
   this = nalter(this,nout);
end
if ~isempty(delta) && size(delta,3) < nout
   delta(:,:,end+1:nout) = delta(:,:,end);
end
if length(se2) < nout
   se2(end+1:nout) = se2(end);
end

% Keep track of whether refresh is needed.
dorefresh = false;

% Assign out-of-lik params in model object.
if ~isempty(delta)
   plist = this.name(loglikoptions.outoflik);
   for i = 1 : length(plist)
      index = strcmp(plist{i},this.name);
      this.assign(1,index,:) = delta(1,i,:);
   end
   dorefresh = true;
end

% Scale std devs by estimated factor in model object.
if loglikoptions.relative
   this = stdscale(this,sqrt(se2));
   dorefresh = true;
end

if dorefresh
   this = refresh(this);
end

if strcmpi(output,'dbase')
   [F,pe,delta,ans,ans,data] = ...
      outputdbase_(this,F,pe,delta,[],[],data,range,loglikoptions);
else
   F = F{1};
end

end
% End of primary function.
