function m = loadobj(m)

% Convert array of occurences to sparse matrix.
if ~issparse(m.occur)
   m.occur = sparse(m.occur(:,:));
end

% Convert symbolic-derivative chars to function handles.
if isstruct(m) && ~isfield(m,'deqtnF')
   m.deqtnF = {};
else
   for i = 1 : length(m.deqtnF)
      for j = 1 : length(m.deqtnF{i})
         if ischar(m.deqtnF{i}{j}) && ~isempty(m.deqtnF{i}{j})
            m.deqtnF{i}{j} = ...
               regexprep(m.deqtnF{i}{j},'^@\([^\)]*\)','','once');
            if m.eqtntype(i) <= 2
               preamble = '@(x,t,L,G)';
            else
               preamble = '@(x,t,ttrend)';
            end
            m.deqtnF{i}{j} = eval([preamble,m.deqtnF{i}{j}]);
         end
      end
   end
end

% Convert full-equation chars to function handles.
for i = 1 : length(m.eqtnF)
   if ischar(m.eqtnF{i}) && ~isempty(m.eqtnF{i})
      m.eqtnF{i} = regexprep(m.eqtnF{i},'^@\([^\)]*\)','','once');
      if m.eqtntype(i) <= 2
         preamble = '@(x,t,L,G)';
      else
         preamble = '@(x,t,ttrend)';
      end
      m.eqtnF{i} = eval([preamble,m.eqtnF{i}]);
   end
end

% Add empty dtrends equations if missing.
ny = sum(m.nametype == 1);
if ny > 0 && sum(m.eqtntype == 3) == 0
   m.eqtn(end+(1:ny)) = {''};
   m.eqtnS(end+(1:ny)) = {''};
   m.eqtnF(end+(1:ny)) = {@(x,t,ttrend)0};
   m.eqtnlabel(end+(1:ny)) = {''};
   m.eqtntype(end+(1:ny)) = 3;
   m.occur(end+(1:ny),:) = false;
end

% Add empty dynamic links if missing.
if isstruct(m) && ~isfield(m,'refresh')
   nname = sum(m.nametype <= 4);
   m.eqtn(end+(1:nname)) = {''};
   m.eqtnS(end+(1:nname)) = {''};
   m.eqtnF(end+(1:nname)) = {[]};
   m.eqtnlabel(end+(1:nname)) = {''};
   m.eqtntype(end+(1:nname)) = 4;
end

% Add unknown shock types if missing.
if isstruct(m) && ~isfield(m,'shocktype')
   m.shocktype = nan([1,sum(m.nametype == 3)]);
end
if any(isnan(m.shocktype))
   m = shocktype_(m);
end

if isstruct(m)
   m = model(m);
end

end