function varargout = loglik(this,data,range,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.loglik">idoc model.loglik</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

% <a href="model/loglik">LOGLIK</a>  Evaluate likelihood function in frequency domain.
%
% Syntax:
%   [mloglik,se2] = loglik(this,dpack,'domain','f',...)   
%   [mloglik,se2] = loglik(this,dbase,range,'domain','f'...)
% Output arguments:
%   mloglik [ numeric ] Minus log-likelihood.
%   se2 [ numeric ] Estimated common variance factor (if option 'relative' is true).
%   this [ model ] Model.
%   dpack [ cell | numeric ] Input datapack or data array.
%   dbase [ cell | numeric ] Input <a href="databases.html">database</a>.
%   range [ numeric ] Time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.
% <a href="options.html">Optional input arguments:</a>
%   'band' [ numeric | <a href="default.html">[2,Inf]</a> ] Band of periodicities used to evaluate likelihood.
%   'domain' [ <a href="default.html">'time'</a> | 'freq' ] Evaluate likelihood function in time or frequency domain.
%   'exclude' [ cellstr | <a href="default.html">empty</a> ] List of measurement variables to be excluded from likelihood function.
%   'relative' [ <a href="default.html">true</a> | false ] Scale MSE matrices by estimated variance factor.
%   'zero' [ true | <a href="default.html">false</a> ] Include zero frequency (if zero lies in 'band').
%
% Nota bene:
%   <a href="matlab: help model/filte"r>MODEL/FILTER</a>, <a href="matlab: help model/loglik">MODEL/LOGLIK</a> (time domain) and <a href="matlab: help model/kalmanf">MODEL/KALMAN</a> differ only in their output arguments.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

persistent loglikdata loglikrange options loglikoptions output;

% If loglik(m) is called without any further input arguments, the ones
% passed in last will be used.
if nargin == 1
   if isempty(loglikdata)
      error_(73);
   end
else
   % Get array of measurement variables.
   loglikdata = datarequest('y',this,data,range);
   loglikrange = range;
   % Process |loglik| options.
   [options,varargin] = passvalopt('model.loglik',varargin{:});
   % Process |loglikt_| and |loglikf_| options.
   loglikoptions = loglikopt_(this,loglikrange,options.domain,varargin{:});
   if strncmpi(options.domain,'t',1)
      input = dataformat(data);
      output = outputformat(input,options);
   end      
end

%**************************************************************************
% Function body.

if strncmpi(options.domain,'f',1)
   % Frequency domain.
   [I,freq,delta] = fourierdata_(this,loglikdata,loglikoptions);
   [varargout{1:2}] = loglikf_(this,I,freq,delta,loglikoptions);
else
   % Time domain (Kalman filter).
   [varargout{1:nargout}] = ...
      kalman_(this,loglikdata,loglikrange,[],loglikoptions);
   if nargout <= 2
      return
   end
   varargout(end+1:9) = {[]};
   if strcmpi(output,'dbase')    
      [varargout{[3,4,5,6,8,9]}] = ...
         outputdbase_(this,varargout{[3,4,5,6,8,9]},loglikrange,loglikoptions);
   end
end

if ~options.persist
   loglikdata = [];
   loglikrange = [];
   output = [];
   options = [];
   loglikoptions = [];
end

end
% End of primary function.