function m = nalter(m,n)
% <a href="matlab: edit model/nalter">NALTER</a>  Query, increase or decrease number of alternative parameterizations.
%
% Syntax:
%   n = nalter(m)
%   m = nalter(m,n)
% Output arguments for syntax
%   n [ numeric ] Current number of alternative parameterisations.
%   m [ model ] Model with increased or decreased number of alternative parameterisations.
% Required input arguments:
%   m [ model ] Model.
%   n [ numeric ] Desired number of alternative parameterisations.
%
%   The number of parameterisations is changed either by copying the last one (when increasing)
%   or chopping off the last ones (when decreasing).

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body..

nalt = length(m);

if nargin == 1
   m = nalt;
   return
end

if n > nalt
   % Expand the number of parameterisations by copying the last one:
   % Call m(end+1:n) = m(end);
   s1 = struct();
   s1.type = '()';
   s1.subs = {nalt+1:n};
   s2 = struct();
   s2.type = '()';
   s2.subs = {nalt};
   m = subsasgn(m,s1,subsref(m,s2));
else
   % Reduce the number of parameterisations by cutting off the last ones:
   % Call m(n+1:end) = [];
   s = struct();
   s.type = '()';
   s.subs = {n+1:nalt};
   m = subsasgn(m,s,[]);
end

end
% End of primary function.