function [options,varargin] = loglikopt_(this,range,domain,varargin)
% LOGLIKTOPT  Optional input arguments for LOGLIKT2_ and LOGLIKF_.

if strncmp(domain,'t',1)
   % Time domain options.
   [options,varargin] = passvalopt('model.kalman',varargin{:});
elseif strncmp(domain,'f',1)
   % Freq domain options.
   [options,varargin] = passvalopt('model.loglikf',varargin{:});
end

if strncmp(domain,'t',1)
   if ischar(options.dtrends)
      options.dtrends = ~options.deviation;
   end
end

% Out-of-lik parameters.
if strncmp(domain,'t',1)
   if isempty(options.outoflik)
      options.outoflik = [];
   else
      if ischar(options.outoflik)
         options.outoflik = charlist2cellstr(options.outoflik);
      end
      options.outoflik = options.outoflik(:)';
      offset = sum(this.nametype < 4);
      index = offset + ...
         findnames(this.name(this.nametype == 4),options.outoflik);
      isnanindex = isnan(index);
      if any(isnanindex)
         % Unknown parameter names.
         error_(68,options.outoflik(isnanindex));
      end
      options.outoflik = index;
   end
end

ny = sum(this.nametype == 1);
% Measurement variables exluded from likelihood function.
if isempty(options.exclude)
   options.exclude = false([ny,1]);
elseif islogical(options.exclude)
   options.exclude = vec(options.exclude);
   if length(options.exclude) < ny
      options.exclude(end+1:ny) = false;
   elseif length(options.exclude) > ny
      options.exclude = options.exclude(1:ny);
   end
elseif ~isempty(options.exclude) && ~islogical(options.exclude)
   tmpindex = [];
   if ischar(options.exclude)
      options.exclude = charlist2cellstr(options.exclude);
   end
   if iscellstr(options.exclude)
      tmpindex = findnames(this.name(this.nametype == 1),options.exclude(:)');
      if any(isnan(tmpindex))
         warning_(1,options.exclude(isnan(tmpindex)));
         tmpindex = tmpindex(~isnan(tmpindex));
      end
   end
   options.exclude = false([ny,1]);
   options.exclude(tmpindex) = true;
end

% Convert user-supplied std database to varvec. Add one period of
% pre-sample NaN variances (these are always given by the model object).
if ~isempty(range)
   [options.varvec,options.anyvarvec,options.varvecindex] = ...
      std2varvec_(this,options.std,range,1);
end

% Objective function.
if strncmp(domain,'t',1)
   switch lower(options.objective)
      case 'prederr'
         % Weighted prediction errors.
         options.objective = 2;
         if isempty(options.weighting)
            options.weighting = sparse(eye(ny));
         elseif numel(options.weighting) == 1
            options.weighting = sparse(eye(ny)*options.weighting);
         elseif any(size(options.weighting) == 1)
            options.weighting = sparse(diag(options.weighting));
         end
         if size(options.weighting,1) ~= ny || ndims(options.weighting) > 2
            error_(69);
         end
      otherwise
         % Minus log likelihood.
         options.objective = 1;
   end
end

end
% End of primary function.