function [F,pe,delta,Pdelta,pred,smooth] = ...
   outputdbase_(this,F,pe,delta,Pdelta,pred,smooth,range,options)

% The IRIS Toolbox 2009/10/29.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

ny = size(F{1},1);
nper = length(range);
ndata = size(F{1},4);
template = tseries();

% MSE matrices and std devs for prediction errors.
if ~isempty(F)
   tmpf = F{1};
   F = struct();
   F.mse = tmpf;
   for i = 1 : ny
      x = permute(tmpf(i,i,:,:),[3,4,1,2]);
      index = x <= 0;
      x(index) = 0;
      x(~index) = sqrt(x(~index));
      if this.log(i)
         x = exp(x);
      end
      F.std.(this.name{i}) = ...
         replace(template,x,range(1),this.namelabel{i});
   end
end

% Prediction errors.
if ~isempty(pe)
   tmppe = pe;
   ref = cell([1,ndims(tmppe)-1]);
   ref(:) = {':'};
   pe = struct();
   for i = find(this.nametype == 1)
      if this.log(i)
         tmppe(i,ref{:}) = exp(tmppe(i,ref{:}));
      end
      x = permute(tmppe(i,ref{:}),[2,3,4,1]);
      pe.(this.name{i}) = ...
         replace(template,x,range(1),this.namelabel{i});
   end
end

% Out-of-lik parameters.
if ~isempty(delta)
   tmpdelta = delta;
   delta = struct();
   plist = this.name(options.outoflik);
   for i = 1 : numel(plist)
      delta.mean.(plist{i}) = permute(tmpdelta(1,i,:),[1,3,2]);
   end
   if ~isempty(Pdelta)
      delta.mse = Pdelta;
      for i = 1 : numel(plist)
         delta.std.(plist{i}) = permute(sqrt(Pdelta.mse(i,i,:)),[1,3,2]);
      end
      Pdelta = [];
   end
end

% Prediction data.
if ~isempty(pred)
   pred = dp2db(this,pred);
end

% Smoother data.
if ~isempty(smooth)
   smooth = dp2db(this,smooth);
end

end
% End of primary function.         