function [m,npath,eigval] = solve(m,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.solve">idoc model.solve</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox 2008/10/20.
% Copyright 2007-2009 Jaromir Benes.

options = passvalopt('model.solve',varargin{1:end});

if options.forward > 0 && options.expand == 0
   options.expand = options.forward;
end

%**************************************************************************
% Function body.

% No parameterisations.
if isempty(options.alt) || (islogical(options.alt) && ~any(options.alt))
   return
end

if isnumeric(options.alt) && any(isinf(options.alt))
   options.alt = 1 : size(m.assign,3);
end

% Refresh dynamic links.
if options.refresh && ~isempty(m.refresh)
   m = refresh(m,options.alt);
end

if options.warning
   % Warning if some parameters are not assigned.
   [flag,list] = isnan(m,'parameters',options.alt);
   if flag
      warning_(2,list);
   end
   % Warning if some log-lin variables have non-positive steady state.
   chklog_(m,options.alt);
end

[m,npath] = solve_(m,options.expand, ...
   options.tolerance,options.alt,options.select,options.system);

if options.warning && any(npath ~= 1) && iswarning('model')
   % No stable solution.
   index = npath == 0;
   if any(index)
      warning_(26,{sprintf(' #%g',find(index))});
   end
   % Infinitely many stable solutions.
   index = isinf(npath);
   if any(index)
      warning_(27,{sprintf(' #%g',find(index))});
   end
   % Complex derivatives.
   index = imag(npath) ~= 0;
   if any(index)
      warning_(29,{sprintf(' #%g',find(index))});
   end
   % NaN derivatives.
   index = isnan(npath);
   if any(index)
      warning_(33,{sprintf(' #%g',find(index))});
   end
   % Singularity in state space.
   index = npath == -1;
   if any(index)
      warning_(32,{sprintf(' #%g',find(index))});
   end
end

if nargout > 2
   eigval = m.eigval;
end

end
% End of function body.