function [s,range,eselect] = srf(m,time,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.srf">idoc model.srf</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

options = passvalopt('model.srf',varargin{:});

if ischar(options.select)
   options.select = charlist2cellstr(options.select);
end

% Tell whether time is nper or range.
if length(time) == 1 && round(time) == time && time > 0
   range = 1 : time;
else
   range = min(time) : max(time);
end
nper = length(range);

%**************************************************************************
% Function body..

[ny,nx,nf,nb,ne,np,nalt] = size_(m);

% Select shocks.
if ischar(options.select) || iscellstr(options.select)
   select = findnames(options.select,m.name(m.nametype == 3));
   select = ~isnan(select);
else
   select = true([1,ne]);
end
nselect = sum(select);

shksize = nan([1,ne,nalt]);
if strcmp(options.size,'std')
   shksize = m.assign(1,end-sum(m.nametype == 3)+1:end,:);
else
   shksize(1,select,:) = options.size;
end
shksize = shksize(:,select,:);

Phi = nan([ny+nx,nselect,nper,nalt]);
[flag,index] = isnan(m,'solution');
for ialt = find(~index)
   [T,R,K,Z,H,D,U,Omega] = sspace_(m,ialt,false);
   Phi(:,:,:,ialt) = time_domain.srf( ...
      T,R(:,select),K,Z,H(:,select),D,U,Omega(select,select), ...
      nper,shksize(1,:,ialt));
end

% Report solutions not available.
if flag
   warning_(44,sprintf(' #%g',find(index)));
end

% Create output database.
s = struct();

% Create time series for measurement and transition variables.
id = [m.solutionid{1:2}];
realid = real(id);
imagid = imag(id);
elist = m.name(m.nametype == 3);
eselect = elist(select);
comment = repmat(eselect,[1,1,nalt]);
template = tseries(range,zeros([size(Phi,3),size(Phi,2),size(Phi,4)]),comment);
for i = find(imagid == 0)
   x = permute(Phi(i,:,:,:),[3,2,4,1]);
   % Find max lag of this variable and fill in pre-sample zeros.
   % Add one pre-sample zero for each variable whatever its max lag.
   maxlag = -min(imagid(real(id) == realid(i))) + 1;
   tmpsize = size(x);
   x = x(end:-1:1,:);
   x(end+(1:maxlag),:) = 0;
   x = x(end:-1:1,:);
   x = reshape(x,[tmpsize(1)+maxlag,tmpsize(2:end)]);
   if options.log && m.log(realid(i))
      x = exp(x);
   end
   s.(m.name{realid(i)}) = replace(template,x,range(1)-maxlag,comment);
end

% Create time series for shocks.
for i = 1 : ne
   s.(elist{i}) = template;
end
for i = 1 : nselect
   s.(eselect{i})(1,i,:) = shksize(1,i,:);
end

% Add parameter database.
for i = find(m.nametype == 4)
   s.(m.name{i}) = permute(m.assign(1,i,:),[1,3,2]);
end

end
% End of primary function.