function [T,R,K,Z,H,D,U,Omega,list,altlist] = sspace(m,varargin)
   % SSPACE  State-space representation of the model.

   % The IRIS Toolbox.
   % Copyright 2007-2010 Jaromir Benes.

nalt = size(m.solution{1},3);
if nargin > 1 && isnumeric(varargin{1})
   alt = varargin{1}(:).';
   varargin(1) = [];
else
   alt = 1 : nalt;
end

options = passvalopt('model.sspace',varargin{:});

%**************************************************************************
% Function body.

[T,R,K,Z,H,D,U,Omega] = sspace_(m,alt,true);
ny = size(Z,1);
nx = size(T,1);

if ~options.triangular
   % T <- U*T/U;
   % R <- U*R;
   % K <- U*K;
   % Z <- Z/U;
   % U <- eye
   [ny,nx,nf,nb,ne,np,nalt] = size_(m);
   for i = 1 : length(alt)
      T(:,:,i) = T(:,:,i) / U(:,:,i);
      T(nf+1:end,:,i) = U(:,:,i)*T(nf+1:end,:,i);
      R(nf+1:end,:,i) = U(:,:,i)*R(nf+1:end,:,i);
      K(nf+1:end,:,i) = U(:,:,i)*K(nf+1:end,:,i);
      Z(:,:,i) = Z(:,:,i) / U(:,:,i);
      U(:,:,alt(i)) = eye(size(U));
   end
end

if options.removeinactive
   if length(alt) == 1
      index = diag(Omega) ~= 0;
      R = reshape(R,[nx,ne,size(R,2)/ne]);
      R = R(:,index,:);
      R = R(:,:);
      H = H(:,index);
      Omega = Omega(index,index);
      if nargout > 8
         list{3} = list{3}(index);
      end
   else
      warning_(47);
   end
end

if nargout > 8
   list = cell([1,3]);
   list{1} = vector_(m,'y');
   list{2} = vector_(m,'x');
   list{3} = vector_(m,'e');
   if nargout > 9
      altlist = list([2,1,3]);
   end
end

end
% End of primary function.