function varargout = subsref(this,s)
% <a href="matlab: edit model/subsref">SUBSREF</a>  Subscripted reference to model objects.
%
% Syntax:
%   m(index)        (1)
%   m.name          (2)
%   m.name(index)   (3)
% Required input arguments:
%   m [ model ] Model.
% Required input arguments for syntax (1) and (3):
%   index [ logical | numeric | empty] Index of requested parameterisations.
% Required input arguments for syntax (2) and (3):
%   name [ char ] Name of requested parameter or variable.
%
% Nota bene:
%   Syntax (1) produces a model object with the requested subset of parameterisations.
%   Syntax (2) produces a vector of values currently assigned to the requested parameters, or a vector
%   of steady-state values for the requested variable.
%   Syntax (3) produces the same as Syntax (2) for the requested subset of parameterisations.

% The IRIS Toolbox 2009/11/24.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

nalt = size(this.assign,3);

% Use |chksubsref| to return always either of the following:
% * this(index).name -> this.name(index)
% * this(index1).name(index2) -> this.name(index1(index2))
% for integer index nd replaces 'end' & ':'.
if any(strcmp(s(1).type,{'()','{}'}))
   if ischar(s(1).subs{1})
      [varargout{1:length(s(1).subs)}] = get(this,s(1).subs{:});
      return
   end
   s = chksubsref_(s,nalt);
end

% Fast reference |m.name|.
if s(1).type == '.'
   index = strcmp(s(1).subs,this.name);
   if ~any(index)
      error_(72,s(1).subs);
   end
   varargout{1} = permute(this.assign(1,index,:),[1,3,2]);
   if numel(s) > 1
      varargout{1} = subsref(varargout{1},s(2:end));
   end
elseif any(strcmp(s(1).type,{'()','{}'}))
   % m(index) or m{index}
   index = s(1).subs{1};
   if any(index > nalt)
      error('Index exceeds number of parameterisations.');
   end
   refer_();
   varargout{1} = this;
end

% End of function body.

   %***********************************************************************
   % Nested function refer_().
   function refer_()
      this.assign = this.assign(1,:,index);
      this.eigval = this.eigval(1,:,index);
      this.icondix = this.icondix(1,:,index);
      for i = 1 : length(this.solution)
         this.solution{i} = this.solution{i}(:,:,index);
      end
      for i = 1 : length(this.expand)
         this.expand{i} = this.expand{i}(:,:,index);
      end
   end
   % End of nested function refer_().

end
% End of primary function.