function [m,QQ,WW,Qlist,Wlist,count,YY,LL,discrep] = tcorule(m,uname,ueqtn,RR,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc model.tcorule">idoc model.tcorule</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

options = passvalopt('model.tcorule',varargin{:});

if ischar(uname)
   uname = {uname};
end

if ischar(ueqtn)
   ueqtn = {ueqtn};
end

%**************************************************************************
% Function body.

% N.B.: Constants and measurement errors ignored

realsmall = getrealsmall();
ny = length(m.systemid{1});
nx = length(m.systemid{2});
nf = sum(imag(m.systemid{2}) >= 0);
nb = nx - nf;
ne = length(m.systemid{3});
nalt = size(m.assign,3);

nweight = size(RR,3);
if size(RR,2) == 1
   RR = permute(RR,[2,1,3]);
end

% Size of weighting matrix/vector == number of measurement variables.
if size(RR,2) ~= ny || ~any(size(RR,1) == [1,ny])
   error_(58);
end

% At least one non-zero element in weighting matrix/vector.
if any(all(all(RR == 0,1),2))
   error_(59);
end

if options.warning
   % Some parameters are not assigned.
   [flag,list] = isnan(m,'parameters');
   if flag
      warning_(2,list);
   end
   % Some log-lin variables have non-positive steady state.
   chklog_(m);
end

unameix = findnames(m.name,uname);
ueqtnix = findnames(m.eqtnlabel(m.nametype == 2),ueqtn);
% All names found?
index = isnan(unameix);
if any(index)
   error_(60,uname(index));
end
% All equation labels found?
index = isnan(ueqtnix);
if any(index)
   error_(61,ueqtn(index));
end
% Number of instruments.
nu = length(unameix);
if length(ueqtnix) ~= nu
   error_(62);
end

nloop = max([nalt,nweight]);
m = alter(m,nloop);
count = zeros([1,nloop]);
discrep = nan([1,3,nloop]);
notconverged = [];
display = options.display > 0 ...
         && ~isnan(options.display) ...
         && ~isinf(options.display);
      
for iloop = 1 : nloop
   
   if display
      fprintf('Parameterisation #%g\n',iloop);
   end
   
   if iloop <= nweight
      R = RR(:,:,iloop);
      if size(R,1) == 1
         % Input is a weighting vector.
         Rdiag = R(:).';
         R = diag(R);
         isrdiag = true;
      else
         % Input is a weighting matrix.
         Rdiag = diag(R).';
         isrdiag = all(all((diag(Rdiag) - R) == 0));
         % RR must be symmetric.
         if maxabs(triu(R) - transpose(tril(R))) > realsmall
            error_(63);
         end
      end
   end
   
   % Number of alt has been already expanded to nloop.
   % Fetch unsolved system matrices.
   eqselect = eqselect_(m,iloop);
   eqselect(m.eqtntype > 2) = false;
   [ans,deriv] = deriv_(m,eqselect,iloop);
   [ans,sys] = system_(m,deriv,eqselect,iloop);
   % Drop the current policy rule from the equations.
   sys.A{2}(ueqtnix,:) = 0;
   sys.B{2}(ueqtnix,:) = 0;
   sys.E{2}(ueqtnix,:) = 0;
   sys.K{2}(ueqtnix,:) = 0;
   % Add the policy instrument and its identity to the equations.
   for i = 1 : length(unameix)
      uix = real(m.systemid{2}) == unameix(i) & imag(m.systemid{2}) == -1;
      if any(uix)
         % Time t policy instrument is a predermined variable.
         sys.A{2}(ueqtnix(i),uix) = -1;
      else
         uix = real(m.systemid{2}) == unameix(i) & imag(m.systemid{2}) == 0;
         % Time t policy instrument is a non-predermined variable.
         sys.B{2}(ueqtnix(i),uix) = -1;
      end
   end
   
   % A*[xf+;xb] + B*[xf;xb-] + C*e + D*u = 0
   A = sys.A{2};
   B = sys.B{2};
   C = sys.E{2};
   D = sparse(zeros([nx,nu]));
   D(ueqtnix,1:nu) = eye(nu);
   
   % G*y + H*xb + J*e = 0
   G = sys.A{1};
   H = sys.B{1};
   % J = sys.E{1};
   
   %{
if m.linear
  % add constant
  A(end+1,end+1) = 1;
  B(end+1,end+1) = -1;
  B(1:end-1,end) = sys.K{2}(:,1);
  C(end+1,:) = 0;
  D(end+1,:) = 0;
  H(:,end+1) = sys.K{1}(:,1);
  nx = nx + 1;
  nb = nb + 1;
end
   %}
   
   A1 = A(:,1:nf);
   A2 = A(:,nf+1:end);
   B1 = B(:,1:nf);
   B2 = B(:,nf+1:end);
   
   Gi = inv(full(G));
   B2CD = full([B2,C,D]);

   if iloop == 1 || options.reset
      % E(xf) = Z*xb;
      Z = options.initexp([nf,nb]);
      % Value = y'*R*y + beta*xb'*P*xb.
      P = zeros(nb);
      % Policy function u = Q*[xb-;e].
      Q = inf([nu,nb+ne]);
   end
   
   count(iloop) = 0;
   
   if isrdiag
      rnonzero = Rdiag ~= 0;
   else
      rnonzero = any(R ~= 0,1);
   end
   
   iterate = count(iloop) < options.maxiter;
   
   warningStatus = warning();
   warning('off','MATLAB:nearlySingularMatrix');
   while iterate
      
      % Step I
      % find [xf;xb] = E*[xb-;e;u]
      if options.ginverse
         E = ginverse(full(-[B1,A1*Z+A2])) * B2CD;
      else
         lastwarn('');
         E = full(-[B1,A1*Z+A2]) \ B2CD;
         if ~isempty(lastwarn())
            E = ginverse(full(-[B1,A1*Z+A2])) * B2CD;
         end
      end
      E1 = E(1:nf,:);
      E11 = E1(:,1:nb);
      E12 = E1(:,nb+(1:ne));
      E13 = E1(:,nb+ne+(1:nu));
      E2 = E(nf+1:end,:);
      E21 = E2(:,1:nb);
      E22 = E2(:,nb+(1:ne));
      E23 = E2(:,nb+ne+(1:nu));
      
      % Step II
      % find y = K*[xb-;e;u]
      K = -Gi * full([H*E21,H*E22,H*E23]); % J (measurement errors) ignored
      
      % Step III
      % convert
      %    y'*R*y + beta*xb'*P*xb
      % to
      %    [xb-;e;u]'*Rtilda*[xb-;e;u]
      % where
      %    Rtilda = [L,M;M',N]
      %    Rtilda = K'*R*K + beta*E2'*P*E2
      Rtilda = options.beta*transpose(E2)*P*E2;
      if isrdiag
         Kt_R = zeros(nb+ne+nu);
         Kt_R(:,rnonzero) = transpose(K(rnonzero,:)) .* Rdiag(ones([1,nb+ne+nu]),rnonzero);
         Rtilda = Rtilda + Kt_R(:,rnonzero)*K(rnonzero,:);
      else
         Rtilda = Rtilda + transpose(K(rnonzero,:))*R(rnonzero,rnonzero)*K(rnonzero,:);
      end
      L = Rtilda(1:nb+ne,1:nb+ne);
      M = Rtilda(1:nb+ne,nb+ne+(1:nu));
      N = Rtilda(nb+ne+(1:nu),nb+ne+(1:nu));
      
      % Step IV
      % Minimise
      %    [xb-;e;u]'*[L,M;M',N]*[xb-;e;u]
      % s.t. transition equations
      % and solve for u
      %    u = Q*[xb-;e]
      Q0 = Q;
      lastwarn('');
      Q = -N \ transpose(M);
      if ~isempty(lastwarn())
         Q = -pinv(N) * transpose(M);
      end
      
      % Step V
      % find Y such that
      %    [xb-;e]'*Y*[xb-;e] == y'*R*y + beta*xb'*P*xb
      % with
      %    u = Q*[xb-;e]
      % and update P
      MQ = M*Q;
      Y = L + MQ + transpose(MQ) + transpose(Q)*N*Q;
      P0 = P;
      P = Y(1:nb,1:nb);
      
      % Step VI
      % find
      %    xf = S*[xb-;e]
      % and update Z
      S1 = [E11,E12] + E13*Q;
      Z0 = Z;
      Z = S1(:,1:nb);

      count(iloop) = count(iloop) + 1;
      converged = all(abs(Q(:)-Q0(:)) <= options.tolrule) && all(abs(P(:)-P0(:)) <= options.tolvalue) && all(abs(Z(:)-Z0(:)) <= options.tolexp);
      iterate = ~converged && count(iloop) < options.maxiter;
      if display && count(iloop) == 1
         fprintf('%10s%15s%15s%15s\n','Iter','Rule','Value fcn','Expectations');
      end
      if display && mod(count(iloop),options.display) == 0 || count(iloop) == 1 || ~iterate
         discrep(1,:,iloop) = [max(abs(Q(:)-Q0(:))),max(abs(P(:)-P0(:))),max(abs(Z(:)-Z0(:)))];
         fprintf('%10g%15g%15g%15g\n',count(iloop),discrep(1,:,iloop));
      end
      
   end
   warning(warningStatus);
   
   discrep(1,:,iloop) = [max(abs(Q(:)-Q0(:))),max(abs(P(:)-P0(:))),max(abs(Z(:)-Z0(:)))];
   
   % Convergence not reached.
   if ~converged
      notconverged(end+1) = iloop;
      fprintf('Not converged after %g iterations.\n',count(iloop));
   else
      fprintf('Converged after %g iterations.\n',count(iloop));
   end
   fprintf('\n');
   
   % Step VII
   % convert Q into "targeting rule" W*xb = 0
   
   % find [xf;xb] = E*[xb-;e;u]
   E = full(-[B1,A1*Z+A2]) \ B2CD;
   E23 = E(nf+1:end,nb+ne+(1:nu));
   
   % find y = V*xb
   V = -Gi*H;
   
   % convert
   %    y'*R*y + beta*xb'*P*xb
   % to
   %    xb'*U*xb
   % U := V'*R*V + beta*P
   U = transpose(V)*R*V + options.beta*P;
   
   % Find targeting rule.
   W = transpose(E23)*U;
   
   % Plug targeting rule into the original system.
   sys.A{2}(ueqtnix,nf+1:end) = W;
   sys.B{2}(ueqtnix,:) = 0;
   % try to plug instrument rule into original system
   %sys.B{2}(ueqtnix,nf+1:end) = Q(:,1:nb);
   %sys.E{2}(ueqtnix,:) = Q(:,nb+(1:ne));
   
   % calculate constant terms to preserve existing steady state
   if m.linear
      xbar = trendarray_(m,m.systemid{2}(nf+1:end),0,false,iloop);
      sys.K{2}(ueqtnix,1) = -W*xbar;
   end
   
   % Try to solve the model with the new rule.
   m = solve(m,'system',sys,'alt',iloop,'warning',true);
   
   QQ(:,:,iloop) = Q;
   WW(:,:,iloop) = W;
   YY(:,:,iloop) = Y;
   LL(:,:,iloop) = L;
   
end

if ~isempty(notconverged)
   warning_(42,sprintf(' #%g',notconverged));
end

% List of variables in the instrumental and targeting rules.   
if nargout > 4
   key = m.systemid{2}(nf+1:end);
   Qlist = [vector_(m,key),vector_(m,'e')];
   Wlist = vector_(m,key + 1i);
end

end
% End of primary function.