classdef sstate < userdataobj & getsetobj 

% TODO: Pass in parameters when constructing sstate to enable using
% !if..!else..!end and !switch..!case..!end.

   properties
      fname = '';
      type = cell([1,0]);
      input = cell([1,0]);
      eqtn = cell([1,0]);
      solvefor = cell([1,0]);
      logs = cell([1,0]);
      allbut = false([1,0]);
      growth = cell([1,0]);
      growthnames = [];
      label = cell([1,0]);
   end
   
   properties (Dependent = true)
      nblock
   end
   
   methods
      
      function this = sstate(inputfile,varargin)
         this = this@userdataobj();
         this = this@getsetobj();
         if nargin == 0
            return
         end
         this.fname = inputfile;
         p = irisparser(this.fname);
         this = sstate(p,this,varargin{:});
      end
      
      varargout = compile(this,varargin)
      
      function n = get.nblock(this)
         n = numel(this.type);
      end      
      
      function flag = isempty(this)
         flag = isempty(this.type);
      end
      
   end
   
   methods (Access = 'protected')
      varargout = specget(varargin)
      function varargout = specset(varargin)
      end
      function varargout = specsize(this)
         varargout = {1,1};
      end
   end
   
   methods (Access = 'protected', Static = true)
      
      function s = growthstring(name,shift)
         if shift < 0
            s = sprintf('d%sm%g',name,shift);
         else
            s = sprintf('d%sp%g',name,shift);
         end
      end
      
   end
   
   methods (Static = true)
      
      function flag = islog(name,logs,allbut)
         flag = any(strcmp(name,logs));
         if allbut
            flag = ~flag;
         end
      end
      
      function invalid = chkreserved(eqtn,solvefor)
         % MuPAD reserved words.
         reserved = { ...
            'E','I','D','O', ...
            'beta','zeta','theta','psi','gamma', ...
            'Ci','Si','Ei', ...
            };
         invalid = {};
         for i = 1 : numel(reserved)
            tmp = regexp([eqtn,solvefor],['\<',reserved{i},'\>'],'once');
            if any(~cellfun(@isempty,tmp))
               invalid{end+1} = reserved{i};
            end
         end
      end
   
      varargout = error(varargin)
      
   end
   
end