function this = convert(this,freq2,range,varargin)
% To get help for this IRIS function
% * type <a href="matlab: idoc tseries.convert">idoc tseries.convert</a>, or
% * open <a href="matlab: helpbrowser">Matlab Product Help</a> and go the The IRIS Toolbox at the bottom of the Contents pane,

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

freq1 = datfreq(this.start);
freq2 = recognisefreq_(freq2);

if isempty(freq2)
   error('Cannot determine requested output frequency.');
end

if nargin < 3
   range = Inf;
end

if freq1 == freq2
   % No conversion.
   if isequal(range,Inf)
      return
   else
      this.data = rangedata(this,range);
      this.start = range(1);
      this = cut_(this);
      return
   end%if
elseif freq1 == 0
   % Conversion of daily series.
   options = passvalopt('tseries.convertdaily',varargin{:});
   call = @dailyaggregate_;
else
   % Conversion of Y, Z, Q, B, or M series.
   if freq1 > freq2
      % Aggregate.
      options = passvalopt('tseries.convertaggregate',varargin{:});
      if ~isempty(options.function)
         options.method = options.function;
      end      
      call = @aggregate_;
   else
      % Interpolate.
      options = passvalopt('tseries.convertinterp',varargin{:});
      if any(strcmpi(options.method,{'quadsum','quadavg'}))
         % Quadratic interpolation matching sum or average.
         call = @interpmatch_;
      else
         % Built-in interp1.
         call = @interp_;
      end
   end
end

%**************************************************************************
% Function body.

this = call(this,range,freq1,freq2,options);
   
end
% End of primary function.