function x = groupcols(x,groups,varargin)
% GROUPCOLS  Create a new multivariate time series by applying a function
% on groups of columns of the original times series.

% The IRIS Toolbox 2009/07/07.
% Copyright 2007-2009 Jaromir Benes.

default = {...
   'function',@sum,@isfunc,...
   'log',false,@islogical,...
};
options = passvalopt(default,varargin{:});

%********************************************************************
% Function body..

x.data = x.data(:,:);
[nper,ncol] = size(x.data);

if options.log
   x.data = log(x.data);
end

ngroups = numel(groups);
data = zeros([nper,ngroups]);
used = false([1,ncol]);
for i = 1 : ngroups
   if isnumeric(groups{i}) || islogical(groups{i})
      index = groups{i};
   elseif iscellstr(groups{i}) ...
         || (ischar(groups{i}) && ~strcmp(groups{i},'*'))
      if ischar(groups{i})
         groups{i} = charlist2cellstr(groups{i});
      end
      index = findnames(x.comment,groups{i});
      index(isnan(index)) = [];      
   elseif i == ngroups && strcmp(groups{i},'*')
      index = ~used;
   else
      index = [];
   end
   data(:,i) = options.function(x.data(:,index),2);
   used(index) = true;
end
x.data = data;
x.comment = cell([1,ngroups]);
x.comment(:) = {''};

if options.log
   x.data = exp(x.data);
end

end
% End of primary function.