function x = aggregate_(x,range,fromfreq,tofreq,options)
% AGGREGATE_  Convert high frequency to low frequence.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if ischar(options.method)
   options.method = str2func(options.method);
end

%**************************************************************************
% Function body.

if isnan(x.start) && isempty(x.data)
   return
end
if datfreq(x.start) ~= fromfreq
  error('Input argument must be a %s time series.',cellref({'annual','semi-annual','quarterly','bimonthly','monthly'},fromfreq == [1,2,4,6,12]));
end
if isempty(range)
   x = empty(x);
   return
end
if ~any(isinf(range))
   x = resize(x,range);
end

datfunc = {@yy,@zz,@qq,@bb,@mm};
fromdate = datfunc{fromfreq == [1,2,4,6,12]};
todate = datfunc{tofreq == [1,2,4,6,12]};

startyear = dat2ypf(get(x,'start'));
endyear = dat2ypf(get(x,'end'));

fromdata = getdata_(x,fromdate(startyear,1):fromdate(endyear,fromfreq));
fromdatasize = size(fromdata);
nper = fromdatasize(1);
fromdata = fromdata(:,:);
nfromdata = size(fromdata,2);
factor = fromfreq/tofreq;
todata = nan([nper/factor,nfromdata]);
for i = 1 : size(fromdata,2)
   tmpdata = reshape(fromdata(:,i),[factor,nper/factor]);
   if ~isequal(options.select,Inf)
      tmpdata = tmpdata(options.select,:);
   end
   if options.ignorenan && any(isnan(fromdata(:,i)))
      for j = 1 : size(tmpdata,2)
         index = ~isnan(tmpdata(:,j));
         todata(j,i) = options.method(tmpdata(index,j),1);
      end
   else
      todata(:,i) = options.method(tmpdata,1);
   end
end
todata = reshape(todata,[nper/factor,fromdatasize(2:end)]);

x.start = todate(startyear,1);
x.data = todata;
x = cut_(x);

end
% End of primary function.

%**************************************************************************
% Subfunction first().
function x = first(x,varargin)
   x = x(1,:);
end
% End of subfunction first().

%**************************************************************************
% Subfunction last().
function x = last(x,varargin)
   x = x(end,:);
end
% End of subfunction last().
