function [x,t] = bpass_(x,band,varargin)
% <a href="matlab: edit double/bpass">BPASS</a>  Christiano-Fitzgerald band-pass filter.
%
% Called from within <a href="matlab: edit tseries/bpass">tseries/bpass</a>. Check <a href="matlab: help tseries/bpass">help tseries/bpass</a> for help.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

default = {...
   'detrend',true,@islogicalscalar, ...
   'log',false,@islogicalscalar, ...
   'regress',true,@islogicalscalar, ...
   'unitroot',true,@islogicalscalar, ...
   };
options = passvalopt(default,varargin{:});

if ~isnumeric(band)
   error('Incorrect type of input argument(s).');
end

%**************************************************************************
% Function body.

lo = max([2,min(band)]);;
hi = max(band);

if options.log
   x = log(x);
end

t = nan(size(x));
sample0 = false([1,size(x,1)]);
for i = 1 : size(x,2)
   sample = getsample(transpose(x(:,i)));
   nobs = sum(sample);
   if any(sample ~= sample0)
      % Calculate projection matrix.
      A = christiano_fitzgerald_( ...
         nobs,lo,hi,double(options.unitroot),0);
   end
   xi = x(sample,i);
   if options.detrend
      if options.regress
         M = [ones(size(xi)),(1:nobs)'];
         b = M\xi;
         ti = M*b;
      else
         d = (xi(end)-xi(1))/(nobs-1);
         ti = xi(1) + (0:nobs-1)'*d;
      end
   else
      ti = mean(xi);
      ti = ti(ones(size(xi)));
   end
   xi = xi - ti;
   x(sample,i) = A*xi;
   t(sample,i) = ti;
   sample0 = sample;
end

if options.log
   x = exp(x);
   t = exp(t);
end

end
% End of primary function.