function [start,data,dim] = init_(dates,data)
% INIT_  Create start date and data for new time series.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

dates = dates(:);
nper = length(dates);
nobs = size(data,1);
dim = size(data);
dim = dim(2:end);
if nobs == 0 && (all(isnan(dates)) || nper == 0)
  start = NaN;
  data = zeros([0,dim]);
  return
end

if size(data,1) ~= nper
  error('Number of dates and number of data points in first dimension must match.');
end

% Expand higher dimensions into 2D.
tmpsize = size(data);
data = data(:,:);

% Remove NaN dates.
index = isnan(dates);
if any(index)
  data(index,:) = [];
  dates(index) = [];
end
start = min(dates);
data_ = data;

% Number of proper dates.
nper = round(max(dates) - start + 1);
if isempty(nper)
   nper = 0;
end

% Assign data points at proper dates.
data = nan([nper,tmpsize(2:end)]);
index = round(dates - start + 1);
data(index,:) = data_;
data = reshape(data,[size(data,1),tmpsize(2:end)]);

end
% End of primary function.
