function x = interp_(x,range1,freq1,freq2,options)
% Called from within tseries/convert. Convert low-frequency series to
% high-frequency series by interpolating.

% The IRIS Toolbox 2009/05/26. Copyright (c) 2007-2008 Jaromir Benes.

%******************************************************************** !
%Function body.

if isnan(x.start) && isempty(x.data)
   return
end
if isempty(range1)
   x = empty(x);
   return
end
if ~any(isinf(range1))
   range1 = range1(1) : range1(end);
end

[xdata,range1] = getdata_(x,range1);
xsize = size(xdata);
xdata = xdata(:,:);

[startyear1,startper1] = dat2ypf(range1(1));
[endyear1,endper1] = dat2ypf(range1(end));

startyear2 = startyear1;
endyear2 = endyear1;
% Find the earliest freq2 period contained (at least partially) in freq1
% start period.
startper2 = 1 + floor((startper1-1)*freq2/freq1);
% Find the latest freq2 period contained (at least partially) in freq1 end
% period.
endper2 = ceil((endper1)*freq2/freq1);
range2 = datcode(startyear2,startper2,freq2) : datcode(endyear2,endper2,freq2);

grid1 = dat2grid(range1,options.position);
grid2 = dat2grid(range2,options.position);
xdata2 = interp1(grid1,xdata,grid2,options.method,'extrap');
if size(xdata2,1) == 1 && size(xdata2,2) == length(range2)
   xdata2 = vec(xdata2);
else
   xdata2 = reshape(xdata2,[size(xdata2,1),xsize(2:end)]);
end
x.start = range2(1);
x.data = xdata2;
x = cut_(x);

end
% End of primary function.