function x = trend2_(x,method,bp)

switch lower(method)
   case 'regress'
      x = regress_(x,bp);
   case 'diff'
      x = diff_(x,bp);
end

end

%**************************************************************************
% Subfunction regress_().
function xdata = regress_(xdata,bp)
   [nper,nx] = size(xdata);
   nbp = length(bp);
   M = [ones([nper,1]),(1:nper).',zeros([nper,nbp])];
   for i = 1 : nbp
      M(bp(i):end,2+i) = (0 : nper-bp(i)).';
   end
   for i = 1 : nx
      obs = false([nper,1]);
      j = ~isnan(xdata(:,i));
      obs(find(j,1,'first') : find(j,1,'last')) = true;
      b = M(obs,:) \ xdata(obs,i);
      xdata(obs,i) = M(obs,:) * b;
      xdata(~obs,i) = NaN;
   end
end
% End of subfunction regress_().

%**************************************************************************
% Subfunction diff_().
function xdata = diff_(xdata,bp)
   [nper,nx] = size(xdata);
   if isempty(bp) || bp(1) ~= 1
      bp = [1,bp];
   end
   if bp(end) ~= nper
      bp = [bp,nper];
   end
   nbp = length(bp);
   sample = false([nper,nbp-1]);
   for i = 1 : nbp-1
      sample(bp(i):bp(i+1),i) = true;
   end
   tt = 0;
   for i = 1 : nx
      obs = false([nper,1]);
      j = ~isnan(xdata(:,i));
      obs(find(j,1,'first') : find(j,1,'last')) = true;
      for k = 1 : nbp-1
         n = sum(obs & sample(:,k));
         b = mean(diff(xdata(obs & sample(:,k),i),1,1));
         tt = [tt;tt(end)+b*(1:n-1).'];
      end
      tt = tt + mean(xdata(obs,i) - tt);
      xdata(obs,i) = tt;
      xdata(~obs,i) = NaN;
   end
end
% End of subfunction diff_().