function [b,bstd,e,estd,yfit,range] = regress(y,x,range,varargin)
% REGRESS  Ordinary or weighted least-square regression. Calls |lscov|.

% The IRIS Toolbox 2009/06/10.
% Copyright 2007-2009 Jaromir Benes.

default = {...
   'weighting',[],@(x) (isnumeric(x) && isempty(x)) || istseries(x),...
};
options = passvalopt(default,varargin{:});

if nargin < 3
   range = Inf;
end

%********************************************************************
% Function body.

if length(range) == 1 && isinf(range)
   range = get([x,y],'minrange');
else
   range = range(1) : range(end);
end

xdata = rangedata(x,range);
ydata = rangedata(y,range);

if isempty(options.weighting)
   [b,bstd,evar] = lscov(xdata,ydata);
else
   w = rangedata(options.weighting,range);
   [b,bstd,evar] = lscov(xdata,ydata,w);
end
estd = sqrt(evar);

if nargout > 2
   e = replace(y,ydata - xdata*b,range(1));
end

if nargout > 4
   yfit = replace(y,xdata*b,range(1));
end

end
% End of primary function.