function x = set(x,varargin)
% SET  Set time series attributes.
%
% Syntax:
%   x = set(x,attrib1,value1,attrib2,value2,...)
% Required input arguments:
%   x [ tseries ] Time series.
%   attrib [ char ] Attribute
%   value [ anything ] Value to be assigned to attribute.
% Attributes:
%   'comment' [ char ]
%   'freq' [ numeric ]
%   'end' [ numeric ]
%   'range' [ numeric ]
%   'start' [ numeric ]

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if ~iscellstr(varargin(1:2:end))
   error('Incorrect type of input argument(s).');
end

%**************************************************************************
% Function body.

invalid = cell([1,0]);
for i = 1 : 2 : nargin-1
   if ~set_(varargin{i},varargin{i+1})
      invalid{end+1} = varargin{i};
   end
end

if ~isempty(invalid)
   warning(sprintf('Unrecognised attribute: ''%s''. ',invalid{:}));
end
% End of function body.

   %***********************************************************************
   % Nested function set_()
   function flag = set_(attribute,value)
      attribute(isspace(attribute)) = '';
      attribute = lower(attribute);
      flag = true;
      switch attribute
         case {'start','startdate','first'}
            x.start = value;
         case {'end','enddate','last'}
            x.start = value - size(x.data,1) + 1;
         case {'data'}
            x.data = value;
         case 'comment'
            x = comment(x,value);
         otherwise
            flag = false;
      end
   end
   % End of nested function set_().

end
% End of primary function.