function x = subsasgn(x,s,y)
% SUBSASGN  Tseries object assignment.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

%********************************************************************
% Function body.

% Variable Editor's calling SUBSASGN cannot be distinguish from a regular
% assignment in the workspace (i.e. we cannot judge the call by dbstack).
% We try to guess whether this is a call from Variable Editor based on the
% 1st dimension's subscripts. This, however, cannot work for indeterminate
% frequencies.
if isVariableEditor_(x,s)
   x = variableEditor_('subsasgn',x,s,y);
   return
end

% RHS input must exist as a tseries object.
if ~istseries(x) ...
   error('iris:tseries','Invalid time series assignment.');
end

% First reference can be a lag or lead.
[x,s,shift] = lagorlead_(x,s);
if isempty(s)
   return
end

% Either {} or () can be used.
if isequal(s(1).type,'{}')
   s(1).type = '()';
end

% After a lag or lead, only one ()-reference is allowed.
if length(s) > 1 || ~isequal(s(1).type,'()')
   error('iris:tseries','Invalid time series assignment.');
end

x = setdata_(x,s,y);

% Shift start date back.
if shift ~= 0
   x.start = x.start + shift;
end


end
% End of primary function.


