function [x,range] = subsref(x,s)
% SUBSREF  Subscripted reference function for tseries objects.
%
% Syntax:
%   x(dates)
%   x{dates}
%   x(dates,...)
%   x{dates,...}
%   x.comment
%   x.comment(...)
%   x.comment{...}
% Output arguments:
%   x [ tseries ] Referenced time series.
% Required input arguments:
%   dates [ numeric ] Dates or time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

% Detect calls from Variable Editor 2008b or higher.
if isVariableEditor_(x,s)
   x = variableEditor_('subsref',x,s);
   return
end

% First reference can be a lag or lead.
[x,s] = lagorlead_(x,s);
if isempty(s)
   return
end

switch s(1).type
   case '()'
      [x,range] = getdata_(x,s(1).subs{:});
   case '{}'
      [ans,range,x] = getdata_(x,s(1).subs{:});
      x = cut_(x);
      if length(s) > 1
         x = subsref(x,s(2:end));
      end
   otherwise
      error('iris:tseries', ...
         'Invalid time series reference.');
end

end
% End of primary function.