classdef getsetobj
   % GETSETOBJ  Implement generic GET, SET, SIZE, DISPLAY methods.
   
   methods (Access = 'public')
      
      function this = getsetobj()
      end
      
      function varargout = get(this,varargin)
         if ~iscellstr(varargin)
            error('Incorrect type(s) of input argument(s).');
         end
         varargout = cell(size(varargin));
         query = lower(strtrim(varargin));
         for i = 1 : numel(query)
            [varargout{i},flag] = specget(this,query{i});
            if ~flag
               invalidQuery(this,varargin{i});
            end
         end
      end
      
      function [this,flag] = set(this,varargin)
         if ~iscellstr(varargin(1:2:end))
            error('Incorrect type(s) of input argument(s).');
         end
         query = lower(strtrim(varargin(1:2:end)));
         value = varargin(2:2:end);
         for i = 1 : numel(query)
            [this,flag] = specset(this,query{i},value{i});
            if ~flag
               invalidQuery(this,varargin{2*i-1});
            end
         end
      end
      
      function varargout = size(this,dim)
         tmpsize = specsize(this);
         if nargout <= 1
            if nargin > 1
               varargout{1} = [tmpsize{:}];
               varargout{1} = varargout{1}(dim);
            else
               varargout{1} = [1,tmpsize{end}];
            end
         else
            varargout = cell([1,nargout]);
            varargout(1:nargout) = tmpsize(1:nargout);
         end
      end
      
      function display(this)
         if isequal(get(0,'FormatSpacing'),'compact')
            disp([inputname(1) ' =']);
            disp(this);
         else
            disp(' ')
            disp([inputname(1) ' =']);
            disp(' ');
            disp(this);
         end
      end
      
   end
   
   methods (Abstract = true, Access = 'protected')
      varargout = specget(varargin)
      varargout = specset(varargin)
      varargout = specsize(varargin)
   end
   
   methods (Access = 'private')
      
      function invalidQuery(this,query)
         error('iris:getsetobj', ...
            'Invalid query into a %s object: ''%s''.',...
            class(this),query);
      end
      
   end
   
end