function [outputformat,range,y,e] = datarequest(this,data,range,options)

ny = size(this,1);
if any(isinf(range))
   range = Inf;
   infrange = true;
else
   if ~isempty(range)
      range = range(1) : range(end);
   end
   infrange = false;
end

if isstruct(data)
   ynames = get(this,'ylist');
   if infrange
      range = dbrange(data,ynames);
   end
   nper = length(range);
   y = nan([nper,0]);
   for i = 1 : ny
      if isfield(data,ynames{i})
         yi = rangedata(data.(ynames{i}),range);
         yi = permute(yi,[1,3,2]);
      else
         yi = nan([nper,1,size(y,3)]);
      end
      y = [y,yi];
   end
   if nargout > 3
      enames = get(this,'elist');
      e = nan([nper,0]);
      for i = 1 : ny
         if isfield(data,enames{i})
            ei = rangedata(data.(enames{i}),range);
            ei = permute(ei,[1,3,2]);
         else
            ei = zeros([nper,1,size(e,3)]);
         end
         e = [e,ei];
      end
   end
   inputformat = 'dbase';
elseif istseries(data)
   [y,range] = rangedata(data,range);
   if size(y,2) == 2*ny
      e = y(:,ny+1:end,:);
      y = y(:,1:ny,:);
   else
      e = zeros(size(y));
   end
   inputformat = 'tseries';
elseif ~isempty(data)
   if infrange
      range = 1 : size(data,1);
   end
   nper = length(range);
   index = range >= 1 & range <= size(data,1);
   y = nan([nper,size(data,2),size(data,3)]);
   y(index,:,:) = data(range(index),:,:);
   if size(y,2) == 2*ny
      e = y(:,ny+1:end,:);
      y = y(:,1:ny,:);
   else
      e = zeros(size(y));
   end
   inputformat = 'array';
else
   if infrange
      nper = 0;
   else
      nper = length(range);
   end
   y = nan([nper,ny]);
   e = nan([nper,ny]);
   inputformat = 'array';
end

% Transpose data.
y = permute(y,[2,1,3]);
if nargout > 3
   e(isnan(e)) = 0;
   e = permute(e,[2,1,3]);
end

% Determine output format.
if nargin > 3
   if strcmpi(options.output,'auto')
      outputformat = inputformat;
   else
      outputformat = options.output;
   end
else
   outputformat = '';
end

end