function name = dbnames(d,varargin)

options = passvalopt('data.dbnames',varargin{:});

%**************************************************************************
% Function body.

% Empty name filter and empty class filter returns empty list.
if isempty(options.namefilter) && isempty(options.classfilter)
   name = {};
   return
end

% Get the database's field names. Infs in both filters return all names.
name = fieldnames(d).';
if isequal(options.namefilter,Inf) && isequal(options.classfilter,Inf)
   return
end
n = length(name);

% Filter the names.
if isequal(options.namefilter,Inf)
   nameTest = true([1,n]);
elseif isempty(options.namefilter)
   nameTest = false([1,n]);
else
   x = regexp(name,options.namefilter,'once');
   nameTest = ~cellfun(@isempty,x);
end
   
% Filter the classes.
if isequal(options.classfilter,Inf)
   classTest = true([1,n]);
elseif isempty(options.classfilter)
   classTest = false([1,n]);
else
   c = cellfun(@class,struct2cell(d),'uniformOutput',false).';
   x = regexp(c,options.classfilter,'once');
   classTest = ~cellfun(@isempty,x);
end

% Return the names tha pass both tests.
name = name(nameTest & classTest);
   
end
% End of primary function.