function [fg,ax,ln] = dbplot(d,list,range,varargin)
% DBPLOT  Plot a batch of series, or their transformations, from a
% database.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

[options,varargin] = passvalopt('data.dbplot',varargin{:});

% Swap list and range if needed.
if isnumeric(list) && (ischar(range) || iscellstr(range))
   [list,range] = deal(range,list);
end

P = inputParser();
P.addRequired('d',@isstruct);
P.addRequired('list',@(x) ischar(x) || iscellstr(x));
P.addRequired('range',@isnumeric);
P.parse(d,list,range);

% Convert 'a,b,c' to {'a','b','c'} if needed.
if ischar(list)
   list = charlist2cellstr(list);
end

if isnumeric(range) && length(range) == 1 && isinf(range)
   range = dbrange(d,list);
end

%**************************************************************************
% Function body.

nlist = length(list);
nsub = ceil(sqrt(nlist));

if strcmpi(options.subplot,'auto')
   if nsub*(nsub-1) >= nlist
      nsub = [nsub,nsub-1];
   else
      nsub = [nsub,nsub];
   end
else
   nsub = options.subplot;
end

fg = [];
ax = {[]};
ln = {{}};

transform = '';
if ~isempty(options.transform)
   tokens = regexp(func2str(options.transform),'@\((\w+)\)\s*(.*)\s*','tokens','once');
   transform = tokens{2};
end

if ~isfunc(options.title)
   options.title = parseLabels_(options.title);
   if isempty(options.title)
      options.title = list;
   end
end
options.xlabel = parseLabels_(options.xlabel);
options.ylabel = parseLabels_(options.ylabel);
options.zlabel = parseLabels_(options.zlabel);

fg(end+1) = figure();
count = 0;
for i = 1 : nlist
   if ~isempty(regexp(list{i},'^\w+$','once'))
      x = d.(list{i});
   else
      x = dbeval(d,list{i});
   end
   if ~isempty(options.transform)
      x = options.transform(x);
   end
   count = count + 1;
   if count > prod(nsub)
      fg(end+1) = figure();
      ax{end+1} = [];
      ln{end+1} = {};
      count = 1;
   end
   ax{end}(end+1) = subplot(nsub(1),nsub(2),count);
   if ~isinf(options.round) && ~isnan(options.round) && istseries(x)
      x = round(x,options.round);
   end
   ln{end}{end+1} = options.plotfunc(range,x,varargin{:});
   if options.tight
      axis('tight');
   end
   grid('on');
   if isfunc(options.title)
      tmptitle = options.title(x);
      if iscellstr(tmptitle)
         tmptitle = tmptitle{1};
      end
   else
      tmptitle = options.title{i};
   end
   if ~isempty(transform)
      tmptitle = [tmptitle,', ',transform];
   end
   title(tmptitle,'interpreter','none');
   if ~isempty(options.xlabel)
      xlabel(options.xlabel{i});
   end
   if ~isempty(options.ylabel)
      xlabel(options.ylabel{i});
   end
   if ~isempty(options.zlabel)
      xlabel(options.zlabel{i});
   end   
   if options.zeroline
      zeroline(ax{end}(end));
   end
   if ~isempty(options.highlight)
      highlight(options.highlight);
   end
end

if ~isempty(options.style)
   qstyle(fg,options.style);
end

   %***********************************************************************
   % Nested function parseLabels_().
   function labels = parseLabels_(labels)
      if ~isempty(labels)
         if ischar(labels)
            labels = {labels};
         end
         labels = repeatlast(labels,2,nlist);
      end
   end
   % End of nested function parseLabels_().

end
% End of primary function.