function [list0,list,expr] = ...
   dbquery(dbase0,namemask,exprmask,classfilter,namefilter,namelist)
% DBQUERY  Database query.

%**************************************************************************
% Function body.

% Name filter.
if iscellstr(namefilter) && isempty(namelist)
   namelist = namefilter;
end

field = fieldnames(dbase0).';
if ischar(namefilter) && ~isempty(namefilter)
   [ans,list0,tokens] = strfun.matchindex(field,namefilter);
elseif iscellstr(namelist) && ~isempty(namelist)
   list0 = namelist;
   tokens = cell(size(list0));
   [tokens{:}] = deal(cell([1,0]));
elseif isnumeric(namefilter) && isinf(namefilter)
   list0 = field;
   tokens = cell(size(list0));
   [tokens{:}] = deal(cell([1,0]));
else
   list0 = {};
   tokens = cell(size(list0));
   [tokens{:}] = deal(cell([1,0]));
end

% Class filter.
if ~(isnumeric(classfilter) && isinf(classfilter))
   classlist = cell(size(list0));
   for i = 1 : numel(list0)
      classlist{i} = class(dbase0.(list0{i}));
   end
   index = strfun.matchindex(classlist,classfilter);
   list0 = list0(index);
   tokens = tokens(index);
end

% New names.
if nargout > 1
   if ~isempty(namemask)
      list = cell(size(list0));
      for i = 1 : numel(list0)
         list{i} = unmask_(namemask,list0{i},tokens{i}{:});
      end
   else
      list(1:length(list0)) = {''};
   end
end

% Expressions.
if nargout > 2
   if ~isempty(exprmask)
      expr = cell(size(list0));
      for i = 1 : numel(list0)
         expr{i} = unmask_(exprmask,list0{i},tokens{i}{:});
      end
   else
      expr(1:length(list0)) = {''};
   end
end

end
% End of primary function.

%**************************************************************************
% Subfunction unmask_().
function unmask = unmask_(mask,varargin)
   if isempty(mask)
      unmask = '';
   else
      unmask = mask;
      for i = 1 : nargin-1
         unmask = strrep(unmask,sprintf('lower($%g)',i-1),lower(varargin{i}));
         unmask = strrep(unmask,sprintf('upper($%g)',i-1),upper(varargin{i}));
         unmask = strrep(unmask,sprintf('$%g',i-1),varargin{i});
      end
      unmask = regexprep(unmask,'\$\d*','');
   end
end
% End of subfunction unmask_().
