function [range,freq] = dbrange(d,list)
% DBRANGE  Find the minimum range that encompasses the listed series.

if nargin < 2
   list = fieldnames(d);
elseif ischar(list)
   list = charlist2cellstr(list);
end

%**************************************************************************
% Function body.

mindate = Inf;
maxdate = -Inf;
nlist = numel(list);
start = nan([1,nlist]);

for i = 1 : nlist
   if isfield(d,list{i}) && istseries(d.(list{i}))
      start(i) = startdate(d.(list{i}));
      finish = enddate(d.(list{i}));
      if ~isnan(start(i))
         mindate = min([mindate,start(i)]);
         maxdate = max([maxdate,finish]);
      end
   end
end
   
if isinf(mindate)
   mindate = NaN;
end

if isinf(maxdate)
   maxdate = NaN;
end

range = mindate : maxdate;

if nargout > 1
   freq = datfreq(start);
end

end
% End of primary function.