function dpack = dpresize(dpack,range,cellindex)
% DPRESIZE  Resize datapack or its cell in time dimension.
%
% Syntax:
%   dpack = dpresize(dpack,range)
% Output arguments:
%   dpack [ cell ] Resized datapack.
% Required input arguments:
%   dpack [ cell ] Datapack to be resized.
%   range [ numeric ] New time range, i.e. <a href="dates.html">IRIS serial date numbers</a>.

% The IRIS Toolbox 2009/09/02.
% Copyright 2007-2009 Jaromir Benes.

if nargin < 3
   cellindex = 1 : 3;
end

%********************************************************************
% Function body.

if numel(range) == 1 && isinf(range)
  return
end

% Same input range and output range.
oldrange = dpack{4};
if round(oldrange(1) - range(1)) == 0 ...
      && round(oldrange(end) - range(end)) == 0
   return
end

% Resolve [-Inf,date] or [date,Inf].
if length(range) == 2
   if isinf(range(1))
      range(1) = oldrange(1);
   end
   if isinf(range(2))
      range(2) = oldrange(end);
   end
end
newrange = range(1) : range(end);

% Tell apart mean or mse datapacks.
mse = numel(dpack) > 5 && isfield(dpack{5},'mse') && dpack{5}.mse;

noldrange = numel(oldrange);
index = round(newrange - oldrange(1)) + 1;
npre = sum(index < 1);
npost = sum(index > noldrange);
index = index(index >= 1 & index <= noldrange);

if mse
   for i = transpose(cellindex(:))
      if isempty(dpack{i})
         continue
      end
      tmpsize = [ ...
         size(dpack{i},1), ...
         size(dpack{i},2), ...
         size(dpack{i},3), ...
         size(dpack{i},4) ...
      ];
      dpack{i} = cat(3, ...
         nan([tmpsize(1:2),npre,tmpsize(4)]), ...
         dpack{i}(:,:,index,:), ...
         nan([tmpsize(1:2),npost,tmpsize(4)]) );
   end
else
   for i = transpose(cellindex(:))
      if isempty(dpack{i})
         continue
      end
      tmpsize = [ ...
         size(dpack{i},1), ...
         size(dpack{i},2), ...
         size(dpack{i},3) ...
      ];
      dpack{i} = [ ...
         nan([tmpsize(1),npre,tmpsize(3)]), ...
         dpack{i}(:,index,:), ...
         nan([tmpsize(1),npost,tmpsize(3)]) ];
   end
end

end
% End of primary function.