function data = outputdata(this,outputformat,range,X,P,getnames)

%**************************************************************************
% Function body.

nx = size(X,1);
range = range(1) : range(end);
nper = numel(range);
ndata = size(X,3);

if numel(P) == 1 && isnan(P)
   nstd = size(X,1);
   std = nan([nstd,nper,ndata]);
elseif ~isempty(P)
   P = time_domain.fixcov(P);
   nstd = min([size(X,1),size(P,1)]);
   std = zeros([nstd,nper,ndata]);
   for i = 1 : ndata
     for j = 1 : nstd
        std(j,:,i) = permute(sqrt(P(j,j,:,i)),[1,3,2,4]);
     end
   end
end

switch outputformat
   case 'tseries'
      template = tseries();
      outputtseries_();
   case 'dbase'
      template = tseries();
      outputdbase_();
   case 'array'
      outputarray_();
end

% End of function body.

   %***********************************************************************
   % Nested function outputtseries_().
   function outputtseries_()
      if isempty(P)
         data = replace(template,permute(X,[2,1,3]),range(1));
      else
         data = struct();
         data.mean = replace(template,permute(X,[2,1,3]),range(1));
         data.std = replace(template,permute(std,[2,1,3]),range(1));
      end
   end
   % End of nested function outputtseries_().

   %***********************************************************************
   % Nested function outputdbase_().
   function outputdbase_()
      data = struct();
      names = get(this,getnames);
      for i = 1 : nx
         data.(names{i}) = ...
            replace(template,permute(X(i,:,:),[2,3,1]),range(1));
      end
      if ~isempty(P)
         data = struct('mean',data,'std',struct());
         for i = 1 : nstd
            data.std.(names{i}) = ...
               replace(template,permute(std(i,:,:),[2,3,1]),range(1));
         end         
      end    
   end
   % End of nested function outputdbase_().

   %***********************************************************************
   % Nested function outputarray_().
   function outputarray_()
      if isempty(P)
         data = permute(X,[2,1,3]);
      else
         data = struct();
         data.mean = permute(X,[2,1,3]);
         data.std = permute(std,[2,1,3]);
      end
   end
   % End of nested function outputarray_().

end
% End of primary function.