function varargout = irisconfigmaster(action,varargin)
% IRISCONFIGMASTER  The IRIS Toolbox master configuration file.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

mlock();
persistent config protected;
if isempty(config)
   config = irisconfig();
end

%**************************************************************************
% Function body.

switch action
case 'get'
   if nargin == 1
      varargout{1} = config;
   else
      notfound = {};
      for i = 1 : length(varargin)
         try
            varargout{i} = config.(varargin{i});
         catch
            notfound{end+1} = varargin{i};
            varargout{i} = NaN;
         end
      end
      if ~isempty(notfound)
         warning('iris:config',...
            '\n*** This is not a valid IRIS option: ''%s''.',...
            notfound{:});
      end
   end
case 'set'
   invalid = {};
   unable = {};
   for i = 1 : 2 : nargin-1
      if any(strcmp(varargin{i},config.protected))
         unable{end+1} = varargin{i};
      elseif isfield(config,varargin{i})
         name = varargin{i};
         value = varargin{i+1};
         if isfield(config.validate,name) ...
               && ~config.validate.(name)(config.(name))
            invalid{end+1} = name;
         else
            config.(name) = value;
         end
      end
   end
   if ~isempty(unable)
      warning('iris:config', ...
         '\n*** This IRIS option is not customisable and its value has not been changed: ''%s''.', ...
         unable{:});
   end
   if ~isempty(invalid)
      warning('iris:config', ...
         '\n*** The value supplied for this IRIS option is invalid and has not been assigned: ''%s''.', ...
         invalid{:});
   end   
case 'reset'
   config = irisconfig();
otherwise
   error('iris:config',...
      'Incorrect type or number of input or output arguments.');
end

end
% End of primary function.