function varargout = irispathmanager(action,varargin)
% IRISPATHMANAGER  IRIS path manager.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

%**************************************************************************
% Function body.

switch lower(action)
   case 'cleanup'
      % Remove all IRIS roots and subs found on the Matlab temporary and
      % permanent search paths.
      varargout{1} = {};
      list = which('irisstartup.m','-all');
      for i = 1 : numel(list)
         root = fileparts(list{i});
         if isempty(root)
            continue
         end
         [root,subs] = genpathcell_(root,{'-','+'});
         if root(end) == ';'
            root(end) = '';
         end
         if ~isempty(subs)
            rmperm_(subs{:});
            rmtemp_(subs{:});
         end
         if ~isempty(root)
            rmperm_(root);
            rmtemp_(root);
         end
         varargout{1}{end+1} = root;
      end
      clear('functions');
      rehash();
   case 'addroot'
      % Add the specified root to both the temporary and permanent search
      % paths.
      addperm_(varargin{1});
      addpath(varargin{1},'-begin');
   case 'addcurrentsubs'
      % Add subfolders within the current root to the temporary search
      % path.
      root = which('irisstartup.m');
      root = fileparts(root);
      [root,subs] = genpathcell_(root,{'-','+'});
      if isempty(subs)
         subs = '';
      else
         subs = [subs{:}];
         addpath(subs,'-begin');
      end
      varargout{1} = subs;
   case 'removecurrentsubs'
      % Remove subfolders within the current root from the temporary and
      % permanent search paths.
      root = which('irisstartup.m');
      root = fileparts(root);
      [root,subs] = genpathcell_(root,{'-','+'});
      if isempty(subs)
         subs = '';
      else
         subs = [subs{:}];
         rmperm_(subs);
         rmtemp_(subs);
      end
      varargout{1} = subs;
end      

end
% End of primary function.

%**************************************************************************
% Subfunction rmtemp_().
function rmtemp_(varargin)
   status = warning('query','all');
   warning('off','MATLAB:rmpath:DirNotFound');
   rmpath([varargin{:}]);
   warning(status);
end
% End of subfunction rmtemp_().

%**************************************************************************
% Subfunction rmperm_().
function flag = rmperm_(varargin)
   status = warning('query','all');
   warning('off','MATLAB:rmpath:DirNotFound');
   p = path();
   path(pathdef());
   rmpath([varargin{:}]);
   flag = savepath();
   path(p);
   warning(status);
end
% End of subfunction rmperm_().

%**************************************************************************
% Subfunction addperm_().
function flag = addperm_(varargin)
   status = warning('query','all');
   warning('off','MATLAB:rmpath:DirNotFound');
   p = path();
   path(pathdef());
   addpath([varargin{:}],'-begin');
   flag = savepath();
   path(p);
   warning(status);
end
% End of subfunction addperm_().

%**************************************************************************
% Subfunction genpathcell_().
function [root,p] = genpathcell_(root,exclude)

p = genpath(root);
if isempty(p)
   p = {};
   return
else
   p = regexp(genpath(root),'.*?;','match');
   if isempty(p)
      return
   else
      root = p{1};
      p = p(2:end);
   end
end

if nargin > 1 && ~isempty(exclude) && ~isempty(p)
   if ischar(exclude)
      exclude = {exclude};
   end
   flag = true(size(p));
   for i = 1 : numel(p)
      for j = 1 : numel(exclude)
         flag(i) = flag(i) && isempty(regexp(p{i},exclude{j},'once'));
         if ~flag(i)
            break
         end
      end
   end
   p = p(flag);
end

end
% End of subfunction genpathcell_().