function irisstartup(varargin)
   % IRISSTARTUP  Start an IRIS session.
   % For help on this function, type <a href="matlab:idoc('config.irisstartup')">idoc config.irisstartup</a>.

   % The IRIS Toolbox.
   % Copyright 2007-2009 Jaromir Benes.

% Start a new IRIS session.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

% IRIS can only run in Matlab Release 2008a and higher.
if matlabrelease() < 2008
   error('iris:startup',...
      'Sorry, <a href="http://www.iris-toolbox.com">The IRIS Toolbox</a> can only run in Matlab R2008a or higher.');
end

% Get the whole iris folder structure. This must be done before
% irisremove(...) otherwise genpathcell(...) will not be found.
% Exclude directories starting with an _ (on top of +, @, and private,
% which are exclude by default).
removed = irispathmanager('cleanup');
root = removed{1};
removed(1) = [];

% Add the current IRIS folder structure to the search path.
addpath(root,'-begin');
irispathmanager('addroot',root);
subs = irispathmanager('addcurrentsubs');

% Reset the default optional arguments in |passvalopt|.
try
   munlock('passvalopt');
end
try
   munlock('irisconfigmaster');
end
clear('functions');
passvalopt();
passvalopt();

% Reset the configuration file.
rehash();
irisreset();
config = irisget();

% Add IRIS extensions to Matlab Editor.
irisextensions();

%**************************************************************************
% Display intro messages.

% Intro message.
fprintf('\n');
fprintf('\t%s\n',irisping());
fprintf('\tCheck out <a href="http://groups.google.com/group/iris-toolbox">The IRIS Toolbox discussion group</a>');
fprintf(' and <a href="http://groups.google.com/group/iris-model-exchange">IRIS Model Exchange</a>.\n');
fprintf('\tCopyright 2007-%s Jaromir Benes.\n',datestr(now,'YYYY'));
fprintf('\n');

% IRIS root folder.
fprintf('\tIRIS root: <a href="file:///%s">%s</a>.\n',root,root);

% Report user config file used.
fprintf('\tUser config file: ');
if isempty(config.userconfigpath)
   fprintf('<a href="matlab: idoc config/irisuserconfig">No user config file found</a>.\n');
else
   fprintf('<a href="matlat: edit %s">%s</a>.\n',config.userconfigpath,config.userconfigpath);
end
   
% TeX/LaTeX executables.
fprintf('\t(La)TeX binary files: ');
if isempty(config.latexpath)
   fprintf('<a href="matlab: edit .m">No TeX/LaTeX installation found</a>.\n');
else
   tmppath = fileparts(config.latexpath);
   fprintf('<a href="file:///%s">%s</a>.\n',tmppath,tmppath);
end

% Report IRIS folders removed.
if ~isempty(removed)
   fprintf('\n\tSuperfluous IRIS folders removed from Matlab path:\n');
   for i = 1 : numel(removed)
      fprintf('\t* <a href="file:///%s">%s</a>\n', ...
         removed{i},removed{i});
   end
end

fprintf('\n');

end
% End of primary function.