function [h,yindex] = conbar(varargin)
% CONBAR  Contribution bar plot function.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if ~isempty(varargin) && length(varargin{1}) == 1 && ishandle(varargin{1})
   ax = varargin{1};
   axes(ax);
   varargin(1) = [];
else
   ax = gca();
end

if length(varargin) > 1 && isnumeric(varargin{1}) && isnumeric(varargin{2})
   x = varargin{1};
   y = varargin{2};
   varargin(1:2) = [];
elseif ~isempty(varargin) && isnumeric(varargin{1})
   y = varargin{1};
   tmpsize = size(y);
   if any(tmpsize(1:2) == 1)
      y = y(:);
   end
   x = 1 : size(y,1);
   varargin(1) = [];
else
   y = [];
   x = [];
end

if isempty(x) || isempty(y)
   h = [];
   yindex = [];
   return
end

default = {...
   'barwidth',0.8,@isnumericscalar,...
   'colormap',get(gcf(),'colormap'),@isnumeric,...
};
options = passvalopt(default,varargin{:});

%**************************************************************************
% Function body.

[nper,ndata] = size(y);
nc = size(options.colormap,1);
colorindex = 1 + round(0 : (nc-1)/(ndata-1) : nc-1);
colors = options.colormap(colorindex,:);

holdstate = ishold(ax);

if length(x) > 1
   avgxstep = mean(diff(x));
else
   avgxstep = 1;
end
d = avgxstep*options.barwidth/2;

h = nan([nper,ndata]);
yindex = [];
for i = 1 : nper
   if all(isnan(y(i,:)))
      continue
   end
   y1 = nan([1,ndata+1]);
   [ysort,sortindex] = sort(y(i,:),2,'descend');
   isnegative = ysort < 0;
   y1(1) = sum(ysort(isnegative));
   y1(2:end) = [-ysort(isnegative),ysort(~isnegative)];
   cy1 = cumsum(y1,2);
   yindex(end+1,:) = [sortindex(isnegative),sortindex(~isnegative)];
   for j = 1 : ndata
      if i == 1 && j > 1
         hold('on');
      end
      h(i,j) = fill(x(i)+[-1,1,1,-1]*d,cy1([j,j,j+1,j+1]),colors(yindex(end,j),:));
   end
end

% Creater an hggroup for all bar clusters except the first one, which is
% used to create conlegend.
hg = hggroup();
set(h(2:end,:),'parent',hg);

% Store the ordering of bars in the first, legend-able cluster.
setappdata(ax,'conbar',true);
setappdata(ax,'index',yindex(1,:));

if holdstate == 0
   hold(ax,'off');
else
   hold(ax,'on');
end

end
% End of primary function.