function g = highlight(varargin)
% <a href="matlab: edit highlight">HIGHLIGHT</a>  Highlight specific time range in time series graph.
%
% Syntax:
%   h = highlight(range,...)      (1)
%   h = highlight(ax,range,...)   (2)
% Output arguments:
%   h [ numeric ] Handle to highlighted area.
% Required input arguments:
%   range [ numeric ] Time range to be highlighted, i.e. <a href="dates.html">IRIS serial date numbers</a>.
% Required input arguments for syntax (2):
%   ax [ numeric ] Handle(s) to axes where the highlight will be added.
% <a href="options.html">Optional input arguments:</a>
%   'colour' [ numeric | <a href="default.html">[0.9,0.9,0.9]</a> ] RGB colour code.
%   'bar' [ true | <a href="default.html">false</a> ] The edges of the highlight area lie between two dates for bar graphs.

% The IRIS Toolbox.
% Copyright 2007-2010 Jaromir Benes.

if all(ishandle(varargin{1}))
   h = varargin{1};
   varargin(1) = [];
else
   h = gca();
end

range = varargin{1};
varargin(1) = [];

if iscell(range)
   g = [];
   for i = 1 : numel(range)
      g = [g,highlight(h,range{i},varargin{:})];
   end
   return
end

options = passvalopt('utils.highlight',varargin{:});

if ~isempty(options.color)
   options.colour = options.color;
end

if ~isempty(options.grade)
   options.colour = options.grade*[1,1,1];
end

%**************************************************************************
% Function body.

g = [];
for hi = h(:)'

   % Preserve the order of figure children.
   fi = get(hi,'parent');
   fich = get(fi,'children');
   % Axes to be on top of lines and patches.
   set(hi,'layer','top');
   flag = getappdata(hi,'tseries');

   if ~isempty(flag) && flag
      range = range(1) : range(end);
      freq = datfreq(range(1));
      timescale = dat2grid(range);
      if isempty(timescale)
         continue
      end
      if freq > 0
         around = 1/(2*freq);
      else
         around = 0.5;
      end
      timescale = [timescale(1)-around,timescale(end)+around];
   else
      timescale = [range(1)-options.around,range(end)+options.around];
   end
   
   ylim = get(hi,'ylim');
   y = ylim([1,1,2,2]);
   x = timescale([1,end,end,1]);
   gi = patch(x,y,options.colour,'parent',hi);
   set(gi,'edgeColor','none');
   set(hi,'yLimMode','manual','yLim',ylim);
  
   % Order patch last.
   ch = get(hi,'children');
   ch(ch == gi) = [];
   ch(end+1) = gi;
   set(hi,'children',ch);

   % Reset the order of figure children.
   set(fi,'children',fich);
   g = [g,gi];

end

if ~isempty(g) && options.excludefromlegend
   % Exclude highlighted area from legend.
   excludefromlegend(g);
   % Exclude from style.
   set(g,'userData','excludeFromStyle');
end

end
% End of primary function.