function [this,sub] = readcdf(fname)
   % READCDF  Read quick-report contents definition file.

   % The IRIS Toolbox.
   % Copyright 2007-2010 Jaromir Benes.

%**************************************************************************
% Function body.

code = file2char(char(fname));
code = strfun.converteols(code);

% Replace old syntax.
code = strrep(code,'?--','!--');
code = strrep(code,'<<-->>','!++');

% remove line and block comments
code = removecomments(code, ...
   {'/*','*/'},{'%{','%}'},{'<!--','-->'}, ...
   '(?<!\\)%','\.\.\.','//');
code = strrep(code,'\%','%');
% replace single quotes with double quotes
code = strrep(code,'''','"');

% Read panels !-- and pages !++
%{
tmp = regexp(code,'!--(?<title>.*?)\n(?<series>.*?)\s*(?<break>!\+\+)?\s*(?=!--|$)','names');
this = struct();
% break down graphs into formulas and labels
for i = 1 : length(tmp)
   this(i).break = ~isempty(tmp(i).break);
   this(i).title = strtrim(tmp(i).title);
   [this(i).formula,this(i).legend] = charlist2cellstr(tmp(i).series);
end
%}

% Find page tags !++ and panel tages !-- !::

tags = '#|!\+\+|!\-\-|!::|!\*\*';
this = regexp(code, ...
   ['(?<tag>',tags,')(?<temp>.*?)(?=',tags,'|$)'],'names');
for i = 1 : length(this)
   temp = regexp(this(i).temp, ...
      '\s*(?<title>[^\n]*)(?<body>\n.*)?','names');
   if isempty(temp)
      temp.title = '';
      temp.body = '';
   end
   this(i).title = temp.title;
   this(i).body = strtrim(temp.body);
end
this = rmfield(this,'temp');

% Find first non-# tag. If it is not !++, add !++.
count = 1;
while count <= length(this) && strncmp(this(count).tag,'#',1)
   count = count + 1;
end

if count <= length(this) && ~strcmp(this(1).tag,'!++')
   this = [this(1:count-1),...
      struct('tag','!++','title','','body',''),...
      this(count:end)];
end

% Remove two consecutive page tags
flag = cellfun(@(x) strcmp(x,'!++'),{this.tag});
this(flag & [flag(2:end),true]) = [];

% Remove !++ at the end of the file
if ~isempty(this) && strcmp(this(end).tag,'!++')
   this(end) = [];
end

% Replace commas outside any brackets with &'s
for i = 1 : length(this)
   if isempty(this(i).body)
      continue
   end
   this(i).body = ...
      strrepoutbrk(this(i).body,',','&',{'(',')'},{'[',']'},{'{','}'});
end

end
% End of primary function.

%**************************************************************************
% CDF syntax

%{ 
   #NxN
   !++ new figure/table
   !-- new line graph
   !:: new bar graph
   !** void panel
%}
