function x = loadstruct(fname)
% <a href="matlab: edit utils/io/loadstruct">LOADSTRUCT</a>  Load struct-convertible object saved previously by SAVESTRUCT.
%
% Syntax:
%   x = loadstruct(fname)
% Output arguments:
%   x [ model | tseries | VAR ] Object loaded.
% Required input arguments:
%   fname [ char ] File name.

% The IRIS Toolbox 2009/04/09.
% Copyright (c) 2007-2008 Jaromir Benes.

%**************************************************************************
% Function body.

% Load all entries.
% Because keywords are loaded with underscores, fix their names.
list = who('-file',fname);
list = list(:).';
index = cellfun(@iskeyword,list);
if any(~index)
   x = load('-mat',fname,list{~index});
else
   x = struct();
end
if any(index)
   state = warning('query');
   warning('off','MATLAB:load:loadingKeywordVariable');
   temp = load('-mat',fname,list{index});
   temp = struct2cell(temp);
   for i = find(index)
      x.(list{i}) = temp{i};
   end
   warning(state);
else
   x = load('-mat',fname);
end

if isfield(x,'SAVESTRUCT_CLASS')
   thisClass = x.SAVESTRUCT_CLASS;
   x = rmfield(x,'SAVESTRUCT_CLASS');
else
   % For bkw compatibility only:
   % Convert IRIS_MODEL=true to CLASS='MODEL' etc.
   if isfield(x,'IRIS_MODEL')
      thisClass = 'model';
      x = rmfield(x,'IRIS_MODEL');
   elseif isfield(x,'IRIS_VAR')
      thisClass = 'VAR';
      x = rmfield(x,'IRIS_VAR');
   elseif isfield(x,'IRIS_RVAR')
      thisClass = 'VAR';
      x = rmfield(x,'IRIS_RVAR');
   elseif isfield(x,'IRIS_SVAR')
      thisClass = 'VAR';
      x = rmfield(x,'IRIS_SVAR');
   elseif isfield(x,'IRIS_TSERIES') || (isfield(x,'start') && isfield(x,'data') && isfield(x,'comment'))
      thisClass = 'tseries';
      x = rmfield(x,'IRIS_TSERIES');
   elseif isfield(x,'IRIS_CONTAINER')
      thisClass = 'container';
      x = rmfield(x,'IRIS_CONTAINER');
   else
      thisClass = 'struct';
   end
end

switch thisClass
   case 'model'
      x = model(x);
      x = loadobj(x);
      
   case 'VAR'
      x = VAR.loadobj(x);
      
   case 'tseries'
      x = tseries(x);
      
   case 'container'
      x = container.loadobj(x);
      
end
      
end
% End of primary function.
