function ln = zeroline(varargin)
% ZEROLINE  Display zero line if Y-axis limits include zero.

% The IRIS Toolbox.
% Copyright 2007-2009 Jaromir Benes.

if isempty(varargin)
   h = gca();
else
   if isnumeric(varargin{1})
      h = varargin{1};
   else
      h = [varargin{1}{:}];
   end
   varargin(1) = [];
end

%**************************************************************************
% Function body.

if strcmp(get(h,'type'),'figure')
   fg = h;
   h = [];
   for i = 1 : numel(fg)
      tmp = findobj(fg(i),'type','axes','parent',fg(i));
      h = [h,tmp(:)'];
   end
end

ln = nan(size(h));
for i = 1 : numel(h)
   ax = h(i);
   yLim = get(ax,'yLim');
   if yLim(1) < 0 && yLim(2) > 0
      xlim = get(ax,'xlim');
      nextplot = get(ax,'nextPlot');
      set(ax,'nextPlot','add');
      ln(i) = plot(ax,xlim,[0,0],'color',[0,0,0]);
      if ~isempty(varargin)
         set(ln(i),varargin{:});
      end
      % Move zero line to background right below the last line.
      ch = get(ax,'children');
      if length(ch) > 1
         set(ax,'children',ch([2:end,1]));
      end
      set(ax,'nextPlot',nextplot);
   end
end
ln(isnan(ln)) = [];

if ~isempty(ln)
   % Exclude zero lines from legend.
   excludefromlegend(ln);
   % Exclude zero lines from style.
   set(ln,'userData','excludeFromStyle');
end

end
% End of primary function.